
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
 
package org.aspectj.ajde;

import java.util.*;

import org.aspectj.ajde.ui.*;
import org.aspectj.asm.*; 

/** 
 * @author	Mik Kersten
 */
public interface BuildConfigManager {
	
	public static final String CONFIG_FILE_SUFFIX = ".lst";
	
	public static final String DEFAULT_CONFIG_LABEL = "<all project files>";
	
	/**
	 * Returns the currently active build configuration file.  The current active 
	 * build configuration file that is set in this class is used for building and 
	 * for updating the structure model.
	 * 
	 * @return	full path to the file
	 */
	public String getActiveConfigFile();
	
	/**
	 * Sets the currently active build configuration file.
	 * 
	 * @param	full path to the file
	 */
	public void setActiveConfigFile(String currConfigFilePath);

	/**
	 * Add a listner that will be notified of build configuration change events
	 */
    public void addListener(BuildConfigListener configurationListener);

	/**
	 * Remove a configuration listener.
	 */
    public void removeListener(BuildConfigListener configurationListener);
     
   	/**
   	 * Build a model for the corresponding configuration file.
   	 * 
   	 * @param	full path to the file
   	 */  
	public BuildConfigModel buildModel(String configFilePath);
	
	/**
	 * Save the given configuration model to the file that it was generated from.
	 */
	public void writeModel(BuildConfigModel model);
	
	/**
	 * Write a list of source files into a configuration file.  File paths will be
	 * written relative to the path of the configuration file.
	 */ 
	public void writePaths(String configFilePath, List paths);
	
	/**
	 * Add files to a configuration.
	 * 
	 * @param configFilePath	full path to the configuration file
	 * @param files			list of full paths to the files to be added
	 */
	public void addFilesToConfig(String configFilePath, List files);

	/**
	 * Remove files from a configuration.
	 * 
	 * @param configFilePath	full path to the configuration file
	 * @param files			list of full paths to the files to be removed
	 */  
	public void removeFilesFromConfig(String configFilePath, List files);
}


