
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */


package org.aspectj.ajde;

import java.util.EventListener;

/**
 * Compiler listeners get notified of compilation events.
 *
 * @author Mik Kersten
 */
public interface BuildListener extends EventListener {
    
    /**
     * Called when a new build is started.
     * 
     * @param   buildConfigFile configuration file used for the last compile
     */
    public void compileStarted(String buildConfigFile);

    /**
     * Called when a build completes.
     * 
     * @param   buildConfigFile configuration file used for the last compile
     * @param   buildTime   	 compilation time in miliseconds
     * @param   succeeded   	 true if build succeeded without errors
     */
    public void compileFinished(String buildConfigFile, int buildTime, boolean succeeded, boolean warnings);
    
    /**
     * Called when a build is aborted before completion.
     * 
     * @param	buildConfigFile	configuration used for the last compile
     * @param	message			message explaining reason for abort
     */
    public void compileAborted(String buildConfigFile, String message);
}

