
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde;

import org.aspectj.ajde.internal.*;
import org.aspectj.ajde.internal.AjdeCompiler;

/**
 * Responsible for the build process, including compiler invocation, threading, and error
 * reporting.
 *
 * @author Mik Kersten
 */
public interface BuildManager {

	/**
	 * Build the structure model for the default build configuration.  No ".class" files
	 * are generated.
	 */
    public void buildStructure();

	/**
	 * Build the default build configuration.
	 */
    public void build();
    
    /**
     * Build the specified build configuration.
     */
    public void build(String configFile);

	/**
	 * Exit the build immediately, before completion.
	 */ 
    public void abortBuild();

	public BuildOptionsAdapter getBuildOptions();

    public AjdeCompiler getCurrCompiler();

    public boolean isStructureDirty();

    public void setStructureDirty(boolean structureDirty);

    public void addListener(BuildListener compilerListener);

    public void removeListener(BuildListener compilerListener);
}

