
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
 
 
package org.aspectj.ajde;

/**
 * When a particular option is not set its documented default is used.
 */
public interface BuildOptionsAdapter {
	
	/**
	 * Use javac to generate .class files.  The default is "false".
	 */
	public boolean getUseJavacMode();
	
	/**
	 * Only relevant with Use Javac or Preprocess modes.  Specify where to place
	 * intermediate .java files.  The default is "workingdir".
	 */
	public String getWorkingOutputPath();
	
	/**
	 * Generate regular Java code into the Working OutputPath.  Don't try to generate
	 * any .class files.  The default is "false".
	 */
	public boolean getPreprocessMode();
	
	/**
	 * Specify character encoding used by source files.  The default is the current
	 * JVM's default.
	 */
	public String getCharacterEncoding();
	
	/**
	 * Support assertions as defined in JLS-1.4.  The default is "false".
	 */
	public boolean getSourceOnePointFourMode();
	  
	/**
	 * Be extra-lenient in interpreting the Java specification.  The default is "false", 
	 * i.e. "regular" mode.
	 */
	public boolean getLenientSpecMode();
	
	/**
	 * Be extra-strict in interpreting the Java specification.  The default is "false", 
	 * i.e. "regular" mode.
	 */
	public boolean getStrictSpecMode();
	
	/**
	 * Make the use of some features from pre-1.0 versions of AspectJ be warnings to ease
	 * porting of old code.  The default is "false".
	 */
	public boolean getPortingMode();
	
	/**
	 * The non-standard, typically prefaced with -X when used with a command line compiler.
	 * The default is no non-standard options.
	 */
	public String getNonStandardOptions();
}
