
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */


package org.aspectj.ajde;

/**
 * This interface should be implemented by a progress monitor that that presents
 * the user with the current state of the compile and estimated finish.
 *
 * @author  Mik Kersten
 */
public interface BuildProgressMonitor {

	/**
	 * The heading that should be used in the title of the progress monitor. 
	 */ 
	public static final String PROGRESS_HEADING = "AspectJ Build";

    /**
     * Start the progress monitor.
     */
    public void start(String configFile);

	/**
	 * Sets the label describing the current progress phase.
	 */
    public void setProgressText(String text);

    /**
     * Jumps the progress bar to <CODE>newVal</CODE>.
     */
    public void setProgressBarVal(int newVal);

    /**
     * Makes the progress bar by one.
     */
    public void incrementProgressBarVal();

	/**
	 * @param	maxVal	sets the value at which the progress will finish.
	 */
    public void setProgressBarMax(int maxVal);

	/**
	 * @return	the value at which the progress monitoring will finish.
	 */
    public int getProgressBarMax();

    /**
     * Jump the progress bar to the end and finish progress monitoring.
     */
    public void finish();
}
