
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde;

import java.io.IOException;
import java.util.List;
import org.aspectj.asm.*;

/**
 * @author	Mik Kersten
 */
public interface EditorAdapter {

	/**
	 * Seek the editor to a source line in the file specified.
	 */
    public void showSourceLine(String filePath, int lineNumber, boolean highlight);

	/**
	 * Seek the editor to a {@link SourceLocation} and highlight if specified.
	 */
    public void showSourceLine(SourceLocation sourceLocation, boolean highlight);

	/**
	 * Seek the editor to a source line in the current file.
	 */
    public void showSourceLine(int lineNumber, boolean highlight);

	/**
	 * @return	full path to the file currently being edited.
	 */
    public String getCurrFile();

	/**
	 * Save the contents of the current file being edited.
	 */ 
    public void saveContents() throws IOException;

	/**
	 * Paste text into the current caret position of the editor.
	 */ 
    public void pasteToCaretPos(String text);
    
	/**
	 * Implement if inline annotations are supported by the editor.  Make null
	 * implementation if inline annotations are not supported.
	 * 
	 * @param	filePath	path to the file that should get the annotation
	 * @param	lineNumber	line number for the annotation
	 * @param	items		list of relations to be rendered as the annotation 
	 */
	public void showSourcelineAnnotation(String filePath, int lineNumber, List items);

	/**
	 * Implement if multipe editor views are supported by the editor.  Make null
	 * implementation if multiple editor views are not supported.
	 * 
	 * @param	filePath	path to the source file
	 * @param	lineNumber	line number of the sourceline
	 */
	//public void addEditorViewForSourceLine(String filePath, int lineNumber);
}
