
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde;

import org.aspectj.asm.*;

/**
 * Used for displaying tasks, such as compiler messages, to the user.
 *
 * @author Mik Kersten
 */
public interface TaskListManager {

//	public static int ERROR_MESSAGE = 0;
//	
//	public static int WARNING_MESSAGE = 1;
//	
//	public static int INFO_MESSAGE = 2;

	/**
	 * Add a task associated with a particular line in a source file.
	 * 
	 * @param	message			description of compiler message
	 * @param	sourceLocation	can be null
	 * @param	kind			one of <code>ERROR_MESSAGE, WARNING_MESSAGE, INFO_MESSAGE</code>
	 */ 
    public void addSourcelineTask(String message, SourceLocation sourceLocation, StructureMessage.Kind kind);

	/**
	 * Add a task associated with the current project.
	 * 
	 * @param	message			description of compiler message
	 * @param	kind			one of <code>ERROR_MESSAGE, WARNING_MESSAGE, INFO_MESSAGE</code>
	 */     
    public void addProjectTask(String message, StructureMessage.Kind kind);
    
    /**
     * Delete all of the currently active tasks.
     */ 
    public void clearTasks();
}

