
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
 
package org.aspectj.ajde.internal;

import java.io.*;

/**
 * Used for writing converting text written to an output stream into
 * a string.
 * 
 * @author Mik Kersten
 */
public class StreamPrintWriter extends PrintWriter {
    private String contents = "";

    public StreamPrintWriter(Writer out) {
        super(out);
    }

    public String getContents() {
        return contents;
    }

    public void flushBuffer() {
        contents = "";
        super.flush();
    }

    public void print(char x) {
        contents += x + "\n";
    }

    public void print(char[] x) {
        contents += new String( x );
    }

    public void print(int x) {
        contents += x;
    }

    public void print(String x) {
        contents += x;
    }

    public void println(char x) {
        contents += x + "\n";
    }

    public void println(char[] x) {
        contents += new String( x ) + "\n";
    }

    public void println(int x) {
        contents += x + "\n";
    }

    public void println(String x) {
        contents += x + "\n";
    }

    public void write( byte[] x ) {
        contents += new String( x );
    }

    public void write( byte[] x, int i1, int i2 ) {
        StringWriter writer = new StringWriter();
        String s = new String( x );
        writer.write( s.toCharArray(), i1, i2 );
        contents += writer.getBuffer().toString();
    }

    public void write( int c ) {
        contents += c;
    }

    public void write( String s ) {
        contents += s;
    }

    public void write( String s, int i1, int i2 ) {
        contents += s;
    }
}
