
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
 
package org.aspectj.ajde.ui;

import java.util.*;
import org.aspectj.ajde.*;
import org.aspectj.ajde.ui.internal.*;
import org.aspectj.asm.*;
import org.aspectj.asm.associations.*;

/**
 * @author	Mik Kersten
 */
public class StructureSearchManager {

	/**
	 * @param		pattern		case-sensitive substring of node name
	 * 
	 * @return 	null if a corresponding node was not found
	 */
	public List findMatches(
		String pattern, 
		ProgramElementNode.Kind kind) {
		
		List matches = new ArrayList();
		StructureModel model = Ajde.getDefault().getStructureModelManager().getStructureModel();
		if (model.equals(StructureModel.NO_STRUCTURE)) {
			return null;
		} else {
			return findMatchesHelper((ProgramElementNode)model.getRoot(), pattern, kind, matches);
		}
	}					
	
	
	private List findMatchesHelper(
		ProgramElementNode node, 
		String pattern, 
		ProgramElementNode.Kind kind,
		List matches) {
			
		if (node != null && node.getName().indexOf(pattern) != -1) {
			if (kind == null || node.getProgramElementKind().equals(kind)) {
				matches.add(node);	
			} 
		}
		
		for (Iterator it = node.getChildren().iterator(); it.hasNext(); ) {
			StructureNode nextNode = (StructureNode)it.next();
			if (nextNode instanceof ProgramElementNode) {
				findMatchesHelper(
					(ProgramElementNode)nextNode, 
					pattern, 
					kind,
					matches); 		
			}
		}
		 
		return matches;		
	}
}