
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui;

import java.util.*;
import org.aspectj.asm.*;
import org.aspectj.ajde.ui.internal.*;

/**
 * @author Mik Kersten
 */
public abstract class StructureView {

	private StructureViewNode rootNode = null; 
	private StructureViewNode activeNode = null; 
	protected StructureViewProperties viewProperties = null;
	protected StructureViewRenderer renderer = null;
	
	public StructureViewProperties getViewProperties() {
		return viewProperties;	
	}  
	
	public StructureViewNode getRootNode() {
		return rootNode; 
	}

	public void setRootNode(StructureViewNode rootNode) {
		this.rootNode = rootNode;
	}

	public void setViewProperties(StructureViewProperties viewProperties) {
		this.viewProperties = viewProperties;
	}

	public void setRenderer(StructureViewRenderer renderer) {
        this.renderer = renderer;
    }

	protected void notifyViewUpdated() {
		if (renderer != null) renderer.updateView(this);
    }

	/**
	 * @return		the view node corresponding to the active ProgramElementNode or null
	 */
	public StructureViewNode getActiveNode() {
		if (activeNode != null 
			&& activeNode.getStructureNode() instanceof ProgramElementNode) {
			return activeNode;
		} else {
			return null;
		}
	}

	/**
	 * Searches from the root node of the view down in order to find matches.
	 * 
	 * @return		the first match
	 */
	public StructureViewNode findCorrespondingViewNode(ProgramElementNode node) {
		return findCorrespondingViewNodeHelper(rootNode, node);
	}

	private StructureViewNode findCorrespondingViewNodeHelper(StructureViewNode node, ProgramElementNode pNode) {
		
		if (node != null
			&& node.getStructureNode() != null 
			&& node.getStructureNode().equals(pNode)) {
			return node;	
		} 
		
		if (node != null && node.getChildren() != null) {
			for (Iterator it = node.getChildren().iterator(); it.hasNext(); ) {
				StructureViewNode foundNode = findCorrespondingViewNodeHelper((StructureViewNode)it.next(), pNode); 		
				if (foundNode != null) return foundNode;
			}
		}
		
		return null;
	}	

	public void setActiveNode(StructureViewNode activeNode) {
		this.activeNode = activeNode;
		if (renderer != null) renderer.setActiveNode(activeNode);
	}
	
	public void setActiveNode(StructureViewNode activeNode, int sourceLine) {
		this.activeNode = activeNode;
		if (renderer != null) renderer.setActiveNode(activeNode, sourceLine);
	}
}
 
