
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
 
package org.aspectj.ajde.ui;

import org.aspectj.asm.*;
import java.util.List;

/**
 * @author Mik Kersten
 */ 
public abstract class StructureViewNodeFactory {

	private AbstractIconRegistry iconRegistry;	

	public StructureViewNodeFactory(AbstractIconRegistry iconRegistry) {
		this.iconRegistry = iconRegistry;			
	}

	public StructureViewNode createNode(StructureNode node) {
		return createNode(node, null);		
	}

	public StructureViewNode createNode(StructureNode node, List children) {
		AbstractIcon icon;
		if (node instanceof ProgramElementNode) {
			ProgramElementNode pNode = (ProgramElementNode)node;
			icon = iconRegistry.getStructureIcon(pNode.getProgramElementKind(), pNode.getAccessibility());
		} else if (node instanceof RelationNode) {
			RelationNode relationNode = (RelationNode)node;
			icon = iconRegistry.getRelationIcon(relationNode.getRelation());
		} else if (node instanceof LinkNode) {
			LinkNode linkNode = (LinkNode)node;
			icon = iconRegistry.getStructureIcon(
				linkNode.getProgramElementNode().getProgramElementKind(), 
				linkNode.getProgramElementNode().getAccessibility());
		} else {
			icon = new AbstractIcon(null);	
		}
		return createConcreteNode(node, icon, children);
	}
	
	/**
	 * Implementors must override this method in order to create new nodes.
	 */ 
	protected abstract StructureViewNode createConcreteNode(StructureNode node, AbstractIcon icon, List children);
}
