/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui;

import java.util.EventListener;

/**
 * View renderers get notified of structure view update events and should
 * update the display of the structure view accordingly.
 *
 * @author Mik Kersten
 */
public interface StructureViewRenderer extends EventListener {

	/**
	 * Implementors should updated the display of the corresponding
	 * file structure view.
	 */ 
    public void updateView(StructureView structureView);
    
    /**
     * Highlights and selects the given node as active.  What "active"
     * means depends on the renderer: a typical activation should cause
     * the corresponding node's sourceline to be highlighted in the
     * active editor.
     */
    public void setActiveNode(StructureViewNode node);
    
    /**
     * Same behavior as <CODE>setActiveNode(StructureViewNode)</CODE> but
     * highlights a particular line within the span of the node.
     * 
     * @param	lineOffset	number of lines after the begin and before the
     * 						end line of the corresponding <CODE>StructureNode</CODE>.
     */ 
	public void setActiveNode(StructureViewNode activeNode, int lineOffset);
}
