
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */


package org.aspectj.ajde.ui;

import java.util.List;

/**
 * This interface needs to be implemented by an IDE extension in order for AJDE
 * to store properties in a way that matches the IDE's property storing facilities.
 * 
 * @author	Mik Kersten
 */
public interface UserPreferencesAdapter {

	/**
	 * Retrieves a global IDE option.
	 */ 
	public String getGlobalPreference(String name);
	
	/**
	 * Retrieves a global IDE option.
	 */ 
	public List getGlobalMultivalPreference(String name);
	
	/**
	 * Sets a global IDE option with a single value.
	 */ 
	public void setGlobalPreference(String name, String value);
	
	/**
	 * Sets a global IDE option with multiple values.
	 */ 
	public void setGlobalMultivalPreference(String name, List values);
	
	/**
	 * Retrieves an option for the currently active project.
	 */ 
	public String getProjectPreference(String name);
	
	/**
	 * Retrieves an option for the currently active project.
	 */ 
	public List getProjectMultivalPreference(String name);
	
	/**
	 * Sets an option for the currently active project.
	 */ 
	public void setProjectPreference(String name, String value);
	
	/**
	 * Sets an option for the currently active project.
	 */ 
	public void setProjectMultivalPreference(String name, List values);	
}
