
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui.internal;

import java.util.*;
import org.aspectj.asm.*;

/**
 * @author Mik Kersten
 */
public class NavigationHistoryModel {
    
    private ProgramElementNode currNode = null;
    private Stack backHistory = new Stack();
    private Stack forwardHistory = new Stack();
    
    /**
     * @return 	null if the history is empty
     */
	public ProgramElementNode navigateBack() {
		if (backHistory.isEmpty() || currNode == null) return null;
		
		forwardHistory.push(currNode);
		currNode = (ProgramElementNode)backHistory.pop();
		return currNode;
	}

    /**
     * @return 	null if the history is empty
     */ 
	public ProgramElementNode navigateForward() {
		if (forwardHistory.isEmpty() || currNode == null) return null;
		
		backHistory.push(currNode);
		currNode = (ProgramElementNode)forwardHistory.pop();
		return currNode;
	}

    
    public void navigateToNode(ProgramElementNode toNode) {
    	if (currNode != null) backHistory.push(currNode);
    	currNode = toNode; 
    }
}
