
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui.swing;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import org.aspectj.ajde.*;
import org.aspectj.asm.*;
import org.aspectj.asm.views.*;
import org.aspectj.ajde.ui.*;
import javax.swing.border.*;

public class AJButtonMenuCombo extends JPanel {
	
    private JButton mainButton;
    private JButton popupButton;
    private JPopupMenu menu;
    private boolean depressable = false;
    private boolean isPressed = false;
    
	public AJButtonMenuCombo(String name, 
		String toolTipText, 
		Icon icon, 
		JPopupMenu menu,
		boolean depressable) {
			
		this.menu = menu;
		this.depressable = depressable;
		mainButton = new JButton();
		mainButton.setIcon(icon);
        mainButton.setBorder(AjdeWidgetStyles.DEFAULT_BORDER);
        mainButton.setToolTipText(toolTipText);
        mainButton.setPreferredSize(new Dimension(22, 20));
        mainButton.setMinimumSize(new Dimension(22, 20));
        mainButton.setMaximumSize(new Dimension(22, 20));   
        
        popupButton = new JButton();
        popupButton.setIcon(AjdeUIManager.getDefault().getIconRegistry().getPopupIcon());
        popupButton.setBorder(BorderFactory.createEmptyBorder());
        popupButton.setToolTipText(toolTipText);
        popupButton.setPreferredSize(new Dimension(13, 20));
        popupButton.setMinimumSize(new Dimension(13, 20));
        popupButton.setMaximumSize(new Dimension(13, 20));           
    
    	PopupListener popupListener = new PopupListener(mainButton);
    	
    	if (depressable) {
    		mainButton.addActionListener(new ButtonActionListener());
    	} else {
        	mainButton.addMouseListener(popupListener); 
    	}
	
        popupButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                popupButton.setBorder(null);
            }
        });

		BorderUpdateListener borderUpdateListner = new BorderUpdateListener();
		mainButton.addMouseListener(borderUpdateListner);
		popupButton.addMouseListener(borderUpdateListner);
		
		popupButton.addMouseListener(popupListener);
		
		this.setLayout(new BorderLayout());
        this.add(mainButton,  BorderLayout.CENTER);
        this.add(popupButton,  BorderLayout.EAST);
		
        this.setMinimumSize(new Dimension(35, 20));
        this.setMaximumSize(new Dimension(35, 20));      
	}

    class ButtonActionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (isPressed) {
            	mainButton.setBorder(AjdeWidgetStyles.DEFAULT_BORDER);
            	isPressed = false;
            } else {
            	mainButton.setBorder(AjdeWidgetStyles.LOWERED_BEVEL_BORDER);
            	isPressed = true;
            }
        }
    }
	

	class BorderUpdateListener extends MouseAdapter {
        public void mouseEntered(MouseEvent e) {
            popupButton.setBorder(AjdeWidgetStyles.RAISED_BEVEL_BORDER);
            mainButton.setBorder(AjdeWidgetStyles.RAISED_BEVEL_BORDER);
        }
        
      	public void mouseExited(MouseEvent e) {
            popupButton.setBorder(AjdeWidgetStyles.DEFAULT_BORDER);
            if (isPressed) {
            	mainButton.setBorder(AjdeWidgetStyles.LOWERED_BEVEL_BORDER);
            } else {
            	mainButton.setBorder(AjdeWidgetStyles.DEFAULT_BORDER);
            }
        }
	}

	class PopupListener extends MouseAdapter {
		private JButton button;

		public PopupListener(JButton button) {
			this.button = button;
		}

	    public void mousePressed(MouseEvent e) {
	        maybeShowPopup(e);
	    }

	    public void mouseReleased(MouseEvent e) {
	        maybeShowPopup(e);
	    }

	    private void maybeShowPopup(MouseEvent e) {
	    	menu.show(e.getComponent(), button.getX(), button.getY() + popupButton.getHeight());
	    }
	}
	
	public void setEnabled(boolean enabled) {
		mainButton.setEnabled(enabled);
		popupButton.setEnabled(enabled);
	}

	public void setMenu(JPopupMenu menu) {
		this.menu = menu;
		this.repaint();
	}

}