/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui.swing;

import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import org.aspectj.asm.*;
import org.aspectj.asm.associations.*;
import org.aspectj.asm.views.*;
import org.aspectj.ajde.*;
import org.aspectj.ajde.internal.*;
import org.aspectj.ajde.ui.*;

/**
 * Responsible for displaying and controlling the configuration and output of a
 * master and slave structure view.
 *
 * @author Mik Kersten
 */
public class BrowserViewManager {

	private StructureViewPanel browserPanel = null;
    private boolean globalMode = true;
    private boolean splitViewMode = false;
    private IconRegistry icons;
  
    private Stack backHistory = new Stack();
    private Stack forwardHistory = new Stack();
    private ProgramElementNode currNode = null;

	private final GlobalStructureView DECLARATION_VIEW;
	private final GlobalStructureView CROSSCUTTING_VIEW;
	private final GlobalStructureView INHERITANCE_VIEW;

    private final GlobalViewProperties DECLARATION_VIEW_PROPERTIES;
    private final GlobalViewProperties CROSSCUTTING_VIEW_PROPERTIES;
    private final GlobalViewProperties INHERITANCE_VIEW_PROPERTIES;

    public BrowserViewManager() {
		java.util.List views = new ArrayList();
		views.add(DECLARATION_VIEW);
		views.add(CROSSCUTTING_VIEW);
		views.add(INHERITANCE_VIEW);
		browserPanel = new StructureViewPanel(views);
    }

    public StructureViewPanel getBrowserPanel() {
        return browserPanel;
    }

    public void showSourcesNodes(java.util.List nodes) {
        for (Iterator it = nodes.iterator(); it.hasNext(); ) {
            ProgramElementNode currNode = null;
            StructureNode structureNode = (StructureNode)it.next();
            if (structureNode instanceof LinkNode) {
                currNode = ((LinkNode)structureNode).getProgramElementNode();
            } else {
                currNode = (ProgramElementNode)structureNode;
            }
            Ajde.getDefault().getEditorManager().addViewForSourceLine(currNode.getSourceLocation().getSourceFilePath(),
                currNode.getSourceLocation().getLineNumber());
        }
    }

    public void extractAndInsertSignatures(java.util.List signatures, boolean calls) {
        PointcutWizard pointcutWizard = new PointcutWizard(signatures);
        pointcutWizard.setVisible(true);
        pointcutWizard.setLocation(AjdeUIManager.getDefault().getRootFrame().getX()+100, AjdeUIManager.getDefault().getRootFrame().getY()+100);
    }
    
    {   
    	DECLARATION_VIEW_PROPERTIES = new GlobalViewProperties(StructureViewProperties.Hierarchy.DECLARATION);
   		CROSSCUTTING_VIEW_PROPERTIES = new GlobalViewProperties(StructureViewProperties.Hierarchy.CROSSCUTTING);
    	INHERITANCE_VIEW_PROPERTIES = new GlobalViewProperties(StructureViewProperties.Hierarchy.INHERITANCE);  
    
        CROSSCUTTING_VIEW_PROPERTIES.addRelation(Advice.METHOD_RELATION);
        CROSSCUTTING_VIEW_PROPERTIES.addRelation(Advice.METHOD_CALL_SITE_RELATION);
        CROSSCUTTING_VIEW_PROPERTIES.addRelation(Advice.CONSTRUCTOR_RELATION);
        CROSSCUTTING_VIEW_PROPERTIES.addRelation(Advice.CONSTRUCTOR_CALL_SITE_RELATION);
        CROSSCUTTING_VIEW_PROPERTIES.addRelation(Advice.HANDLER_RELATION);
        CROSSCUTTING_VIEW_PROPERTIES.addRelation(Advice.INITIALIZER_RELATION);
        CROSSCUTTING_VIEW_PROPERTIES.addRelation(Advice.FIELD_ACCESS_RELATION);

        INHERITANCE_VIEW_PROPERTIES.addRelation(Inheritance.IMPLEMENTS_RELATION);
        INHERITANCE_VIEW_PROPERTIES.addRelation(Inheritance.INHERITS_MEMBERS_RELATION);
        INHERITANCE_VIEW_PROPERTIES.addRelation(Inheritance.INHERITS_RELATION);
     	
     	DECLARATION_VIEW_PROPERTIES.setRelations(Ajde.getDefault().getStructureViewManager().getAvailableRelations());

        CROSSCUTTING_VIEW = Ajde.getDefault().getStructureViewManager().createGlobalView(CROSSCUTTING_VIEW_PROPERTIES);
        INHERITANCE_VIEW = Ajde.getDefault().getStructureViewManager().createGlobalView(INHERITANCE_VIEW_PROPERTIES);
    	DECLARATION_VIEW = Ajde.getDefault().getStructureViewManager().createGlobalView(DECLARATION_VIEW_PROPERTIES);
    }
}


