
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui.swing;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import org.aspectj.ajde.BuildProgressMonitor;
import org.aspectj.ajde.Ajde;

/**
 * This dialog box is open while ajc is compiling the system and displays
 * a corresponding progress bar.
 *
 * @author  Mik Kersten
 */
public class DefaultBuildProgressMonitor extends Thread implements BuildProgressMonitor {

	private BuildProgressPanel progressDialog = null;
	private JDialog dialog = null;

	public DefaultBuildProgressMonitor(Frame parent) {
        dialog = new JDialog(parent, PROGRESS_HEADING, false);
        progressDialog = new BuildProgressPanel();
        dialog.setContentPane(progressDialog);
        dialog.setSize(550, 120);
        dialog.setLocationRelativeTo(parent);	
	}

    /**
     * Start the progress monitor.
     */
    public void start(String configFilePath) {
    	progressDialog.setConfigFile(configFilePath);
		dialog.setLocationRelativeTo(AjdeUIManager.getDefault().getRootFrame());
		dialog.setVisible(true);
    }

	/**
	 * Sets the label describing the current progress phase.
	 */
    public void setProgressText(String text) {
    	progressDialog.setProgressText(text);
    }

    /**
     * Jumps the progress bar to <CODE>newVal</CODE>.
     */
    public void setProgressBarVal(int newVal) {
    	progressDialog.setProgressBarVal(newVal);
    }

    /**
     * Makes the progress bar by one.
     */
    public void incrementProgressBarVal() {
    	progressDialog.incrementProgressBarVal();
    }

	/**
	 * @param	maxVal	sets the value at which the progress will finish.
	 */
    public void setProgressBarMax(int maxVal) {
    	progressDialog.setProgressBarMax(maxVal);
    }

	/**
	 * @return	the value at which the progress monitoring will finish.
	 */
    public int getProgressBarMax() {
		return progressDialog.getProgressBarMax();    	
    }

    /**
     * Jump the progress bar to the end and finish progress monitoring.
     */
    public void finish() {
		progressDialog.finish();
		dialog.dispose();    	
    }
}