
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui.swing;

import javax.swing.*;
import java.util.*;
import java.net.URL;
import org.aspectj.asm.*;
import org.aspectj.ajde.ui.*;

/**
 * Default icons.  Override behavior for custom icons.
 *
 * @author  Mik Kersten
 */
public class IconRegistry extends AbstractIconRegistry {

    //public static IconRegistry INSTANCE = null;
    protected String RESOURCE_PATH = "org/aspectj/ajde/resources/";
    
    private final Icon START_AJDE = makeIcon("actions/startAjde.gif");
    private final Icon STOP_AJDE = makeIcon("actions/stopAjde.gif");
    private final Icon BUILD = makeIcon("actions/build.gif");
    private final Icon DEBUG = makeIcon("actions/debug.gif");
    private final Icon EXECUTE = makeIcon("actions/execute.gif");
    private final Icon AJBROWSER = makeIcon("structure/advice.gif");
	private final Icon AJBROWSER_ENABLED = makeIcon("actions/browserEnabled.gif");
	private final Icon AJBROWSER_DISABLED = makeIcon("actions/browserDisabled.gif");   
	private final Icon STRUCTURE_VIEW = makeIcon("actions/structureView.gif"); 

	private final Icon HIDE_ASSOCIATIONS = makeIcon("actions/hideAssociations.gif"); 
	private final Icon HIDE_NON_AJ = makeIcon("actions/hideNonAJ.gif"); 
	private final Icon GRANULARITY = makeIcon("actions/granularity.gif"); 
	private final Icon AJDE_SMALL = makeIcon("actions/ajdeSmall.gif"); 

	private final Icon ERROR = makeIcon("structure/error.gif");
	private final Icon WARNING = makeIcon("structure/warning.gif");
	private final Icon INFO = makeIcon("structure/info.gif");  

	private final Icon POPUP = makeIcon("actions/popup.gif");
	private final Icon FILTER = makeIcon("actions/filter.gif");
	private final Icon RELATIONS = makeIcon("actions/relations.gif");
	private final Icon ORDER = makeIcon("actions/order.gif");
	
    private final Icon ZOOM_STRUCTURE_TO_FILE_MODE = makeIcon("actions/zoomStructureToFileMode.gif");
    private final Icon ZOOM_STRUCTURE_TO_GLOBAL_MODE = makeIcon("actions/zoomStructureToGlobalMode.gif");
    private final Icon SPLIT_STRUCTURE_VIEW = makeIcon("actions/splitStructureView.gif");
    private final Icon MERGE_STRUCTURE_VIEW = makeIcon("actions/mergeStructureView.gif");
	 
    private final Icon BACK = makeIcon("actions/back.gif");
    private final Icon FORWARD = makeIcon("actions/forward.gif");
    private final Icon SEARCH = makeIcon("actions/search.gif");
    private final Icon OPEN_CONFIG = makeIcon("actions/openConfig.gif");
    private final Icon CLOSE_CONFIG = makeIcon("actions/closeConfig.gif");
    private final Icon SAVE = makeIcon("actions/save.gif");
    private final Icon SAVE_ALL = makeIcon("actions/saveAll.gif");
    private final Icon BROWSER_OPTIONS = makeIcon("actions/browseroptions.gif");
   
    private final Icon ACCESSIBILITY_PUBLIC = makeIcon("structure/accessibility-public.gif");
    private final Icon ACCESSIBILITY_PACKAGE = makeIcon("structure/accessibility-package.gif");
    private final Icon ACCESSIBILITY_PROTECTED = makeIcon("structure/accessibility-protected.gif");
    private final Icon ACCESSIBILITY_PRIVATE = makeIcon("structure/accessibility-private.gif");
    private final Icon ACCESSIBILITY_PRIVILEGED = makeIcon("structure/accessibility-privileged.gif");

	public Icon getAjdeSmallIcon() { return AJDE_SMALL; }
    public Icon getHideAssociationsIcon() { return HIDE_ASSOCIATIONS; }
    public Icon getHideNonAJIcon() { return HIDE_NON_AJ; }
    public Icon getGranularityIcon() { return GRANULARITY; }
    public Icon getErrorIcon() { return ERROR; }
    public Icon getWarningIcon() { return WARNING; }
    public Icon getInfoIcon() { return INFO; }
    public Icon getAJBrowserIcon() { return AJBROWSER; }
    public Icon getAJBrowserEnabledIcon() { return AJBROWSER_ENABLED; }
    public Icon getAJBrowserDisabledIcon() { return AJBROWSER_DISABLED; }
	public Icon getPopupIcon() { return POPUP; }
	public Icon getFilterIcon() { return FILTER; }
	public Icon getOrderIcon() { return ORDER; }
	public Icon getRelationsIcon() { return RELATIONS; }
    public Icon getStartAjdeIcon() { return START_AJDE; }
    public Icon getStopAjdeIcon() { return STOP_AJDE; }
    public Icon getBackIcon() { return BACK; }
    public Icon getForwardIcon() { return FORWARD; }
    public Icon getSearchIcon() { return SEARCH; }
    public Icon getBuildIcon() { return BUILD; }
    public Icon getDebugIcon() { return DEBUG; }
    public Icon getExecuteIcon() { return EXECUTE; }
    public Icon getOpenConfigIcon() { return OPEN_CONFIG; }
    public Icon getCloseConfigIcon() { return CLOSE_CONFIG; }
    public Icon getOpenIcon() { return OPEN_CONFIG; }
    public Icon getSaveIcon() { return SAVE; }
    public Icon getSaveAllIcon() { return SAVE_ALL; }
    public Icon getBrowserOptionsIcon() { return BROWSER_OPTIONS; }
    public Icon getZoomStructureToFileModeIcon() { return ZOOM_STRUCTURE_TO_FILE_MODE; }
    public Icon getZoomStructureToGlobalModeIcon() { return ZOOM_STRUCTURE_TO_GLOBAL_MODE; }
    public Icon getSplitStructureViewIcon() { return SPLIT_STRUCTURE_VIEW; }
    public Icon getMergeStructureViewIcon() { return MERGE_STRUCTURE_VIEW; }
	public Icon getStructureViewIcon() { return STRUCTURE_VIEW; }

    public Icon getAssociationSwingIcon(Relation relation) {
		return convertToSwingIcon(getRelationIcon(relation));	
    }
    
	protected AbstractIcon getStructureIcon(ProgramElementNode.Kind kind, ProgramElementNode.Accessibility accessibility) {
		return getStructureIcon(kind);	
	}
    
    public Icon getStructureSwingIcon(ProgramElementNode.Kind kind, ProgramElementNode.Accessibility accessibility) { 
		return convertToSwingIcon(getStructureIcon(kind, accessibility));    	
    }
	
	public Icon getStructureSwingIcon(ProgramElementNode.Kind kind) {
		return convertToSwingIcon(getStructureIcon(kind));
	}

	public Icon getAccessibilitySwingIcon(ProgramElementNode.Accessibility accessibility) {
		if (accessibility == ProgramElementNode.Accessibility.PUBLIC) {
			return ACCESSIBILITY_PUBLIC;
		} else if (accessibility == ProgramElementNode.Accessibility.PACKAGE) {
			return ACCESSIBILITY_PACKAGE;
		} else if (accessibility == ProgramElementNode.Accessibility.PROTECTED) {
			return ACCESSIBILITY_PROTECTED;
		} else if (accessibility == ProgramElementNode.Accessibility.PRIVATE) {
			return ACCESSIBILITY_PRIVATE;
		} else if (accessibility == ProgramElementNode.Accessibility.PRIVILEGED) {
			return ACCESSIBILITY_PRIVILEGED;
		} else {
			return null;
		}
	}

	public Icon convertToSwingIcon(AbstractIcon iconAdapter) {
		if (iconAdapter != null) {
			return (Icon)iconAdapter.getIconResource();
		} else {
			return null;
		}		
	}

	protected AbstractIcon createIcon(String path) {
		return new AbstractIcon(new ImageIcon(ClassLoader.getSystemResource(path)));		
	}

    protected Icon makeIcon(String iconPath) {
        return new ImageIcon(ClassLoader.getSystemResource(RESOURCE_PATH + iconPath));
    }
}