/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui.swing;

import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import org.aspectj.asm.*;
import org.aspectj.asm.associations.*;
import org.aspectj.asm.views.*;
import org.aspectj.ajde.*;
import org.aspectj.ajde.internal.*;
import org.aspectj.ajde.ui.*;

/**
 * @author Mik Kersten
 */
class PointcutWizard extends JFrame {
    private BrowserViewPanel typeTreeView = null;
    private java.util.List signatures = null;

    JPanel jPanel1 = new JPanel();
    JPanel jPanel2 = new JPanel();
    JPanel jPanel4 = new JPanel();
    JLabel jLabel1 = new JLabel();
    BorderLayout borderLayout1 = new BorderLayout();
    BorderLayout borderLayout2 = new BorderLayout();
    BorderLayout borderLayout3 = new BorderLayout();
    JLabel jLabel4 = new JLabel();
    JPanel jPanel3 = new JPanel();
    JCheckBox jCheckBox5 = new JCheckBox();
    JCheckBox jCheckBox4 = new JCheckBox();
    JCheckBox jCheckBox3 = new JCheckBox();
    JCheckBox jCheckBox2 = new JCheckBox();
    JCheckBox jCheckBox1 = new JCheckBox();
    JButton cancel_button = new JButton();
    JButton ok_button = new JButton();
    JPanel jPanel5 = new JPanel();

    public PointcutWizard(java.util.List signatures) {
        this.signatures = signatures;
        ArrayList views = new ArrayList();
        views.add(StructureViewProperties.Hierarchy.INHERITANCE);
        typeTreeView = new BrowserViewPanel(AjdeUIManager.getDefault().getIconRegistry(), views, StructureViewProperties.Hierarchy.INHERITANCE);
        typeTreeView.updateTree(Ajde.getDefault().getEditorManager().getCurrFile());
        try {
            jbInit();
        }
        catch(Exception e) {
            Ajde.getDefault().getErrorHandler().handleError("Could not initialize GUI.", e);
        }
        this.setSize(400, 400);
        this.setIconImage(((ImageIcon)AjdeUIManager.getDefault().getIconRegistry().getStructureSwingIcon(ProgramElementNode.Kind.POINTCUT)).getImage());
    }

    private Map getViewProperties() {
        Map views = new HashMap();
        GlobalViewProperties INHERITANCE_VIEW = new GlobalViewProperties(StructureViewProperties.Hierarchy.INHERITANCE);
        INHERITANCE_VIEW.addRelation(Inheritance.INHERITS_RELATION);
        views.put(INHERITANCE_VIEW.toString(), INHERITANCE_VIEW);
        return views;
    }

    private void jbInit() throws Exception {
        jLabel1.setFont(new java.awt.Font("Dialog", 0, 11));
        jLabel1.setText("Generate pointcut designator for corresponding joinpoints:");
        jPanel1.setLayout(borderLayout1);
        jPanel4.setLayout(borderLayout2);
        jPanel2.setLayout(borderLayout3);
        jLabel4.setText("Select the target type that will host the generated pointcut:");
        jLabel4.setFont(new java.awt.Font("Dialog", 0, 11));
        jLabel4.setToolTipText("");
        jPanel3.setMaximumSize(new Dimension(32767, 34));
        jCheckBox5.setEnabled(false);
        jCheckBox5.setFont(new java.awt.Font("Dialog", 0, 11));
        jCheckBox5.setSelected(true);
        jCheckBox5.setText("call");
        jCheckBox4.setEnabled(false);
        jCheckBox4.setFont(new java.awt.Font("Dialog", 0, 11));
        jCheckBox4.setText("execution");
        jCheckBox3.setEnabled(false);
        jCheckBox3.setFont(new java.awt.Font("Dialog", 0, 11));
        jCheckBox3.setText("initialization");
        jCheckBox2.setEnabled(false);
        jCheckBox2.setFont(new java.awt.Font("Dialog", 0, 11));
        jCheckBox2.setText("static initialization");
        jCheckBox1.setEnabled(false);
        jCheckBox1.setFont(new java.awt.Font("Dialog", 0, 11));
        jCheckBox1.setText("field get/set");
        cancel_button.setFont(new java.awt.Font("Dialog", 0, 11));
        cancel_button.setText("Cancel");
        cancel_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancel_button_actionPerformed(e);
            }
        });
        ok_button.setText("OK");
        ok_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ok_button_actionPerformed(e);
            }
        });
        ok_button.setFont(new java.awt.Font("Dialog", 0, 11));
        this.setTitle("Pointcut Wizard");
        this.getContentPane().add(jPanel1, BorderLayout.CENTER);
        jPanel1.add(jPanel4, BorderLayout.NORTH);
        jPanel4.add(jLabel1,  BorderLayout.NORTH);
        jPanel4.add(jPanel3, BorderLayout.CENTER);
        jPanel3.add(jCheckBox5, null);
        jPanel3.add(jCheckBox4, null);
        jPanel3.add(jCheckBox3, null);
        jPanel3.add(jCheckBox2, null);
        jPanel3.add(jCheckBox1, null);
        jPanel1.add(jPanel2,  BorderLayout.CENTER);
        jPanel2.add(jLabel4, BorderLayout.NORTH);
        jPanel2.add(typeTreeView,  BorderLayout.CENTER);
        jPanel1.add(jPanel5,  BorderLayout.SOUTH);
        jPanel5.add(ok_button, null);
        jPanel5.add(cancel_button, null);
    }

    private void ok_button_actionPerformed(ActionEvent e) {
        Ajde.getDefault().getEditorManager().pasteToCaretPos(generatePcd());
        this.dispose();
    }

    private void cancel_button_actionPerformed(ActionEvent e) {
        this.dispose();
    }

    private String generatePcd() {
        String pcd = "\n\n" +
            "    pointcut temp(): \n";
        for (Iterator it = signatures.iterator(); it.hasNext(); ) {
            pcd += "        call(* " + it.next() + ")";
            if (it.hasNext()) {
                pcd += " ||";
            } else {
                pcd += ";";
            }
            pcd += "\n";
        }
        return pcd;
    }
}

