
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui.swing;

import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import org.aspectj.asm.*;
import org.aspectj.ajde.ui.*;

/**
 * @author Mik Kersten
 */
public class SwingTreeViewNode extends DefaultMutableTreeNode implements StructureViewNode {

	private StructureNode structureNode;
	private AbstractIcon icon;
	
	public SwingTreeViewNode(StructureNode structureNode, AbstractIcon icon, List children) {
		super(structureNode, true);
		this.structureNode = structureNode;
		this.icon = icon;
		
		if (children != null) {
			for (Iterator it = children.iterator(); it.hasNext(); ) { 
				super.add((SwingTreeViewNode)it.next());	
			}
		}
	}
	
	public StructureNode getStructureNode() {
		return structureNode;	
	}
	
	public AbstractIcon getIcon() {
		return icon;
	}	

	public void add(StructureViewNode child) { 
		super.add((DefaultMutableTreeNode)child);
	}
	
	public void remove(StructureViewNode child) { 
		super.remove((DefaultMutableTreeNode)child);
	}
	
	public List getChildren() {
		if (children == null) {
			return new ArrayList();
		} else {
			return children;
		}	
	}
}

