
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.asm;

import java.io.*;

/**
 * @author Mik Kersten
 */
public class SourceLocation implements Serializable {
	
	private String sourceFilePath = null;
	private String sourceFileName = null;
	private int lineNumber = -1;
	private int endLineNumber = -1;
	private int columnNumber = 0;

	/**
	 * @param		sourceFilePath	modified to '/' delimited path for cross-platform consistency
	 */
	public SourceLocation(String sourceFilePath, 
						   int lineNumber, 
						   int columnNumber) {
		this(sourceFilePath, lineNumber, lineNumber, columnNumber);
	}
	
	/**
	 * @param		sourceFilePath	modified to '/' delimited path for cross-platform consistency
	 */
	public SourceLocation(String sourceFilePath, 
						   int beginLineNumber, 
						   int endLineNumber, 
						   int beginColumnNumber) {
		if (sourceFilePath != null) this.sourceFilePath = sourceFilePath.replace('\\', '/');
		this.lineNumber = beginLineNumber;
		this.endLineNumber = endLineNumber;
		this.columnNumber = beginColumnNumber;
		if (sourceFilePath != null) this.sourceFileName = new File(sourceFilePath).getName();
	}

	public int getColumnNumber() {
		return columnNumber;
	}

	public int getLineNumber() {
		return lineNumber;
	}

	public int getEndLineNumber() {
		return endLineNumber;
	}

	public String getSourceFilePath() {
		return sourceFilePath;
	}
	
	public String getSourceFileName() {
		return sourceFileName;
	}
	
	public String toString() {
		return sourceFilePath + ", begin line: " + lineNumber + ", end line:" + endLineNumber;	
	}
}
