
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.asm.associations;

import java.util.*;
import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.cst.*;
import org.aspectj.compiler.crosscuts.*;
import org.aspectj.compiler.crosscuts.ast.*;
import org.aspectj.asm.*;
import org.aspectj.asm.internal.*;

/**
 * @author Mik Kersten
 */
public class Introduction implements Association {
    public static final String NAME = "Introduction";
    public static final Relation INTRODUCES_RELATION = new Relation("declares member on", "inter-type declared members", NAME, true, false);
    private List relations = new ArrayList();

    public Introduction() {
        relations.add(INTRODUCES_RELATION);
    }

    public List getRelations() {
        return relations;
    }

    public List getRelationNodes(ASTObject astObject) {
        List relations = new ArrayList();
        List introduces = new ArrayList();
        Set forwardCorrs = StructureModelManager.correspondences.getAffects(astObject);
        Set backCorrs = StructureModelManager.correspondences.getAffectedBy(astObject);
        if (astObject instanceof IntroducedDec) {
            for (Iterator it = forwardCorrs.iterator(); it.hasNext(); ) {
                ASTObject node = (ASTObject)it.next();
                LinkNode link = StructureNodeFactory.makeLink(node, false);
                if (node instanceof TypeDec) {
                    introduces.add(link);
                }
            }
            if (!introduces.isEmpty()) relations.add(new RelationNode(INTRODUCES_RELATION, INTRODUCES_RELATION.getForwardNavigationName(), introduces));
        } else {
            for (Iterator it = backCorrs.iterator(); it.hasNext(); ) {
                ASTObject node = (ASTObject)it.next();
                if (astObject instanceof TypeDec) {
                    if (node instanceof IntroducedDec) {
                        introduces.add(StructureNodeFactory.makeLink(node, false));
                    }
                }
            }
            if (!introduces.isEmpty()) relations.add(new RelationNode(INTRODUCES_RELATION, INTRODUCES_RELATION.getBackNavigationName(), introduces));
        }
        return relations;
    }

    public String getName() {
        return NAME;
    }
}
