/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base;

import org.aspectj.compiler.base.ast.*;


import java.util.*;

/*
 * XXXThis completely ignores all issues of return stmts
*/
public class ExprMaker extends MovingWalker {
    public ExprMaker(JavaCompiler compiler) {
        super(compiler);
    }
    
    boolean mayReturn;
    boolean mustReturn;
    
    public Map getInParams() { return inParams; }
    public Map getOutParams() { return outParams; }
    public boolean getNeedsThis() { return needsThis; }
    
    Map inParams = new HashMap();
    Map outParams = new HashMap();
    boolean needsThis = false;
    
    public Expr moveThisExpr(ThisExpr thisExpr, Type thisType) {
        needsThis = true;
        return super.moveThisExpr(thisExpr, thisType);
    }
    
    VarDec lookupVarDec(VarDec oldDec) {
        VarDec ret = (VarDec)remapNodes.get(oldDec);
        if (ret != null) {
            return ret;
        } else {
            FormalDec formal = getAST().makeFormal(oldDec.getType(), oldDec.getId()); //+"XXX");
            if (oldDec.isFinal()) formal.getModifiers().setFinal(true);
            inParams.put(oldDec, formal);
            remapNodes.put(oldDec, formal);
            return formal;
        }
    }

        
    public Expr moveVarExpr(VarExpr var) {
        var.setVarDec(lookupVarDec(var.getVarDec()));
        return var;
    }            
    
    public Expr makeExpr(ASTObject object, boolean allowVoid) {
        final AST ast = getAST();
        CodeDec inCode = object.getEnclosingCodeDec();
        
        BlockStmt block = ast.makeBlock(this.process(object));
        
        Formals formals = ast.makeFormals();
        Exprs args = ast.makeExprs();
        
        // loop over inParams to create call args and formals together
        for (Iterator i = inParams.entrySet().iterator(); i.hasNext(); ) {
            Map.Entry entry = (Map.Entry)i.next();
            VarDec varDec = (VarDec)entry.getKey();
            varDec.getModifiers().setFinal(true);  //??? assumes too much
            args.add(ast.makeVar(varDec));
            
            FormalDec formalDec = (FormalDec)entry.getValue();
            formals.add(formalDec);
        }
        
        Modifiers mods = ast.makeModifiers(Modifiers.PRIVATE);
        mods.setStatic(inCode.isStatic());
        
        Type returnType = inCode.getResultType();
        
        String id = getAST().makeGeneratedName(inCode.getId() + "$ajcPostAround" + 
                      inCode.getBytecodeTypeDec().getBody().size());
        
        MethodDec newMethod = ast.makeMethod(mods, returnType, id, formals, block);
        //newMethod.setSynthetic();
        inCode.getBytecodeTypeDec().getBody().add(newMethod);
        
        Expr thisExpr = ast.makePrimary(newMethod, inCode.getBytecodeTypeDec());
        //if (!newMethod.isStatic()) thisExpr = ast.makeThis();
        
        Expr ret = ast.makeCall(newMethod, thisExpr, args);
        
        if (!allowVoid && newMethod.getResultType().isVoid()) {
            MethodDec voidWrapper = (MethodDec)CopyWalker.copy(newMethod);
            voidWrapper.setId(newMethod.getId() + "$ajcVoidWrapper");
            //voidWrapper.setSynthetic();
            voidWrapper.setResultTypeD( getTypeManager().getObjectType().makeTypeD() );
            newMethod.getBytecodeTypeDec().getBody().add(voidWrapper);
            Expr internalCall = ast.makeCall(
                newMethod,
                ast.makePrimary(newMethod, inCode.getBytecodeTypeDec()),
                voidWrapper.getFormals().makeExprs());
                
            Stmts stmts = ast.makeStmts(
                ast.makeStmt(internalCall), ast.makeReturn(ast.makeNull()));
            voidWrapper.getBody().setStmts(stmts);
            thisExpr = ast.makePrimary(voidWrapper, newMethod.getBytecodeTypeDec());         
            ret = ast.makeCall(voidWrapper, thisExpr, args);
        }
        
        return ret;
    }
}

