/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base;
import org.aspectj.compiler.base.ast.*;

//??? what package does this context really belong in???
import org.aspectj.compiler.crosscuts.ast.*;
import org.aspectj.compiler.crosscuts.*;

import java.util.*;
import java.io.*;

// Why is this a whole pass?  We're walking into _expressions_, for
// heaven's sake!  Well, sadly enough, those expressions could include
// anonymous inner class creation expressions, which have
// initializers, etc, etc.  And this pass is a _precursor_ for
// desugaring inner classes.  So we do a whole walk.

// This walk could be merged with almost any other code walk.

public class InitializerCollector extends WalkerPass {
    private boolean collectSynthetics;

    public InitializerCollector(JavaCompiler compiler, boolean collectSynthetics) {
        super(compiler);
        this.collectSynthetics = collectSynthetics;
    }

    public String getDisplayName() {
        return "collecting initializers";
    }

    public ASTObject process(ASTObject node) {
        node.walk(this);
        if (!(node instanceof TypeDec)) return node;

        TypeDec tyDec = (TypeDec)node;
        tyDec.addAssertionField();
        tyDec.collectInitializers(collectSynthetics);
        return node;
    }
}

