/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base;

import org.aspectj.compiler.base.ast.ASTObject;

import java.io.PrintWriter;

public class InternalCompilerError extends Error {
    public Throwable uncaughtThrowable;
    public ASTObject where;
    private JavaCompiler compiler;
    private String currentSection;

    public InternalCompilerError(JavaCompiler compiler,
                                 Throwable uncaughtThrowable, ASTObject where) {
        this.compiler = compiler;
        this.uncaughtThrowable = uncaughtThrowable;
        if (where == null && compiler != null) {
            where = compiler.getCurrentNode();
        }
        this.where = where;
        if (compiler != null) {
            currentSection = compiler.getCurrentSection();
        }
    }

    public void showWhere(PrintWriter errorWriter) {
        //System.err.println("showing: "+currentSection+", "+where);
        if (currentSection != null) {
            errorWriter.println("The error occured in "+currentSection+".");
            errorWriter.println();
        }

        if (where != null) {
            errorWriter.println("At the time of the error, the compiler was processing ("
                                +where.getClass().getName()+"):");
            ErrorHandler errorHandler = new ErrorHandler(errorWriter);
            errorHandler.setCompiler(compiler);
            errorHandler.showMessage(where, "", true,true);
            errorWriter.println();
            //errorWriter.println(specificCodeRequest);
        } else {
            //errorWriter.println(generalCodeRequest);
        }
        errorWriter.flush();
    }
}
