/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base;
import org.aspectj.compiler.base.ast.*;

import java.util.*;
import java.io.*;

import org.aspectj.tools.ide.SourceLine;
import org.aspectj.util.CollectionUtil;

public class LineMap {
    private File outputFile;
    private String outputDirectory;
    private String outputFileName;

    private int outputLine = 0;

    private ASTObject sourceNode = null;
    //private int sourceLine = -1;
    //private CompilationUnit sourceCompilationUnit;

    private Map outputToSource;
    private Map sourceToOutput;

    public LineMap(File outputFile, Map outputToSource, Map sourceToOutput) throws IOException {
        this.outputFile = outputFile;
        outputFileName = outputFile.getCanonicalPath();
        outputDirectory = new File(outputFileName).getParent();
        this.outputToSource = outputToSource;
        this.sourceToOutput = sourceToOutput;
    }


    final String getOutputFile() {
        return outputFileName;
    }

    public final boolean hasCurrentSourceLine() {
        return sourceNode != null;
    }

    public final int getCurrentSourceLine() {
        return sourceNode != null ? sourceNode.getBeginLine() : -1;
    }

    public final String getCurrentSourceFile() {
        return sourceNode != null ? sourceNode.getSourceFileName() : null;
    }

    public final void addLineMappings(SourceLine sourceLine, SourceLine outputLine) {
        Map map = CollectionUtil.getMapInMap(outputToSource, outputDirectory);
        synchronized (map) {
            map.put(outputLine, sourceLine);
        }

        map = CollectionUtil.getMapInMap(sourceToOutput, sourceNode.getSourceDirectoryName()); //sourceLine.getDirectory());
        synchronized (map) {
            map.put(sourceLine, outputLine);
        }
    }


    public final void noteNewLine() {
        if (sourceNode != null) {
            addLineMappings(new SourceLine(sourceNode.getSourceFileName(), sourceNode.getBeginLine()),
                            new SourceLine(getOutputFile(), outputLine));
        }

        outputLine += 1;
        sourceNode = null;
    }

    public final void noteNode(ASTObject node) {
        //if (true) return;

        // This test shouldn't be needed, but it hides some untidiness
        // that occurs in the parser (where a few nodes don't get context info)
        if (node.getStartPosition() != -1) {
            sourceNode = node;
            //sourceCompilationUnit = node.getCompilationUnit();
            //sourceLine = node.getBeginLine();
        }
    }
}
