/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.FlowCheckerPass;
import org.aspectj.compiler.base.ByteCodeCleanupPass;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;

import java.util.*;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
 * @grammar rand1 && rand2
 */
public class AndAndOpExpr extends ConditionalOpExpr {

    protected LiteralExpr halfFold(Type type, LiteralExpr lit1, LiteralExpr lit2) {
        return type.foldBitwiseOp("&", lit1, lit2);
    }

    // ------------------------------
    // INTRO from FlowCheckerPass

    public void walkFlow(FlowCheckerPass w) {
        // w.setArgs(w.getArgs())
        w.processBoolean(getRand1());
        FlowCheckerPass.Vars p1 = w.getVars();

        w.setVars(p1.getTrue());
        w.processBoolean(getRand2());
        FlowCheckerPass.Vars p2 = w.getVars();

        w.setVars(p2.getTrue(), p1.getFalse().join(p2.getFalse()));
    }

    // ------------------------------
    // INTRO from ByteCodeCleanupPass

    public ASTObject postCleanup(ByteCodeCleanupPass w) {
        if (getRand1().isConstantTrue()) {
            return getRand2();
        } else if (getRand1().isConstantFalse() || getRand2().isConstantTrue()) {
            return getRand1();
        } else {
            return this;
        }
    }

    // ------------------------------
    // bcg
    protected void cgTest(CodeBuilder cb, Label tdest, Label fdest) {
        Label mid = cb.genLabel();
        getRand1().cgTest(cb, mid, fdest);
        cb.emitLabel(mid);
        getRand2().cgTest(cb, tdest, fdest);
    }

    //BEGIN: Generated from @child and @property

    public AndAndOpExpr(SourceLocation location, Expr _rand1, String _op, Expr _rand2) {
        super(location, _rand1, _op, _rand2);

    }
    protected AndAndOpExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        AndAndOpExpr ret = new AndAndOpExpr(getSourceLocation());
        ret.preCopy(walker, this);
        if (rand1 != null) ret.setRand1( (Expr)walker.process(rand1) );
        ret.op = op;
        if (rand2 != null) ret.setRand2( (Expr)walker.process(rand2) );
        return ret;
    }


    public String getDefaultDisplayName() {
        return "AndAndOpExpr(op: "+op+")";
    }

    //END: Generated from @child and @property
}
