/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
  * @grammar
  */
public abstract class AssignableExpr extends Expr {

    public boolean isLhs() {
        ASTObject parent = this.getParent();
        return (parent instanceof BangExpr &&
                ((BangExpr)parent).getLhs() == this);
    }

    // ------------------------------
    // bcg
    protected void cgValue(CodeBuilder cb) {
        cgLvalue(cb);
        cgLtoRvalue(cb);
    }

    /** Generate code to place the lvalue of this expression on the
        top of the stack.  Only assignable expressions have lvalues.
        Some assignable expressions (such as static field access
        expressions) do not have lvalues, and for them this method is
        a noop. */
    abstract protected void cgLvalue(CodeBuilder cb);

    /** Generate code to remove the lvalue at the top of the stack (if
        any) and place the appropriate rvalue on the stack.  Assumes
        that said lvalue was generated by this expression. */
    abstract protected void cgLtoRvalue(CodeBuilder cb);

    /** Generate code to actually assign into this assignable.
        Assumes that the new rvalue is at the top of the stack, and
        that this expression's lvalue (if any) is just below it. */
    abstract protected void cgAssignment(CodeBuilder cb);

    /** Generates code to duplicate the top lvalue on the stack.  Assumes
        that said lvalue on the stack was generated by this expression. */
    abstract protected void cgDupLvalue(CodeBuilder cb);

    /** Generates code to duplicate the top rvalue on the stack and
        places it BELOW the lvalue that may be just below it.  Assumes
        that said lvalue and rvalue on the stack were generated by this
        expression. */
    abstract protected void cgDupRvalue(CodeBuilder cb);

    /** Is this an expression that we can generate an IINC code for?
        That is, a VarExpr to a variable of type int with an index
        that will fit in one byte? */
    protected boolean isIncable() { return false; }

    //BEGIN: Generated from @child and @property

    public AssignableExpr(SourceLocation location) {
        super(location);

    }


    public String getDefaultDisplayName() {
        return "AssignableExpr()";
    }

    //END: Generated from @child and @property
}
