/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import org.aspectj.compiler.base.cst.*;

/**
  * @grammar catch (formal) body
  * @child FormalDec formal
  * @child Stmt body
  */
public class CatchClause extends ASTObject {

    //INTRO from ScopePass
    public void preScope(ScopeWalker walker) { walker.pushBlock(); }
    public ASTObject postScope(ScopeWalker walker) { walker.popBlock(); return this; }

    public void checkSpec() {
        Type exnType = formal.getType();
        Type throwableType = getTypeManager().getThrowableType();
        if (! exnType.isSubtypeOf(throwableType)) {
            formal.getTypeD().showTypeError(exnType, throwableType);
        }
        
        body.requireBlockStmt();
    }

    public void unparse(CodeWriter writer) {
        writer.writeKeyword("catch");
        writer.optionalSpace();
        writer.write('(');
        writer.write(formal);
        writer.write(')');
        writer.optionalSpace();
        writer.write(body);
    }

    // ------------------------------
    // Intro:  FrameLocPass

    public void walkFrameLoc(FrameLocPass walker) {
        int start = walker.getfs();
        super.walkFrameLoc(walker);
        walker.setfs(start);
    }

    //BEGIN: Generated from @child and @property
    protected FormalDec formal;
    public FormalDec getFormal() { return formal; }
    public void setFormal(FormalDec _formal) {
        if (_formal != null) _formal.setParent(this);
        formal = _formal;
    }

    protected Stmt body;
    public Stmt getBody() { return body; }
    public void setBody(Stmt _body) {
        if (_body != null) _body.setParent(this);
        body = _body;
    }

    public CatchClause(SourceLocation location, FormalDec _formal, Stmt _body) {
        super(location);
        setFormal(_formal);
        setBody(_body);
    }
    protected CatchClause(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        CatchClause ret = new CatchClause(getSourceLocation());
        ret.preCopy(walker, this);
        if (formal != null) ret.setFormal( (FormalDec)walker.process(formal) );
        if (body != null) ret.setBody( (Stmt)walker.process(body) );
        return ret;
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return formal;
        case 1: return body;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "formal";
        case 1: return "body";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setFormal((FormalDec)child); return;
        case 1: setBody((Stmt)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 2;
    }

    public String getDefaultDisplayName() {
        return "CatchClause()";
    }

    //END: Generated from @child and @property
}


