/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;
import org.aspectj.compiler.base.bcg.CodeBuilder;

import java.util.*;

/**
 * @grammar rand1 <op> rand2
 */
public abstract class ConditionalOpExpr extends BinopExpr {

    protected Type discoverType() {
        if (! (getRand1().getType().isBoolean() &&
               getRand2().getType().isBoolean())) {
            showOperatorTypeError(op, getRand1().getType(), getRand2().getType());
        }
        return getTypeManager().booleanType;
    }

    protected Type getLiftType() {
        if (getRand1().getType().isBoolean() &&
            getRand2().getType().isBoolean()) {
            return getTypeManager().booleanType;
        } else {
            return getTypeManager().anyType;
        }
    }

    // ------------------------------
    // bcg
    protected void cgOp(CodeBuilder cb, Type ty) {
        throw new RuntimeException("Attempt to generate code for effect op "
                                   + getOp());
    }

    //BEGIN: Generated from @child and @property

    public ConditionalOpExpr(SourceLocation location, Expr _rand1, String _op, Expr _rand2) {
        super(location, _rand1, _op, _rand2);

    }
    protected ConditionalOpExpr(SourceLocation source) {
        super(source);
    }


    public String getDefaultDisplayName() {
        return "ConditionalOpExpr(op: "+op+")";
    }

    //END: Generated from @child and @property
}
