/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;

import org.aspectj.compiler.base.bcg.CodeBuilder;

/**
  * @grammar ;
  */

public class EmptyStmt extends Stmt {

    /**
      * when the -lenient option is used,
      *   we declare that empty stmts don't have to be live.
      * This makes code like: return 2;;;;; legal.
      */
    public boolean mustBeLive() { return !getOptions().lenient; }

    public void unparse(CodeWriter writer) {
        writer.closeStmt();
    }

    // ------------------------------
    // bcg

    protected void cgStmt(CodeBuilder cb) {}

    //BEGIN: Generated from @child and @property

    public EmptyStmt(SourceLocation location) {
        super(location);

    }

    public ASTObject copyWalk(CopyWalker walker) {
        EmptyStmt ret = new EmptyStmt(getSourceLocation());
        ret.preCopy(walker, this);

        return ret;
    }


    public String getDefaultDisplayName() {
        return "EmptyStmt()";
    }

    //END: Generated from @child and @property
}


