/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;

import org.aspectj.compiler.base.bcg.FieldBuilder;
import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
  * @grammar intLiteral
  * @property int intValue
  */
public class IntLiteralExpr extends NumericLiteralExpr {

    public long   getLongValue()   { return (long)intValue; }
    public float  getFloatValue()  { return (float)intValue; }
    public double getDoubleValue() { return (double)intValue; }
    public String getStringValue() { 
        if (getType() instanceof CharType) 
            return "" + (char) intValue; 
        else 
            return "" + intValue; 
    }

    public IntLiteralExpr(SourceLocation source, int _intValue) {
        this(source, source.getCompiler().getTypeManager().intType, "" + _intValue, _intValue);
    }

    public IntLiteralExpr(SourceLocation source, Type type, int _intValue) {
        this(source, type, createUsefulString(type, _intValue), _intValue);
    }

    private static String createUsefulString(Type type, int i) {
        if (type instanceof IntType) return "" + i;
        return "((" + type.getString() + ")" + i + ")";
    }

    public boolean isAssignableTo(Type type) {
        if        (type instanceof ByteType) {
            return (Byte.MIN_VALUE <= intValue) && (intValue <= Byte.MAX_VALUE);
        } else if (type instanceof ShortType) {
            return (Short.MIN_VALUE <= intValue) && (intValue <= Short.MAX_VALUE);
        } else if (type instanceof CharType) {
            return (Character.MIN_VALUE <= intValue) && (intValue <= Character.MAX_VALUE);
        } else if (type instanceof NumericType) {
            return true;
        } else {
            return false;
        }
    }

    // ------------------------------
    // bcg
    protected void cgValue(CodeBuilder cb) {
        cb.emitIntConstant(intValue);
    }
    public void addConstant(FieldBuilder fb) {
        fb.setConstantValue(intValue);
    }
    public boolean isConstantZero() { return intValue == 0; }

    //BEGIN: Generated from @child and @property
    protected int intValue;
    public int getIntValue() { return intValue; }
    public void setIntValue(int _intValue) { intValue = _intValue; }

    public IntLiteralExpr(SourceLocation location, Type _type, String _value, int _intValue) {
        super(location, _type, _value);
        setIntValue(_intValue);
    }
    protected IntLiteralExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        IntLiteralExpr ret = new IntLiteralExpr(getSourceLocation());
        ret.preCopy(walker, this);
        ret.type = type;
        ret.value = value;
        ret.intValue = intValue;
        return ret;
    }


    public String getDefaultDisplayName() {
        return "IntLiteralExpr(type: "+type+", "+"value: "+value+", "+"intValue: "+intValue+")";
    }

    //END: Generated from @child and @property
}
