/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.FlowCheckerPass;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;

import org.aspectj.compiler.base.bcg.FieldBuilder;
import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
  * @grammar literal
  * @property Type type
  * @property String value
  */
public abstract class LiteralExpr extends Expr {


    /** Return a string representing the Literal expr, NOT to be used
        for unparsing purposes, but rather the string that would be
        returned by the {@link String#valueOf} method.
    */
    abstract public String getStringValue();

    // XXX these should all be exceptions!
    public int     getIntValue() { return 0; }
    public long    getLongValue() { return 0L; }
    public float   getFloatValue() { return 0.0f; }
    public double  getDoubleValue() { return 0.0; }
    public boolean getBooleanValue() { return false; }

    public Type discoverType() {
        return type;
    }

    public Expr makeReference() { return this; }

    public boolean canBeCopied() {
        return true;
    }
    
    public boolean isUltimatelyLiteral() {
        return true;
    }

    public void unparse(CodeWriter writer) throws IOException {
        writer.write(value);
    }

    // ------------------------------
    // bcg
    /** Leaves the value of this expression on the stack and casts the
        value to castTo.  We do a better job than for regular
        expressions since literal expressions can be statically cast.  */
    protected void cgValue(CodeBuilder cb, Type castTo) {
        castTo.foldCast(this).cgValue(cb);
    }

    protected void cgEffect(CodeBuilder cb) { }

    abstract public void addConstant(FieldBuilder fb);

    //BEGIN: Generated from @child and @property
    protected Type type;
    public Type getType() { return type; }
    public void setType(Type _type) { type = _type; }

    protected String value;
    public String getValue() { return value; }
    public void setValue(String _value) { value = _value; }

    public LiteralExpr(SourceLocation location, Type _type, String _value) {
        super(location);
        setType(_type);
        setValue(_value);
    }
    protected LiteralExpr(SourceLocation source) {
        super(source);
    }


    public String getDefaultDisplayName() {
        return "LiteralExpr(type: "+type+", "+"value: "+value+")";
    }

    //END: Generated from @child and @property
}
