/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;

import org.aspectj.compiler.base.bcg.FieldBuilder;
import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
  * @grammar longLiteral
  * @property long longValue
  */
public class LongLiteralExpr extends NumericLiteralExpr {

    public int    getIntValue()    { return (int)longValue; }
    public float  getFloatValue()  { return (float)longValue; }
    public double getDoubleValue() { return (double)longValue; }
    public String getStringValue() { return "" + longValue; }

    public LongLiteralExpr(SourceLocation location, long _longValue) {
        this(location, location.getCompiler().getTypeManager().longType, _longValue + "L", _longValue);
    }

    // ------------------------------
    // bcg
    protected void cgValue(CodeBuilder cb) {
        cb.emitLongConstant(longValue);
    }
    public void addConstant(FieldBuilder fb) {
        fb.setConstantValue(longValue);
    }
    public boolean isConstantZero() { return longValue == 0L; }

    //BEGIN: Generated from @child and @property
    protected long longValue;
    public long getLongValue() { return longValue; }
    public void setLongValue(long _longValue) { longValue = _longValue; }

    public LongLiteralExpr(SourceLocation location, Type _type, String _value, long _longValue) {
        super(location, _type, _value);
        setLongValue(_longValue);
    }
    protected LongLiteralExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        LongLiteralExpr ret = new LongLiteralExpr(getSourceLocation());
        ret.preCopy(walker, this);
        ret.type = type;
        ret.value = value;
        ret.longValue = longValue;
        return ret;
    }


    public String getDefaultDisplayName() {
        return "LongLiteralExpr(type: "+type+", "+"value: "+value+", "+"longValue: "+longValue+")";
    }

    //END: Generated from @child and @property
}
