/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.JavaCompiler;

public class NullType extends RefType {
    public NullType(JavaCompiler compiler) {
        super(compiler);
    }

    // ??? This is subtly wrong in the model, but it shouldn't lead to
    // problems.  In actuality, the null type is assignable to all
    // reference types.
    public boolean isSubtypeOf(Type other) {
        return (other instanceof RefType);
    }

    public boolean isAssignableFrom(Type other) {
        //JJH - I believe that NullType is never assignable from anything
        //EH - but it's useful to me to be assignable from the NullType
        return other instanceof NullType;
        //return false;
        //return other.isReferenceType();
    }

    public boolean isMethodConvertableTo(Type other) {
        return other.isReferenceType();
    }
    
    public boolean isCoercableTo(Type other) {
        return true;
    }

    protected void showNotFoundError(String id, ASTObject fromWhere, String kind) {
        getCompiler().showError(fromWhere, "no " + kind + "s on null");
    }

    public boolean isAnyType() {
        return false;
    }

    public String getString() {
        return "null";
    }

    public String toShortString() {
        return "null";
    }

    // vm internals
    public String getDescriptor() {
        throw new RuntimeException("No bytecode type descriptor for Null type");
    }

}
