/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.FlowCheckerPass;
import org.aspectj.compiler.base.AssignmentCheckerPass;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;

import java.util.*;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
 * @grammar (expr)
 * @child Expr expr
 *
 */
public class ParenExpr extends Expr {

    protected void bindNamesSelf() {
        //setType(discoverType());
    }

    protected Type discoverType() {
        return expr.getType();
    }

    public Expr makeReference() {
        Expr ret = expr.makeReference();
        if (!(ret instanceof ParenExpr)) return ret;
        return super.makeReference();
    }
    
    public boolean canBeCopied() {
        return expr.canBeCopied();
    }
    
    public boolean isUltimatelyLiteral() {
        return expr.isUltimatelyLiteral();
    }

    public void unparse(CodeWriter writer) {
        writer.write('(');
        writer.write(expr);
        writer.write(')');
    }

    // ------------------------------
    // INTRO from AssignmentCheckerPass

    public ASTObject postAssignmentCheck(AssignmentCheckerPass walker) {
        if (expr instanceof LiteralExpr) {
            return (LiteralExpr)expr.copy().setSource(this);
        } else {
            return this;
        }
    }

    // ------------------------------
    // bcg
    protected void cgValue(CodeBuilder cb) {
        expr.cgValue(cb);
    }
    protected void cgValue(CodeBuilder cb, Type castTo) {
        expr.cgValue(cb, castTo);
    }
    protected void cgEffect(CodeBuilder cb) {
        expr.cgEffect(cb);
    }
    protected void cgBuffer(CodeBuilder cb) {
        expr.cgBuffer(cb);
    }
    protected void cgTest(CodeBuilder cb, Label tdest, Label fdest) {
        expr.cgTest(cb, tdest, fdest);
    }

    //BEGIN: Generated from @child and @property
    protected Expr expr;
    public Expr getExpr() { return expr; }
    public void setExpr(Expr _expr) {
        if (_expr != null) _expr.setParent(this);
        expr = _expr;
    }

    public ParenExpr(SourceLocation location, Expr _expr) {
        super(location);
        setExpr(_expr);
    }
    protected ParenExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        ParenExpr ret = new ParenExpr(getSourceLocation());
        ret.preCopy(walker, this);
        if (expr != null) ret.setExpr( (Expr)walker.process(expr) );
        return ret;
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return expr;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "expr";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setExpr((Expr)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }

    public String getDefaultDisplayName() {
        return "ParenExpr()";
    }

    //END: Generated from @child and @property
}
