/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import java.io.IOException;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

public abstract class PrimitiveType extends Type {

    public abstract String getName();

    public PrimitiveType(JavaCompiler compiler) {
        super(compiler);
    }

    protected void showNotFoundError(String id, ASTObject fromWhere, String kind) {
        getCompiler().showError(fromWhere, "no " + kind + "s on primitive type: " + getName());
    }

    public Expr getClassExpr() {
        return getAST().makeStaticGet(getRefType(), "TYPE");
    }

    public Expr makeObject(Expr expr) {
        //XXX???expr.typeD = this;
        return getAST().makeNew(getRefType(), expr);
    }

    public Expr fromObject(Expr expr) {
    	//??? can this ever be wrong?
    	if (expr.getType().isPrimitive()) return getAST().makeCast(this, expr);
    	
    	return getAST().makeStaticCall(getTypeManager().getConversionsType(),
    								   getName() + "Value", expr);
    }
    	
//        Expr castExpr = getAST().makeCast(getRefType(), expr);
//        return getAST().makeCall(castExpr, getName()+"Value");
//    }

    public String toShortString() {
        return getName();
    }

    public boolean isEquivalent(Type other) {
        if (other.isAnyType()) return true;
        // Types are guaranteed to have only one instance per
        return this == other;
    }

    public final boolean isCoercableTo(Type other) {
        return this.isEquivalent(other) || this.isCoercableToOtherType(other);
    }
    protected boolean isCoercableToOtherType(Type other) { return false; }

    public final boolean isAssignableFrom(Type other) {
        return this.isEquivalent(other) || this.isAssignableFromOtherType(other);
    }
    protected boolean isAssignableFromOtherType(Type other) { return false; }

    public String toString() {
        return "PrimitiveType(" + getName() + ")";
    }

    public void unparse(CodeWriter writer) throws IOException {
        writer.write(getString());
    }

    // ------------------------------
    // code gen
    final void emitNewarray(CodeBuilder cb) { cb.emitNEWARRAY(getTypeIndex()); }

    final void emitInstanceof(CodeBuilder cb) { unsupportedEmit(); }
}
