/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;

import java.util.*;
import java.io.File;

public abstract class SourceLocation {
    public abstract JavaCompiler getCompiler();

    public int getStartPosition() { return -1; }
    public int getEndPosition() { return -1; }

    public int getBeginLine() { return -1; }

    public int getEndLine() { return -1; }

    public int getBeginColumn() { return -1; }

    public int getEndColumn() { return -1; }

    public Comment getComment() {
        return null;
    }

    public void addComment(Comment _comment) {
        throw new RuntimeException("can't add a comment to this source location");
    }

    public void clearComment() { }

    public void setFormalComment(String comment) {
        //System.out.println("adding formal comment: " + comment + " to " + this.getDefaultDisplayName());
        addComment(new FormalComment(comment));
    }

    public String getFormalComment() { return ""; }


    public CompilationUnit getCompilationUnit() {
        return getCompiler().getWorld().missingCompilationUnit;
    }

    public java.io.File getSourceFile() {
        return getCompilationUnit().getSourceFile();
    }

    public String getSourceFileName() {
        return getCompilationUnit().getSourceCanonicalPath();
    }

    public String getSourceDirectoryName() {
        return getCompilationUnit().getSourceDirectory();
    }


    public void showError(String message) {
        getCompiler().showError(new IntLiteralExpr(this, 1), message);
    }

    public boolean hasSource() {
        return false;
    }
    public boolean fromSource() {
        return hasSource() && getCompilationUnit().getDecs() != null;
    }

    public boolean isSynthetic() {
        return true;
    }

    public ASTObject getSourceObject() { return null; }
}
