/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;

import org.aspectj.compiler.base.bcg.CodeBuilder;

/**
 * @grammar super
 */
public class SuperExpr extends Expr {
    public final Type discoverType() {
        return getThisType().getSuperClassType();
    }
    
    public NameType getThisType() {
    	return (NameType) getDeclaringType();
    }

    public void unparse(CodeWriter writer) {
    	if (inAllowedPosition()) {
        	writer.writeKeyword("super");
    	} else {
    		writer.writeKeyword("this");
    	}
    }


    // ------------------------------
    // Intro: from SpecChecker

	// this is very similar to the test in TypeExpr
    public void checkSpec() {
        super.checkSpec();
  
        if (!inAllowedPosition()) showError("bad identifier");
    }
    
    
    public boolean inAllowedPosition() {
        // we are only allowed in a limited number of places.
        ASTObject self = this;
        ASTObject parent = getParent();

		// the check for the reverse pointer may not actually be necessary 
        if (parent instanceof Expr) {
            if (parent instanceof CallExpr) {
                if (((CallExpr)parent).getExpr() == self) return true;
            } else if (parent instanceof FieldAccessExpr) {
                if (((FieldAccessExpr)parent).getExpr() == self) return true;
            }
        }
        
        return false;
    }    	
    
    
    // ------------------------------
    // INTRO from InnerInfoPass -- should have a more generic name

    public void postInnerInfo(InnerInfoPass walker) {
        if (!walker.isAccessibleExactly(getThisType())) {
            if (!fromSource() && getParent().fromSource()) {
                getParent().showError("no instance available");
            } else {
                showError("no instance available");
            }
        }
    }

    // ------------------------------
    // bcg

    protected void cgValue(CodeBuilder cb) {
        cb.emitALOAD(0);
    }

    //BEGIN: Generated from @child and @property

    public SuperExpr(SourceLocation location) {
        super(location);

    }

    public ASTObject copyWalk(CopyWalker walker) {
        SuperExpr ret = new SuperExpr(getSourceLocation());
        ret.preCopy(walker, this);

        return ret;
    }


    public String getDefaultDisplayName() {
        return "SuperExpr()";
    }

    //END: Generated from @child and @property
}
