/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;

/**
 * @grammar case label: stmts
 * @child Expr label
 * @child Stmts stmts
 */

public class SwitchClause extends ASTObject {

    public void unparse(CodeWriter writer) {
        if (label == null) {
            writer.writeln("default: ");
        } else {
            writer.writeKeyword("case");
            writer.requiredSpace();
            writer.write(label);
            writer.writeln(":");
        }
        if (stmts == null) return;

        writer.indent();
        writer.write(stmts);
        writer.dedent();
    }

    //BEGIN: Generated from @child and @property
    protected Expr label;
    public Expr getLabel() { return label; }
    public void setLabel(Expr _label) {
        if (_label != null) _label.setParent(this);
        label = _label;
    }

    protected Stmts stmts;
    public Stmts getStmts() { return stmts; }
    public void setStmts(Stmts _stmts) {
        if (_stmts != null) _stmts.setParent(this);
        stmts = _stmts;
    }

    public SwitchClause(SourceLocation location, Expr _label, Stmts _stmts) {
        super(location);
        setLabel(_label);
        setStmts(_stmts);
    }
    protected SwitchClause(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        SwitchClause ret = new SwitchClause(getSourceLocation());
        ret.preCopy(walker, this);
        if (label != null) ret.setLabel( (Expr)walker.process(label) );
        if (stmts != null) ret.setStmts( (Stmts)walker.process(stmts) );
        return ret;
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return label;
        case 1: return stmts;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "label";
        case 1: return "stmts";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setLabel((Expr)child); return;
        case 1: setStmts((Stmts)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 2;
    }

    public String getDefaultDisplayName() {
        return "SwitchClause()";
    }

    //END: Generated from @child and @property
}


