/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import org.aspectj.compiler.base.cst.*;

import java.util.Iterator;

/**
  * @grammar (name, )*
  * @children TypeD typeD
  */
public class TypeDs extends ASTObject {
    public boolean contains(TypeD name) {
        final int N = size();
        //System.out.println("N: " + N);
        for (int i = 0; i < N; i++) {
            TypeD thisName = get(i);
            //System.out.println("thisName: " + thisName);
            if (thisName.getType().isEquivalent(name.getType())) {
                //System.out.println("contains: " + name);
                return true;
            }
        }
        return false;
    }


    public void unparse(CodeWriter writer) {
        writer.writeChildrenWithCommas(this);
    }

    public Iterator iterator() {
        return new Iterator() {
                private int i = 0;
                public boolean hasNext() { return i < size; }
                public Object next() { return children[i++]; }
                public void remove() {
                    throw new UnsupportedOperationException();
                }
            };
    }

    //BEGIN: Generated from @child and @property
    protected int size;
    public TypeD[] children;

    public TypeDs(SourceLocation location, TypeD[] _children) {
        super(location);
        for(int i=0; i<_children.length; i++) {
            if (_children[i] != null) _children[i].setParent(this);
        }
        children = _children;
        size = _children.length;
    }

    public TypeDs(SourceLocation location) {
        this(location, new TypeD[] {});
    }

    public TypeDs(SourceLocation location, TypeD child1) {
        this(location, new TypeD[] {child1});
    }

    public TypeDs(SourceLocation location, TypeD child1, TypeD child2) {
        this(location, new TypeD[] {child1, child2});
    }

    public TypeDs(SourceLocation location, TypeD child1, TypeD child2, TypeD child3) {
        this(location, new TypeD[] {child1, child2, child3});
    }

    public ASTObject copyWalk(CopyWalker walker) {
        final int N = size;
        TypeD[] copiedChildren = new TypeD[N];
        int newIndex = 0;
        for(int oldIndex=0; oldIndex<N; oldIndex++) {
            TypeD newChild = (TypeD)walker.process(children[oldIndex]);
            if (newChild != null) copiedChildren[newIndex++] = newChild;
        }
        TypeDs ret = new TypeDs(getSourceLocation(),copiedChildren);
        ret.size = newIndex;
        ret.setSource(this);
        return ret;
    }

    public ASTObject getChildAt(int childIndex) { return get(childIndex); }
    public void setChildAt(int childIndex, ASTObject child) { set(childIndex, (TypeD)child); }
    public String getChildNameAt(int childIndex) { return "typeD"+childIndex; }

    public int getChildCount() { return size; }
    public int size() { return size; }

    public TypeD get(int index) {
        if (index >= size) throw new ArrayIndexOutOfBoundsException();
        return children[index];
    }

    public void set(int index, TypeD child) {
        if (index >= size) throw new ArrayIndexOutOfBoundsException();
        children[index] = child;
        child.setParent(this);
    }

    public void resize(int newSize) {
        if (newSize > children.length) {
            TypeD[] newChildren = new TypeD[children.length*2 + 1];
            System.arraycopy(children, 0, newChildren, 0, children.length);
            children = newChildren;
        }
        size = newSize;
    }

    public void addAll(TypeDs collection) {
        addAll(size, collection);
    }

    public void addAll(int index, TypeDs collection) {
        for(int i=0; i<collection.size(); i++) {
            add(index+i, collection.get(i));
        }
    }

    public void add(TypeD child) {
        add(size, child);
    }

    public void add(int index, TypeD child) {
        if (child == null) return;

        if (index < 0 || index > size) throw new ArrayIndexOutOfBoundsException();

        resize(size+1);

        for(int moveIndex = size-1; moveIndex > index; moveIndex--) {
            children[moveIndex] = children[moveIndex-1];
        }

        children[index] = child;
        child.setParent(this);
    }

    public void remove(int index) {
        if (index < 0 || index > size) throw new ArrayIndexOutOfBoundsException();

        size -= 1;

        for(int moveIndex = index; moveIndex < size; moveIndex++) {
            children[moveIndex] = children[moveIndex+1];
        }
    }

    public String getDefaultDisplayName() {
        return "TypeDs()";
    }

    //END: Generated from @child and @property
}
