/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.FlowCheckerPass;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
 * @grammar typeD;
 * @child TypeD typeD
 */
public class TypeExpr extends Expr {
    public Type discoverType() { return typeD.getType(); }

    public void unparse(CodeWriter writer) {
        //System.out.println("typeD: " + typeD);
        typeD.unparse(writer, true);
    }

    public Expr makeReference() {
        return (Expr)this.copy();
    }

    // ------------------------------
    // Intro: from SpecChecker
	
	// this is very similar to the test in TypeExpr
    public void checkSpec() {
        super.checkSpec();
        // make sure the type is accessible from here
        if (fromSource() && getType() != null &&
            getType().getTypeDec() != null &&
            !getType().getTypeDec().isAccessible(this)) {
            showError("can't access " + getType().getTypeDec().toShortString());
            return;
        }   

        // we are only allowed in a limited number of places.
        ASTObject self = this;
        ASTObject parent = getParent();

		// the check for the reverse pointer may not actually be necessary
        while (true) {
            if (parent instanceof Expr) {
                if (parent instanceof CallExpr) {
                    if ((((CallExpr)parent).getExpr() == self)
                        && ((CallExpr)parent).getMethod().isStatic()) return;
                } else if (parent instanceof FieldAccessExpr) {
                    if ((((FieldAccessExpr)parent).getExpr() == self)
                        && ((FieldAccessExpr)parent).getField().isStatic()) return;
                } else if (parent instanceof ParenExpr) {
                    self = parent;
                    parent = parent.getParent();
                    continue;
                }
            }
            break;
        }
        showError("bad identifier");
    }

    // ------------------------------
    // bcg

    protected void cgValue(CodeBuilder cb) {}
    protected void cgEffect(CodeBuilder cb) {}

    //BEGIN: Generated from @child and @property
    protected TypeD typeD;
    public TypeD getTypeD() { return typeD; }
    public void setTypeD(TypeD _typeD) {
        if (_typeD != null) _typeD.setParent(this);
        typeD = _typeD;
    }

    public TypeExpr(SourceLocation location, TypeD _typeD) {
        super(location);
        setTypeD(_typeD);
    }
    protected TypeExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        TypeExpr ret = new TypeExpr(getSourceLocation());
        ret.preCopy(walker, this);
        if (typeD != null) ret.setTypeD( (TypeD)walker.process(typeD) );
        return ret;
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return typeD;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "typeD";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setTypeD((TypeD)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }

    public String getDefaultDisplayName() {
        return "TypeExpr()";
    }

    //END: Generated from @child and @property
}
