/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import java.io.IOException;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

public final class VoidType extends PrimitiveType {

    public String getName()      { return "void"; }
    public int    getTypeIndex() { return Type.VOID; }
    public Type   getRefType()   { return getTypeManager().getType("java.lang", "Void"); }
    public Expr   getNullExpr()  { return getAST().makeNull(); }

    public Expr makeObject(Expr expr) {
        //XXX this is serious trouble
        return getAST().makeCast(getTypeManager().getObjectType(), getAST().makeNull());
    }

    public Expr fromObject(Expr expr) {
		return getAST().makeStaticCall(getTypeManager().getConversionsType(),
    								   "voidValue", expr);
    }

    public VoidType(JavaCompiler compiler) {
        super(compiler);
    }

    // constant fold

    final LiteralExpr foldCast(LiteralExpr lit) { return unsupportedFold(); }
    final LiteralExpr foldPlusOp(LiteralExpr rand) { return unsupportedFold(); }
    final LiteralExpr foldMinusOp(LiteralExpr rand) { return unsupportedFold(); }
    final LiteralExpr foldBitNotOp(LiteralExpr rand) { return unsupportedFold(); }
    final LiteralExpr foldLogNotOp(LiteralExpr rand) { return unsupportedFold(); }
    final LiteralExpr foldAddOp(LiteralExpr lit1, LiteralExpr lit2) { return unsupportedFold(); }
    final LiteralExpr foldNumericOp(String op, LiteralExpr lit1, LiteralExpr lit2) { return unsupportedFold(); }
    final LiteralExpr foldBitwiseOp(String op, LiteralExpr lit1, LiteralExpr lit2) { return unsupportedFold(); }
    final LiteralExpr foldShiftOp(String op, LiteralExpr lit1, LiteralExpr lit2) { return unsupportedFold(); }
    final LiteralExpr foldEqualityTestOp(String op, LiteralExpr lit1, LiteralExpr lit2) { return unsupportedFold(); }
    final LiteralExpr foldNumericTestOp(String op, LiteralExpr lit1, LiteralExpr lit2) { return unsupportedFold(); }

    // vm internal
    public String getDescriptor() { return "V"; }
    public int getSlotCount() { return 0; }

    // code gen
    public final void emitLoad(CodeBuilder cb, int loc) { }
    public final void emitStore(CodeBuilder cb, int loc) { }
    public final void emitReturn(CodeBuilder cb) { cb.emitRETURN(); }
    final void emitPop(CodeBuilder cb) { }
    final void emitDup(CodeBuilder cb) {  }
    final void emitDupX1(CodeBuilder cb) {  }
    final void emitDupX2(CodeBuilder cb) {  }

    final void emitZero(CodeBuilder cb)       { unsupportedEmit(); }
    final void emitOne(CodeBuilder cb)        { unsupportedEmit(); }
    final void emitMinusOne(CodeBuilder cb)   { unsupportedEmit(); }
    final void emitBitNot(CodeBuilder cb)     { unsupportedEmit(); }
    final void emitLogNot(CodeBuilder cb)     { unsupportedEmit(); }
    final void emitNeg(CodeBuilder cb)        { unsupportedEmit(); }
    final void emitAdd(CodeBuilder cb)        { unsupportedEmit(); }
    final void emitAload(CodeBuilder cb)      { unsupportedEmit(); }
    final void emitAstore(CodeBuilder cb)     { unsupportedEmit(); }
    final void emitNumericOp(CodeBuilder cb, String op) { unsupportedEmit(); }
    final void emitBitwiseOp(CodeBuilder cb, String op) { unsupportedEmit(); }
    final void emitShiftOp(CodeBuilder cb, String op)   { unsupportedEmit(); } 
    final void emitCast(CodeBuilder cb, Type castTo)    { unsupportedEmit(); }
    final void emitEqualityCompare(CodeBuilder cb, String op, Label t, Label f) { 
        unsupportedEmit(); 
    }
    final void emitNumericCompare(CodeBuilder cb, String op, Label t, Label f) { 
        unsupportedEmit(); 
    }


}
