/* -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.bcg.pool;

import org.aspectj.compiler.base.bcg.*;

import java.io.*;
import java.util.*;

import org.aspectj.compiler.base.ast.ArrayType;
import org.aspectj.compiler.base.ast.NameType;
import org.aspectj.compiler.base.ast.RefType;

public abstract class Constant implements Comparable {

    static final byte UTF_TAG                  = 1;
    static final byte INT_TAG                  = 3;
    static final byte FLOAT_TAG                = 4;
    static final byte LONG_TAG                 = 5;
    static final byte DOUBLE_TAG               = 6;
    static final byte CLASS_TAG                = 7;
    static final byte STRING_TAG               = 8;
    static final byte FIELD_REF_TAG            = 9;
    static final byte METHOD_REF_TAG           = 10;
    static final byte INTERFACE_METHOD_REF_TAG = 11;
    static final byte NAME_AND_TYPE_TAG        = 12;

    private int index = 0;
    private int popularity = 0;
    public void makePopular() { popularity++; }

    void setIndex(int index) { this.index = index; }
    public short getIndex() { return (short)index; }
    public void writeIndex(DataOutputStream stream) throws IOException {
        stream.writeShort((short)index);
    }
    public void writeBriefIndex(DataOutputStream stream) throws IOException {
        stream.writeByte((byte)index);
    }
    abstract void writeTo(DataOutputStream stream) throws IOException;
    abstract void readFrom(DataInputStream stream, ConstantPool pool) throws IOException;
    public int compareTo(Object o) {
        // sorting most popular first
        int other = ((Constant)o).popularity;
        
        return (popularity > other) ? +1 : (popularity == other) ? 0 : -1;
    }
    boolean isBig() { return false; } // overridden in Long and Double
    public void display(int indent, boolean inline) {
        System.err.print(this);
    }

    static String rep(int index, String x) {
        return x + ":" + Integer.toHexString(index);
    }
}
