/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.cst;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;
import java.util.*;

/**
 * captures the scope of var, meth and type declarations
 * 
 */
public abstract class Scope extends CompilerObject {
    private Scope parent;
    
    public Scope(JavaCompiler compiler, Scope parent) {
        super(compiler);
        this.parent = parent;
    }
    
    
    /**
     * Returns the parent to be used by ScopeWalker for pushing and popping Scopes
     */
    public final Scope getStackParent() { return parent; }
    
    public Scope getParent() { return parent; }
    
    public void setStackParent(Scope p) { parent = p; }
    
    /**
     * Finds the matching type for an unqualified type name.
     */
    public Type findType(String name, ASTObject fromWhere) {
        return getParent().findType(name, fromWhere);
    }

    /**
     * Returns a properly bound Expr for an unqualified name that appears in an
     * expr position.  Will return null if that's not possible.
     */
    public Expr bindUnqualifiedName(String name, ASTObject fromWhere) {
        return getParent().bindUnqualifiedName(name, fromWhere);
    }
    
    /**
     * According to section 15.12.1 of JLS 2nd edition the rules for looking up an
     * unqualified method name are that you find the first enclosing type that has
     * a member with a matching name, and then you use the rules of applicability and
     * accessibility in that type only to find the actual method.
     *
     * This method implements the first step in that process.
     */
    public Type findMethodLookupType(String name, ASTObject fromWhere) {
        return getParent().findMethodLookupType(name, fromWhere);
    }
    
    public String shortToString() { return "scope"; }
    public String toString() {
        String ret = this.shortToString();
        if (getStackParent() != null) ret += "<-" + getStackParent().toString();
        return ret;
    }
}
