/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.cst;

import java.util.*;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.cst.*;

import org.aspectj.compiler.base.CodeWriter;


/**
  * @grammar packageName.name ?qualified
  * @child Name name
  */

public class UnresolvedNameTypeD extends TypeD {
    public Type getType() {
        return getTypeManager().TYPE_NOT_FOUND;
    }
    

    //INTRO from ScopeWalker
    public ASTObject postScope(ScopeWalker walker) {
        Type resolvedType = name.resolveType(walker.getScope());
        return new ResolvedTypeD(getSourceLocation(), resolvedType);
    }    
    
    //BEGIN: Generated from @child and @property
    protected Name name;
    public Name getName() { return name; }
    public void setName(Name _name) {
        if (_name != null) _name.setParent(this);
        name = _name;
    }
    
    public UnresolvedNameTypeD(SourceLocation location, Name _name) {
        super(location);
        setName(_name);
    }
    protected UnresolvedNameTypeD(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        UnresolvedNameTypeD ret = new UnresolvedNameTypeD(getSourceLocation());
        ret.preCopy(walker, this);
        if (name != null) ret.setName( (Name)walker.process(name) );
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return name;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "name";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setName((Name)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }
    
    public String getDefaultDisplayName() {
        return "UnresolvedNameTypeD()";
    }
    
    //END: Generated from @child and @property
}
