/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts;

import org.aspectj.compiler.crosscuts.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;
import org.aspectj.util.CollectionUtil;

import java.util.*;

/**************************************************************************
 * Here's a new implementation of advice weaving that works basically on
 * a per-file basis.  It does require global knowledge of all the aspects
 * in the system.  The only way that restriction will ever go away is after
 * we start to support weaving at class load time.
 */
public class AdviceWeaver extends WalkerPass {
    
    public AdviceWeaver(JavaCompiler compiler) {
        super(compiler);
    }

    public String getDisplayName() {
        return "weaving advice";
    }

    protected void postProcess(ASTObject object) {
        if (object instanceof TypeDec) weaveInto((TypeDec)object);
    }

    public void weaveInto(TypeDec typeDec) {
        getCompiler().showMessage("    weaving into " + typeDec.toShortString());
        applyAspectsOf(typeDec);
        implementPlans(typeDec.joinPoints0);
        implementPlans(typeDec.joinPoints1);
        implementPlans(typeDec.joinPoints2, typeDec.getInitializerExecutionJoinPoints());
        implementPlans(typeDec.getInitializerExecutionJoinPoints());
    }
    
    void applyAspectsOf(TypeDec typeDec) {
        getWorld().runTypeDecPlanners(typeDec, TypeDecPlanner.BODY);
    }

    void implementPlans(Collection jps) {
        implementPlans(jps, null);
    }
    
    void implementPlans(Collection jps, Collection skipSet) {
        for (Iterator i = jps.iterator(); i.hasNext(); ) {
            JoinPoint joinPoint = (JoinPoint)i.next();
            if (skipSet != null && skipSet.contains(joinPoint)) continue;
            joinPoint.implementPlans();
        }
    }
}

