/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts;

import org.aspectj.compiler.crosscuts.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;
import org.aspectj.util.CollectionUtil;

import java.util.*;

public class BuildAdvicePlannersPass extends AbstractCompilerPass {

    public BuildAdvicePlannersPass(JavaCompiler compiler) {
        super(compiler);
    }

    public String getDisplayName() {
        return "building advice planners";
    }

    public void transform(TypeDec td) {
        if (td instanceof AspectDec) {
            initializeAspect((AspectDec)td);
        }
        
        // also do this on all inner types
        final int N = td.getBody().size();
        for (int i = 0; i < N; i++) {
            Dec dec = td.getBody().get(i);
            if (dec instanceof TypeDec) transform( (TypeDec)dec );
        }
    }
    
    public void initializeAspect(final AspectDec aspectDec) {
        //??? I don't think this loses anything
        if (aspectDec.isAbstract()) return;
        
        getCompiler().enterNode(aspectDec);
        
        PerClause perClause = aspectDec.getPerClause();
        if (perClause != null)  {
            JpPlanner planner = perClause.makeInitializerPlanner(
                    new PlanData(aspectDec, null));
            if (planner != null) getWorld().addJpPlanner(planner);            
        }
        
        Collection jpPlannerMakerList = aspectDec.getJpPlannerMakers();
        
        getCompiler().showMessage("  initializing advice in " +
                                  aspectDec.toShortString());
        
        for(Iterator i = jpPlannerMakerList.iterator(); i.hasNext(); ) {
            JpPlannerMaker jpPlannerMaker = (JpPlannerMaker)i.next();
            JpPlanner planner =
                jpPlannerMaker.makePlanner(new PlanData(aspectDec, jpPlannerMaker));
            if (planner != null) {
                getWorld().addJpPlanner(planner);
                getCompiler().showMessage("     " + jpPlannerMaker.toShortString());
            }
        }

        for (Iterator i = aspectDec.getExtraPlanners().values().iterator();
             i.hasNext(); )
        {
            getWorld().addJpPlanner((JpPlanner)i.next());
        }

        // System.out.println("  jpp: " + joinPointPlanners);
        getCompiler().exitNode(aspectDec);
    }

}

