/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;
import org.aspectj.util.FuzzyBoolean;

/**
  * @grammar args(typeNamePattern, ...)
  * @child FormalsPattern argTypeNames
  */
public class ArgsPcd extends Pcd {
    public String toShortString() {
        return "args(" + getArgTypeNames().toShortString() + ")";
    }

    public boolean allowsNameBinding() { return true; }
    public void checkStatic() { showNonStaticError(); }
    
    public JpPlanner makePlanner(PlanData planData) {
        return new JpPlanner() {
            public FuzzyBoolean fastMatch(JoinPoint jp) {
                //XXX optimize this: getArgTypeNames().matchesInstances(jp.getArgTypes());
                return FuzzyBoolean.MAYBE; 
            }
            public JpPlan makePlan(JoinPoint jp) {
                Exprs argsExprs = jp.makeArgsExprs();
                if (argsExprs == null) return JpPlan.NO_PLAN;
                return getArgTypeNames().makePlan(argsExprs, jp);
            }
        };
    }    
    
    //BEGIN: Generated from @child and @property
    protected FormalsPattern argTypeNames;
    public FormalsPattern getArgTypeNames() { return argTypeNames; }
    public void setArgTypeNames(FormalsPattern _argTypeNames) {
        if (_argTypeNames != null) _argTypeNames.setParent(this);
        argTypeNames = _argTypeNames;
    }
    
    public ArgsPcd(SourceLocation location, FormalsPattern _argTypeNames) {
        super(location);
        setArgTypeNames(_argTypeNames);
    }
    protected ArgsPcd(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        ArgsPcd ret = new ArgsPcd(getSourceLocation());
        ret.preCopy(walker, this);
        if (argTypeNames != null) ret.setArgTypeNames( (FormalsPattern)walker.process(argTypeNames) );
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return argTypeNames;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "argTypeNames";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setArgTypeNames((FormalsPattern)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }
    
    public String getDefaultDisplayName() {
        return "ArgsPcd()";
    }
    
    //END: Generated from @child and @property
}

