/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.JavaCompiler;
/**
 * @grammar pcd1 ? pcd2
 * @child Pcd pcd1
 * @child Pcd pcd2
 */

public abstract class BinaryPcd extends Pcd {
    abstract protected String getSymbol();

    public String toShortString() {
        return getPcd1().toShortString() + " " + getSymbol() + " " + 
                getPcd2().toShortString();
    }

    public void checkStatic() { getPcd1().checkStatic(); getPcd2().checkStatic(); }    
    
//    public boolean isAbstract(PlanData planData) {
//        return getPcd1().isAbstract(planData) || getPcd2().isAbstract(planData);
//    }    
    
    //BEGIN: Generated from @child and @property
    protected Pcd pcd1;
    public Pcd getPcd1() { return pcd1; }
    public void setPcd1(Pcd _pcd1) {
        if (_pcd1 != null) _pcd1.setParent(this);
        pcd1 = _pcd1;
    }
    
    protected Pcd pcd2;
    public Pcd getPcd2() { return pcd2; }
    public void setPcd2(Pcd _pcd2) {
        if (_pcd2 != null) _pcd2.setParent(this);
        pcd2 = _pcd2;
    }
    
    public BinaryPcd(SourceLocation location, Pcd _pcd1, Pcd _pcd2) {
        super(location);
        setPcd1(_pcd1);
        setPcd2(_pcd2);
    }
    protected BinaryPcd(SourceLocation source) {
        super(source);
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return pcd1;
        case 1: return pcd2;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "pcd1";
        case 1: return "pcd2";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setPcd1((Pcd)child); return;
        case 1: setPcd2((Pcd)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 2;
    }
    
    public String getDefaultDisplayName() {
        return "BinaryPcd()";
    }
    
    //END: Generated from @child and @property
}
