/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;
import org.aspectj.util.FuzzyBoolean;

/**
  * @grammar modifiers [declaringType].new(formals) throws
  * @child FormalsPattern formalsPattern
  * @child NameTypeDsPattern throwsPattern
  */
public class ConstructorPattern extends CodePattern {
    public String toShortString() {
        //XXX this ignores throws for now
        String mods = modifiers.toShortString();
        if (mods.length() > 0) mods += " ";
        mods += makeDeclaringTypeString() + "new" + formalsPattern.toShortString();
        if (throwsPattern != null) {
            mods += " throws ...";
        }

        return mods;
    }

    public String getLookupId() { return "new"; }

    protected boolean isStaticMatch(SemanticObject so) {
        return true;
    }
    
    public FuzzyBoolean matches(JoinPoint jp) {
        if (! (jp.getTargetSO() instanceof Constructor) ) {
            return FuzzyBoolean.NO;
        }
        
        Constructor m = (Constructor)jp.getTargetSO();
        
        if (!formalsPattern.matches(m.getFormals())) {
            return FuzzyBoolean.NO;
        }
        if (throwsPattern != null && !throwsPattern.matches(m.getThrows())) {
            return FuzzyBoolean.NO;
        }
        
        return super.matches(jp);
    }    
    
    //BEGIN: Generated from @child and @property
    protected FormalsPattern formalsPattern;
    public FormalsPattern getFormalsPattern() { return formalsPattern; }
    public void setFormalsPattern(FormalsPattern _formalsPattern) {
        if (_formalsPattern != null) _formalsPattern.setParent(this);
        formalsPattern = _formalsPattern;
    }
    
    protected NameTypeDsPattern throwsPattern;
    public NameTypeDsPattern getThrowsPattern() { return throwsPattern; }
    public void setThrowsPattern(NameTypeDsPattern _throwsPattern) {
        if (_throwsPattern != null) _throwsPattern.setParent(this);
        throwsPattern = _throwsPattern;
    }
    
    public ConstructorPattern(SourceLocation location, Modifiers _modifiers, GenTypeName _declaringTypeName, FormalsPattern _formalsPattern, NameTypeDsPattern _throwsPattern) {
        super(location, _modifiers, _declaringTypeName);
        setFormalsPattern(_formalsPattern);
        setThrowsPattern(_throwsPattern);
    }
    protected ConstructorPattern(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        ConstructorPattern ret = new ConstructorPattern(getSourceLocation());
        ret.preCopy(walker, this);
        if (modifiers != null) ret.setModifiers( (Modifiers)walker.process(modifiers) );
        if (declaringTypeName != null) ret.setDeclaringTypeName( (GenTypeName)walker.process(declaringTypeName) );
        if (formalsPattern != null) ret.setFormalsPattern( (FormalsPattern)walker.process(formalsPattern) );
        if (throwsPattern != null) ret.setThrowsPattern( (NameTypeDsPattern)walker.process(throwsPattern) );
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 2: return formalsPattern;
        case 3: return throwsPattern;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 2: return "formalsPattern";
        case 3: return "throwsPattern";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 2: setFormalsPattern((FormalsPattern)child); return;
        case 3: setThrowsPattern((NameTypeDsPattern)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 4;
    }
    
    public String getDefaultDisplayName() {
        return "ConstructorPattern()";
    }
    
    //END: Generated from @child and @property
}

