/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;
import org.aspectj.util.FuzzyBoolean;

/**
 * @grammar ..
 */

public class EllipsesFakeTypeName extends GenTypeName {
    public String toShortString() {
        return "..";
    }

    public boolean matches(Type type) {
        return false;
    }
    public FuzzyBoolean matchesInstance(Type type) {
        return FuzzyBoolean.NO;
    }
    
    public JpPlan makePlan(JoinPoint point, Expr expr) {
        return null;
    }    
    
    //BEGIN: Generated from @child and @property
    
    public EllipsesFakeTypeName(SourceLocation location) {
        super(location);
    
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        EllipsesFakeTypeName ret = new EllipsesFakeTypeName(getSourceLocation());
        ret.preCopy(walker, this);
    
        return ret;
    }
    
    
    public String getDefaultDisplayName() {
        return "EllipsesFakeTypeName()";
    }
    
    //END: Generated from @child and @property
}

