/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;
import org.aspectj.util.FuzzyBoolean;

/**
 * @grammar name1 && name2
 * @child GenTypeName name1
 * @child GenTypeName name2
 */

public class IntersectTypeName extends GenTypeName {
    public String toShortString() {
        return name1.toShortString() + " && " + name2.toShortString();
    }

    public boolean matches(Type type) {
        return name1.matches(type) && name2.matches(type);
    }
    
    public FuzzyBoolean matchesInstance(Type type) {
        return name1.matchesInstance(type).and(name2.matchesInstance(type));
    }

    public JpPlan makePlan(JoinPoint jp, Expr expr) {
        JpPlan plan1 = name1.makePlan(jp, expr);
        JpPlan plan2 = name2.makePlan(jp, expr);
        return plan1.and(plan2);
    }    
    
    //BEGIN: Generated from @child and @property
    protected GenTypeName name1;
    public GenTypeName getName1() { return name1; }
    public void setName1(GenTypeName _name1) {
        if (_name1 != null) _name1.setParent(this);
        name1 = _name1;
    }
    
    protected GenTypeName name2;
    public GenTypeName getName2() { return name2; }
    public void setName2(GenTypeName _name2) {
        if (_name2 != null) _name2.setParent(this);
        name2 = _name2;
    }
    
    public IntersectTypeName(SourceLocation location, GenTypeName _name1, GenTypeName _name2) {
        super(location);
        setName1(_name1);
        setName2(_name2);
    }
    protected IntersectTypeName(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        IntersectTypeName ret = new IntersectTypeName(getSourceLocation());
        ret.preCopy(walker, this);
        if (name1 != null) ret.setName1( (GenTypeName)walker.process(name1) );
        if (name2 != null) ret.setName2( (GenTypeName)walker.process(name2) );
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return name1;
        case 1: return name2;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "name1";
        case 1: return "name2";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setName1((GenTypeName)child); return;
        case 1: setName2((GenTypeName)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 2;
    }
    
    public String getDefaultDisplayName() {
        return "IntersectTypeName()";
    }
    
    //END: Generated from @child and @property
}

