/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.util.FuzzyBoolean;

import java.util.*;

/**
  * @grammar *(pattern)
  * @child DecPattern pattern
  */
public abstract class KindedPcd extends Pcd {
    public void checkStatic() { }
    public String toShortString() {
        return getKind() + "(" + getPattern().toShortString() + ")";
    }

    public abstract String getKind();
    public abstract int getJpKind();

    public JpPlanner makePlanner(PlanData planData) {
        return new JpPlanner() {
            public FuzzyBoolean fastMatch(JoinPoint jp) {
                if (!(jp.getTypeCode() == getJpKind())) return FuzzyBoolean.NO;
                
                
                return getPattern().matches(jp);
            }
        };
    }    
    
    //BEGIN: Generated from @child and @property
    protected DecPattern pattern;
    public DecPattern getPattern() { return pattern; }
    public void setPattern(DecPattern _pattern) {
        if (_pattern != null) _pattern.setParent(this);
        pattern = _pattern;
    }
    
    public KindedPcd(SourceLocation location, DecPattern _pattern) {
        super(location);
        setPattern(_pattern);
    }
    protected KindedPcd(SourceLocation source) {
        super(source);
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return pattern;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "pattern";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setPattern((DecPattern)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }
    
    public String getDefaultDisplayName() {
        return "KindedPcd()";
    }
    
    //END: Generated from @child and @property
}

