/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;

import org.aspectj.compiler.base.JavaCompiler;

/**
  * @grammar (!?public|private|...)*
  * @property int falseValue
  *
  */
public class ModifiersPattern extends Modifiers {
    //public ModifiersPattern(String[] labels, String[] falseLabels) {
    //    this(modifierValue(labels), modifierValue(falseLabels));
    //}

    //!!! strange definition of matching here...
    public boolean matches(Modifiers otherModifiers) {
        // first check that no false modifiers are defined
        if ((falseValue & otherModifiers.getValue()) != 0) return false;

        // then delegate to the super
        return super.matches(otherModifiers);
    }    
    
    //BEGIN: Generated from @child and @property
    protected int falseValue;
    public int getFalseValue() { return falseValue; }
    public void setFalseValue(int _falseValue) { falseValue = _falseValue; }
    
    public ModifiersPattern(SourceLocation location, int _value, int _falseValue) {
        super(location, _value);
        setFalseValue(_falseValue);
    }
    protected ModifiersPattern(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        ModifiersPattern ret = new ModifiersPattern(getSourceLocation());
        ret.preCopy(walker, this);
        ret.value = value;
        ret.falseValue = falseValue;
        return ret;
    }
    
    
    public String getDefaultDisplayName() {
        return "ModifiersPattern(value: "+value+", "+"falseValue: "+falseValue+")";
    }
    
    //END: Generated from @child and @property
}
