/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;


import java.util.*;

import org.aspectj.compiler.base.JavaCompiler;

/**
 * A Pcd is a filter that matches JoinPoints.  For JoinPoints that it matches
 * it will produce a plan of how to apply relevant advice to that JoinPoint.
 */
//XXX This extends Expr so I can use the ExprParser to parse them
//XXX This is a flaw in the design of my JavaParser class
abstract public class Pcd extends Expr {

    public Pcd(SourceLocation source) {
        super(source);
    }

    //XXX see comment above for why this is messed up
    protected Type discoverType() { return null; }

    abstract public String toShortString();

    /**
     * Build a partial plan for this pointcut on the given joinPoint in the
     * context of the planData.
     *
     * @returns null if there is no plan
     */
    abstract public JpPlanner makePlanner(PlanData planData);

    /**
     *  In AspectJ 1.0 only target, this, and args allow name binding.
     */
    public boolean allowsNameBinding() { return false; }
    
    /**
     * This should call "showNonStaticError()" for any Pcd that can not be used
     * for purely static matching.  This is used by 'declare error/warning' and
     * 'declare soft'.
     */
    public abstract void checkStatic();
    
    protected void showNonStaticError() {
        showError("dynamic matching PCD not allowed in this context");
    }
    
    // this is here to give Crista better warnings
//    public boolean isAbstract(PlanData planData) {
//        return false;
//    }
}
