/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;


import java.util.*;

import org.aspectj.util.*;

/**
 * @grammar eachJVM()
 */

public class PerSingleton extends PerClause {
    FieldDec aspectField;
    
    public String toShortString() { return "issingleton"; }
    
    public JpPlanner makeInnerPlanner(PlanData planData) {
        return new JpPlanner() {
            public FuzzyBoolean fastMatch(JoinPoint jp) {
                return FuzzyBoolean.MAYBE;
            }
            public JpPlan makePlan(JoinPoint jp) {
                JpPlan plan = new JpPlan(jp);
                plan.test = maybeMakeTest(jp);
                plan.setInstanceExpr(makeAspectInstance());
                return plan;
            }
        };
    }
    
    public JpPlanner makeInitializerPlanner(PlanData planData) {
        // no initializers here
        return JpPlanner.NO_PLANNER;
    }

    String makeAspectFieldName() {
        return getAST().makeGeneratedName("aspectInstance");
    }

    FieldDec makeAspectField() {
        final AST ast = getAST();
        FieldDec ret = ast.makeField(
            ast.makeModifiers(Modifiers.PUBLIC | Modifiers.FINAL | Modifiers.STATIC),
            getAspectType(), makeAspectFieldName(), ast.makeNew(getAspectType()));
        //ret.setSynthetic();
        return ret;
    }

    //XXX this only works most of the time
    Expr maybeMakeTest(JoinPoint point) {
        final AST ast = getAST();
        
        if (getAspectType().isSubtypeOf(point.getBytecodeType())) {
            return ast.makeCall(hasAspectMethod, ast.makeTypeExpr(getAspectType()));
        } else {
            return null;
        }
    }

    protected MethodDec makeAspectOfMethod() {
        /* public static A aspectOf() {
               return aspect$;
           }
        */
        final AST ast = getAST();
        BlockStmt body = ast.makeBlock(ast.makeReturn(makeAspectInstance()));
        return makeAspectOfMethod(ast.makeFormals(), body);
    }


    protected MethodDec makeHasAspectMethod() {
        /* public static A hasAspect() {
               return aspect$ != null;
           }
        */
        final AST ast = getAST();
        BlockStmt body = ast.makeBlock(ast.makeReturn(
            ast.makeBinop("!=", makeAspectInstance(), ast.makeNull())));
        return makeHasAspectMethod(ast.makeFormals(), body);
    }


    Expr makeAspectInstance() {
        return getAST().makeGet(aspectField);
    }

    public Expr makeAspectOfExpr(Expr fromObject) { /* A.aspectOf() */
        return getAST().makeCall(aspectOfMethod, getAST().makeTypeExpr(getAspectType()));
    }


    public void setupAspect() {
        TypeDec typeDec = getAspectDec();
        aspectField = makeAspectField();
        typeDec.addToBody(aspectField);
        super.setupAspect();
    }    
    
    //BEGIN: Generated from @child and @property
    
    public PerSingleton(SourceLocation location) {
        super(location);
    
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        PerSingleton ret = new PerSingleton(getSourceLocation());
        ret.preCopy(walker, this);
    
        return ret;
    }
    
    
    public String getDefaultDisplayName() {
        return "PerSingleton()";
    }
    
    //END: Generated from @child and @property
}

