/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.joinpoints;

import org.aspectj.compiler.crosscuts.ast.*;
import org.aspectj.compiler.base.ast.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;

/**
 */
public class CFlowEntryPlan extends JpPlan {
    private CFlowPlanner cflowPlanner;
    private JpPlan plan;

    public CFlowEntryPlan(JpPlan innerPlan, CFlowPlanner cflowPlanner) {
        super(innerPlan.joinPoint);

        this.cflowPlanner = cflowPlanner;
        plan = innerPlan;
    }

    public int getPreSortOrder() {
        if (cflowPlanner.getIncludesRoot()) return CFLOWENTRY_ROOT;
        else return CFLOWENTRY_NOROOT;
    }

    /**
     * These plans are ordered based on dependencies, i.e.
     *   eachcflow(P && !cflow(P))
     * The ordering will make sure that the plan for cflow(P) is run before the
     * plan for the eachcflow in order to generate the correct behavior.
     * 
     * This is the only case where plans are ordered based on internal dependencies.
     */
    public int compareTo(Object o) {
        int ret = super.compareTo(o);
        if (ret != 0) return ret;
           
        if (! (o instanceof CFlowEntryPlan) ) return 0;
        
        CFlowEntryPlan p = (CFlowEntryPlan)o;
        if (cflowPlanner.getIncludesRoot() != p.cflowPlanner.getIncludesRoot()) return 0;
     
        // now check to see if there's a dependency
        if (plan.dependsOn(p.cflowPlanner)) {
            //System.out.println("depends");
            if (p.cflowPlanner.getIncludesRoot()) return +1;
            else return -1;
        } else if (p.plan.dependsOn(cflowPlanner)) {
            //System.out.println("depends");
            if (cflowPlanner.getIncludesRoot()) return -1;
            else return +1;
        } else {
            //System.out.println("no depends");
            return 0;
        }
    }

    
    public void wrapJoinPoint(JoinPoint jp) {
        jp.setStmts(wrapPushPop(jp.getStmts()));
    }
    
    public Stmts wrapPushPop(Stmts stmts) {
        return cflowPlanner.wrapPushPop(plan, stmts);
    }    
    
    public String toString() {
        return "cflow" + super.toString();
    }
}
