/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
package org.aspectj.compiler.crosscuts.joinpoints;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.ast.*;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.util.JavaStrings;

import java.util.*;


public abstract class CallJp extends ExprJp {
    private Formals codeFormals = null;
    protected Exprs argValues;
    protected Exprs innerArgs;

    public CallJp(AnyCallExpr callExpr, JoinPoint enclosingJp) {
        super(callExpr, enclosingJp); 
    }

    public Dec getTargetDec() { return getCalledCodeDec(); }

    public AnyCallExpr getCallExpr() { return (AnyCallExpr)expr; }

    public Exprs makeArgsExprs() {
        getCodeFormals();
        return (Exprs)CopyWalker.copy(argValues); //getArgsFormals().makeExprs();
    }
    
    public abstract Type getTargetType();
    public abstract Type getTargetExprType();
    public abstract boolean isMethodCall();

    public CodeDec getCalledCodeDec() {
        return getCallExpr().getCodeDec();
    }

    public boolean canThrow(Type t) { return getCalledCodeDec().canThrow(t); }
    public Collection getPossibleCheckedExceptions() {
    	return getCalledCodeDec().getPossibleCheckedExceptions();
    }

    public Formals getCodeFormals() {
        if (codeFormals == null) {
            final AST ast = getAST();
            codeFormals = ast.makeFormals();
            argValues = ast.makeExprs();
            innerArgs = ast.makeExprs();

            Formals calledFormals = getCalledCodeDec().getFormals();
            final int N = calledFormals.size();
            for (int i = 0; i < N; i++) {
                FormalDec formal =
                        ast.makeFinalFormal(calledFormals.get(i).getType(),
                                            calledFormals.get(i).getId());
                argValues.add(ast.makeVar(formal));
                codeFormals.add(formal);
                innerArgs.add(ast.makeVar(formal));
            }
        }
        return codeFormals;
    }

    protected abstract ASTObject makeInnerCall(Expr targetExpr, Exprs argsExprs);

    protected AnonymousMethodExpr makeSyntheticCall() {
        return makeAnonMethodExpr(getCodeFormals(), getCallExpr().getExpr(), 
                                  getCallExpr().getArgs());
    }
}
