/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.joinpoints;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.*;

public abstract class CodeBodyJp extends JoinPoint {
    private JoinPoint enclosingJoinPoint;

    public CodeBodyJp(JoinPoint enclosingJoinPoint) {
        super(enclosingJoinPoint.getCompiler());
        this.enclosingJoinPoint = enclosingJoinPoint;
    }

    public Expr makeStaticEnclosingExecutionJoinPointExpr() {
        if (getEnclosingExecutionJoinPoint() == null) return getAST().makeNull();
        
        return getEnclosingExecutionJoinPoint().makeStaticJoinPointVarExpr();
    }

    public JoinPoint getEnclosingExecutionJoinPoint() {
        return enclosingJoinPoint;
    }

    
    //XXX move most of the default behavior in JoinPoin/Jp to here
    public abstract ASTObject getSourceLocation();

    public boolean isStaticContext() { return enclosingJoinPoint.isStaticContext(); }

    public abstract String toString();

    public void makeCorrespondences() {
        if (plans == null) return;
        super.makeCorrespondences();
        
        Dec enclosingDec = getSourceLocation().getEnclosingCodeDec();
        if (!enclosingDec.fromSource() || enclosingDec instanceof InitializerDec) {
        	enclosingDec = enclosingDec.getLexicalType().getTypeDec();
        }
        
        //System.out.println(this + ": " + enclosingDec);

        ((AspectJCompiler)getCompiler()).getCorrespondences().addPointsTo(
            enclosingDec, getSourceLocation());
    }
}
