/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.base;

import java.util.*;

/**
 * Checker.java
 *
 *
 * Created: Wed Sep 27 16:47:49 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public class Checker  {

    private Debugger debugger;

    public Checker(Debugger debugger) {
        this.debugger = debugger;
    }

    public boolean check() {
        return
            checkJavaVersion()        &
            checkRuntimeEnvironment() ;
    }

    protected boolean checkJavaVersion() {
        return
               checkClass("com.sun.jdi.VirtualMachine",
                         "tools.jar required to run ajdb")
            && checkClass("com.sun.jdi.event.LocatableEvent",
                          "tools.jar required to run ajdb")
            && checkClass("com.sun.jdi.VMDisconnectedException",
                          "JDK 1.3 or later tools.jar required to run ajdb")
            ;
    }

    protected boolean checkClass(String className, String msg) {
        try {
            Class c = Class.forName(className);
        } catch (ClassNotFoundException cnfe) {
            debugger.fatal(msg);
            return false;
        }
        return true;
    }

    protected boolean checkRuntimeEnvironment() {
        return true;
    }
}
