/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */
package org.aspectj.debugger.base;

import org.aspectj.debugger.request.*;
import com.sun.jdi.*;
import com.sun.jdi.request.*;
import java.util.*;
import java.io.File;

public interface CommandReceiver {

    public Object runCommand(Options opts)
        throws NoVMException, DebuggerException;
    public Object runCommand(String className, String exArgs)
        throws NoVMException, DebuggerException;
    public Object runCommand(String className, String vmArgs, String exArgs,
                             boolean isSuspended, int debugTraceMode)
        throws NoVMException, DebuggerException;
    public EventRequest clearAtCommand(String className, int line)
        throws NoVMException, DebuggerException;
    public EventRequest clearInCommand(String className, String methodProto)
        throws NoVMException, DebuggerException;
    public EventRequest clearOnCommand(String sourceName, int line)
        throws NoVMException, DebuggerException;
    public List clearCommand()
        throws NoVMException, DebuggerException;
    public List clearAllCommand()
        throws NoVMException, DebuggerException;
    public EventRequest stopAtCommand(String className, int line)
        throws NoVMException, DebuggerException;
    public EventRequest stopInCommand(String className, String methodProto)
        throws NoVMException, DebuggerException;
    public EventRequest stopOnCommand(String sourceName, int line)
        throws NoVMException, DebuggerException;
    public List stopCommand()
        throws NoVMException, DebuggerException;
    public Object contCommand()
        throws NoVMException, DebuggerException;
    public List threadsCommand(String threadGroupName)
        throws NoVMException, DebuggerException;
    public List threadGroupsCommand()
        throws NoVMException, DebuggerException;
    public List fieldsCommand(String className)
        throws NoVMException, DebuggerException;
    public List methodsCommand(String className)
        throws NoVMException, DebuggerException;
    public List classesCommand()
        throws NoVMException, DebuggerException;
    public ReferenceType classCommand(String className)
        throws NoVMException, DebuggerException;
    public ThreadReference threadCommand(String threadName)
        throws NoVMException, DebuggerException;
    public ThreadGroupReference threadGroupCommand(String threadGroupName)
        throws NoVMException, DebuggerException;
    public List suspendCommand(List threadNames)
        throws NoVMException, DebuggerException;
    public List resumeCommand(List threadNames)
        throws NoVMException, DebuggerException;
    public List whereCommand(String threadName)
        throws NoVMException, DebuggerException;
    public List localsCommand()
        throws NoVMException, DebuggerException;
    public ThreadReference interruptCommand(String threadName)
        throws NoVMException, DebuggerException;
    public StackFrame upCommand(int frames)
        throws NoVMException, DebuggerException;
    public StackFrame downCommand(int frames)
        throws NoVMException, DebuggerException;
    public StepRequest stepCommand()
        throws NoVMException, DebuggerException;
    public StepRequest stepUpCommand()
        throws NoVMException, DebuggerException;
    public StepRequest stepiCommand()
        throws NoVMException, DebuggerException;
    public StepRequest nextCommand()
        throws NoVMException, DebuggerException;
    public Value printCommand(Object valueRep)
        throws NoVMException, DebuggerException;

    public Value dumpCommand(Object valueRep)
        throws NoVMException, DebuggerException;
    public Value evalCommand(Object valueRep)
        throws NoVMException, DebuggerException;
    public Value setCommand(Object lvalue, Object rvalue)
        throws NoVMException, DebuggerException;
    public ClasspathRequest.Package classpathCommand()
        throws NoVMException, DebuggerException;
    public LockInformation lockCommand(Object valueRep)
        throws NoVMException, DebuggerException;
    public ThreadLockInformation threadlocksCommand(String threadName)
        throws NoVMException, DebuggerException;
    public WatchpointRequest watchAccessCommand(String className,
                                                String fieldName)
        throws NoVMException, DebuggerException;
    public WatchpointRequest watchAllCommand(String className,
                                             String fieldName)
        throws NoVMException, DebuggerException;
    public WatchpointRequest unwatchAccessCommand(String className,
                                                  String fieldName)
        throws NoVMException, DebuggerException;
    public WatchpointRequest unwatchAllCommand(String className,
                                               String fieldName)
        throws NoVMException, DebuggerException;
    public ExceptionRequest catchCommand(String className)
        throws NoVMException, DebuggerException;
    public ExceptionRequest ignoreCommand(String className)
        throws NoVMException, DebuggerException;
    public TraceMethodsRequest.EntryExitPair traceMethodsCommand(String threadName)
        throws NoVMException, DebuggerException;
    public UntraceMethodsRequest.EntryExitPair untraceMethodsCommand(String threadName)
        throws NoVMException, DebuggerException;
    public List excludeCommand(List classNames)
        throws NoVMException, DebuggerException;
    public Object quitCommand()
        throws NoVMException, DebuggerException;
    public Object helpCommand()
        throws NoVMException, DebuggerException;
    public Object versionCommand()
        throws NoVMException, DebuggerException;
    public ThreadReference killCommand(String threadName, String valueRep)
        throws NoVMException, DebuggerException;
    public String useCommand(String sourcePath)
        throws NoVMException, DebuggerException;
    public String workingdirCommand(String workingdirPath)
        throws NoVMException, DebuggerException;    
    public List listCommand()
        throws NoVMException, DebuggerException;
    public List listCommand(String sourceName)
        throws NoVMException, DebuggerException;
    public SourceManager.SourceLine listCommand(String sourceName, int lineNumber)
        throws NoVMException, DebuggerException;
    public List listCommand(String sourceName, int startLine, int endLine)
        throws NoVMException, DebuggerException;
    public String monitorCommand(String command)
        throws NoVMException, DebuggerException;
    public List monitorCommand()
        throws NoVMException, DebuggerException;
    public MonitorRequest unmonitorCommand(int number)
        throws NoVMException, DebuggerException;
    public List readCommand(String sourceName)
        throws NoVMException, DebuggerException;
    public List readCommand(String sourceName, boolean silent)
        throws NoVMException, DebuggerException;
    public Object bangBangCommand(String lastCommand)
        throws NoVMException, DebuggerException;
    public Object importCommand(String importPath)
        throws NoVMException, DebuggerException;
    public Object deportCommand(String importPath)
        throws NoVMException, DebuggerException;
    public List viewCommand(String source)
        throws NoVMException, DebuggerException;
    public String tostringCommand()
        throws NoVMException, DebuggerException;
    public VirtualMachine connectCommand()
        throws NoVMException, DebuggerException;
    public File pwdCommand()
        throws NoVMException, DebuggerException;
    public List lsCommand(String fileName)
        throws NoVMException, DebuggerException;    

    public Options getOptions();
}
