/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.base;

import org.aspectj.debugger.request.*;
import com.sun.jdi.*;
import com.sun.jdi.event.*;
import com.sun.jdi.request.*;
import java.io.*;
import java.util.*;

/**
 * DebuggerApp.java
 *
 *
 * Created: Fri Aug 25 13:53:39 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public interface DebuggerApp  {
    public void runCommand(String className, String exArgs)
        throws NoVMException, DebuggerException;
    public void runCommand(String className, String vmArgs, String exArgs,
                           boolean isSuspended, int debugTraceMode)
        throws NoVMException, DebuggerException;
    public void clearAtCommand(String className, int line, EventRequest request)
        throws NoVMException, DebuggerException;
    public void clearInCommand(String className, String methodProto, EventRequest request)
        throws NoVMException, DebuggerException;
    public void clearOnCommand(String sourceName, int line, EventRequest request)
        throws NoVMException, DebuggerException;
    public void clearCommand(List breakpoints)
        throws NoVMException, DebuggerException;
    public void clearAllCommand(List breakpoints)
        throws NoVMException, DebuggerException;
    public void stopAtCommand(String className, int line, EventRequest request)
        throws NoVMException, DebuggerException;
    public void stopInCommand(String className, String methodProto, EventRequest request)
        throws NoVMException, DebuggerException;
    public void stopOnCommand(String sourceName, int line, EventRequest request)
        throws NoVMException, DebuggerException;
    public void stopCommand(List breakpoints)
        throws NoVMException, DebuggerException;
    public void contCommand()
        throws NoVMException, DebuggerException;
    public void threadsCommand(String threadGroupName, List threads)
        throws NoVMException, DebuggerException;
    public void threadGroupsCommand(List threadGroups)
        throws NoVMException, DebuggerException;
    public void fieldsCommand(String className, List fields)
        throws NoVMException, DebuggerException;
    public void methodsCommand(String className, List methods)
        throws NoVMException, DebuggerException;
    public void classesCommand(List classes)
        throws NoVMException, DebuggerException;
    public void classCommand(String className, ReferenceType refType)
        throws NoVMException, DebuggerException;
    public void threadCommand(String threadName, ThreadReference threadRef)
        throws NoVMException, DebuggerException;
    public void threadGroupCommand(String threadGroupName, ThreadGroupReference threadGroupRef)
        throws NoVMException, DebuggerException;
    public void suspendCommand(List threadNames, List threads)
        throws NoVMException, DebuggerException;
    public void resumeCommand(List threadNames, List threads)
        throws NoVMException, DebuggerException;
    public void whereCommand(String threadName, List frames)
        throws NoVMException, DebuggerException;
    public void localsCommand(List locals)
        throws NoVMException, DebuggerException;
    public void interruptCommand(String threadName, ThreadReference threadRef)
        throws NoVMException, DebuggerException;
    public void upCommand(int frames, StackFrame currentFrame)
        throws NoVMException, DebuggerException;
    public void downCommand(int frames, StackFrame currentFrame)
        throws NoVMException, DebuggerException;
    public void stepCommand(StepRequest request)
        throws NoVMException, DebuggerException;
    public void stepUpCommand(StepRequest request)
        throws NoVMException, DebuggerException;
    public void stepiCommand(StepRequest request)
        throws NoVMException, DebuggerException;
    public void nextCommand(StepRequest request)
        throws NoVMException, DebuggerException;
    public void printCommand(Object valueRep, Value value)
        throws NoVMException, DebuggerException;
    public void dumpCommand(Object valueRep, Value value)
        throws NoVMException, DebuggerException;
    public void evalCommand(Object valueRep, Value value)
        throws NoVMException, DebuggerException;
    public void setCommand(Object lvalue, Object rvalue, Value oldValue, Value newValue)
        throws NoVMException, DebuggerException;
    public void classpathCommand(String baseDirectory, List paths)
        throws NoVMException, DebuggerException;
    public void lockCommand(Object valueRep, LockInformation lockInfo)
        throws NoVMException, DebuggerException;
    public void threadlocksCommand(String threadName, ThreadLockInformation threadLockInfo)
        throws NoVMException, DebuggerException;
    public void watchAccessCommand(String className, String fieldName, WatchpointRequest request)
        throws NoVMException, DebuggerException;
    public void  watchAllCommand(String className, String fieldName, WatchpointRequest request)
        throws NoVMException, DebuggerException;
    public void unwatchAccessCommand(String className, String fieldName, WatchpointRequest request)
        throws NoVMException, DebuggerException;
    public void unwatchAllCommand(String className, String fieldName, WatchpointRequest request)
        throws NoVMException, DebuggerException;
    public void catchCommand(String className, ExceptionRequest request)
        throws NoVMException, DebuggerException;
    public void ignoreCommand(String className, ExceptionRequest request)
        throws NoVMException, DebuggerException;
    public void traceMethodsCommand(String threadName, TraceMethodsRequest.EntryExitPair pair)
        throws NoVMException, DebuggerException;
    public void untraceMethodsCommand(String threadName, UntraceMethodsRequest.EntryExitPair pair)
        throws NoVMException, DebuggerException;
    public void excludeCommand(List classNames, List classes)
        throws NoVMException, DebuggerException;
    public void quitCommand()
        throws NoVMException, DebuggerException;
    public void helpCommand(String helpString)
        throws NoVMException, DebuggerException;
    public void versionCommand(Object version)
        throws NoVMException, DebuggerException;
    public void killCommand(String threadName, String valueRep, ThreadReference threadRef)
        throws NoVMException, DebuggerException;
    public void useCommand(String sourcePath, String newSourcePath)
        throws NoVMException, DebuggerException;
    public void workingdirCommand(String workingdirPath, String newWorkingdirPath)
        throws NoVMException, DebuggerException;    
    public void listCommand(List sourceLines)
        throws NoVMException, DebuggerException;
    public void listCommand(String sourceName,  List sourceLines)
        throws NoVMException, DebuggerException;
    public void listCommand(String sourceName, int lineNumber, SourceManager.SourceLine sl)
        throws NoVMException, DebuggerException;
    public void listCommand(String sourceName, int startLine, int endLine, List sourceLines)
        throws NoVMException, DebuggerException;
    public void monitorCommand(String command, String result)
        throws NoVMException, DebuggerException;
    public void monitorCommand(List monitors)
        throws NoVMException, DebuggerException;
    public void unmonitorCommand(int number, MonitorRequest request)
        throws NoVMException, DebuggerException;
    public void readCommand(String fileName, List commands)
        throws NoVMException, DebuggerException;
    public void viewCommand(String sourceName, List lines)
        throws NoVMException, DebuggerException;
    public void tostringCommand(String tostring)
        throws NoVMException, DebuggerException;
    public void connectCommand(VirtualMachine vm)
        throws NoVMException, DebuggerException;
    public void pwdCommand(File file)
        throws NoVMException, DebuggerException;
    public void lsCommand(String dir, List files)
        throws NoVMException, DebuggerException;    
    


    public void handleParseException(/*Parser.*/ParseException e);
    public void handleNoVMException(NoVMException e);
    public void handleDebuggerException(DebuggerException e);
    public void handleVMDisconnectedException(VMDisconnectedException e);
    public void handleInternalException(Throwable t);

    public boolean wantsToExit();
    public boolean canRestart();

    public PrintStream getOutputStream();
    public void outln(Object o);
    public void log(Object o);

    //public VirtualMachine findVM(String vmArgs, String commandLine);
    public VirtualMachine connect(String vmArgs, String className, String commandLine);
}
