/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.base;

import java.util.*;

/**
 * Options.java
 *
 *
 * Created: Thu Sep 28 09:31:33 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public class Options  {

    public Options() {
        set("extra");
    }

    public final static String helpString =
        "Usage: ajdb <options> <class> <arguments>\n" +
        "\n" +
        "where options include:\n" +
        "    -help             print out this message and exit\n" +
        "    -sourcepath <directory>\n" +
        "                      directory in which to look for source files\n" +
//          "    -attach <address>\n" +
//          "                      attach to a running VM at the specified\n" +
//          "                      address using standard connector\n" +
//          "    -listen <address>\n" +
//          "                      wait for a running VM to connect at the\n" +
//          "                      specified address using standard connector\n" +
//          "    -listenany\n" +
//          "                      wait for a running VM to connect at any available\n" +
//          "                      address using standard connector\n" +
//          "    -launch\n" +
//          "                      launch VM immediately instead of waiting for 'run' command\n" +
//          "    -connect <connector-name>:<name1>=<value1>,...\n" +
//          "                      connect to target VM using named connector with\n" +
//          "                      listed argument values\n" +
//          "    -dbgtrace [flags] print info for debugging jdb\n" +
//          "    -thotspot         run the application in the Hotspot(tm) Performance Engine\n" +
//          "    -tclassic         run the application in the Classic VM\n" +
        "    -read <file>      read the startup file\n" +
        "    -gui              start up in GUI mode\n" +
        "    -extra            enable extra features unique to ajdb\n" +
        "    -workingdir       set the user's workingdir" +
        "\n" +
        "options forwarded to debuggee process:\n" +
        "    -v -verbose[:class|gc|jni]\n" +
        "                      turn on verbose mode\n" +
        "    -D<name>=<value>  set a system property\n" +
        "    -classpath <directories separated by \";\">\n" +
        "                      list directories in which to look for classes\n" +
        "    -X<option>        non-standard target VM option\n" +
        "\n" +
        "<class> is the name of the class to begin debugging\n" +
        "<arguments> are the arguments passed to the main() method of <class>\n" +
        "\n" +
        "For command help type 'help' at jdb prompt\n" +
        "";
    
    public final static String helpString() {
        return helpString;
    }
    
    private HashMap opts = new HashMap();

    private String className = "";
    public String getClassName() {
        return className;
    }

    private String commandLine = "";
    public String getCommandLine() {
        return commandLine;
    }

    public String getFullCommandLine() {
        String line = "";
        if (!empty(className)) {
            line += className;
        }
        if (!empty(commandLine)) {
            line += " " + commandLine;
        }
        line = line.trim();
        System.out.println("line="+  line);
        return line;
    }

    private static boolean empty(String str) {
        return str == null || "".equals(str.trim());
    }

    public final static String[] javaArgs = {
        "classpath",
        "v",
        "verbose",
        "verbose:class",
        "verbose:gc",
        "verbose:jni",
        "X",
        "D",
    };

    public void fill(String str) {
        Vector v = new Vector();
        StringTokenizer tok = new StringTokenizer(str);
        String t;
        while (tok.hasMoreTokens()) {
            t = tok.nextToken();
            if (t.startsWith("\"")) {
                String end;
                boolean done = t.endsWith("\"");
                while(!done && tok.hasMoreTokens()) {
                    t += (end = tok.nextToken().trim());
                    if (end.endsWith("\"")) {
                        done = true;
                    }
                }
            }
            v.add(t);
        }
        String[] args = new String[v.size()];
        Iterator iter = v.iterator();
        for (int i = 0; i < args.length; i++) {
            args[i] = v.get(i) + "";
        }
        fill(args);
        
    }

    public void fill(String[] args) {
        realFill(removeNoWarn(args));
        Debug.setOptions(this);
    }

    private String[] removeNoWarn(String[] args) {
        for (int i = 0; i < args.length; i++) {
            String opt = args[i];
            if (opt(opt, "nowarn")) {
                String[] newArgs = new String[args.length - 1];
                System.arraycopy(args, 0, newArgs, 0, i);
                System.arraycopy(args, i+1, newArgs, i, args.length-i-1);
                set("nowarn");
                return newArgs;
            }
        }
        return args;
    }

    private void realFill(String[] args) {
        int i = 0;
        String opt = "";
        String arg = "";
        while (i < args.length) {
            opt = args[i++];
            if (opt(opt, "extra")) {
                set("extra");
            } else if (opt(opt, "now")) {
                set("now");
            } else if (opt(opt, "noextra")) {
                unset("extra");
            } else if (opt(opt, "nowarn")) {
                set("nowarn");
            } else if (opt(opt, "help")) {
                help();
            } else if (opt(opt, "sourcepath")) {
                check(args, i, "-sourcepath");
                set("sourcepath", args[i++]);
            } else if (opt(opt, "attach")) {
                check(args, i, "-attach");
                set("attach", args[i++]);
            } else if (opt(opt, "listen")) {
                check(args, i, "-listen");
                set("listen", args[i++]);
            } else if (opt(opt, "listenany")) {
                set("listenany");
            } else if (opt(opt, "launch")) {
                set("launch");
            } else if (opt(opt, "connect")) {
                check(args, i, "-connect");
                set("connect", args[i++]);
            } else if (opt(opt, "dbgtrace")) {
                if (isArg(args, i+1)) {
                    set("dbgtrace", args[i++]);
                } else {
                    set("dbgtrace");
                }
            } else if (opt(opt, "thotspot")) {
                set("thotspot");
            } else if (opt(opt, "tclassic")) {
                set("tclassic");
            } else if (opt(opt, "v")) {
                set("v");
            } else if (opt(opt, "verbose")) {
                set("verbose");
            } else if (opt(opt, "verbose:class")) {
                set("verbose:class");
            } else if (opt(opt, "verbose:gc")) {
                set("verbose:gc");
            } else if (opt(opt, "verbose:jni")) {
                set("verbose:jni");
            } else if (opt(opt, "demo")) {
                set("demo");
                Debugger.demoMode = true;
            } else if (opt(opt, "classpath")) {
                check(args, i, "-classpath");
                set("classpath", args[i++]);
            } else if (opt(opt, "workingdir")) {
                check(args, i, "-workingdir");
                set("workingdir", args[i++]);
            } else if (opt(opt, "read")) {
                check(args, i, "-read");
                set("read", args[i++]);
            } else if (opt(opt, "debug")) {
                set("debug");
            } else if (opt.startsWith("-D")) {
                arg = strip("-D", opt, "<name>=<value>");
                if (arg.indexOf("\"") != -1) {
                    while (i < args.length) {
                        String next = args[i++];
                        arg += " " + next;
                        if (next.endsWith("\"")) {
                            break;
                        }
                    }
                }
                set("D", arg); 
                // fyi D, X resolve to setListOption(String, String)
            } else if (opt.startsWith("-X")) {
                arg = strip("-X", opt, "<option>");
                set("X", arg);
            } else if (isArg(opt)) {
                warn(opt + " is not a valid option");
                set(deopt(opt));
            } else {
                className = opt;
                //commandLine = opt;
                while (i < args.length) {
                    commandLine += " " + args[i++];
                }
            }
        }
    }

    private boolean opt(String opt, String str) {
        return opt.equals("-" + str) || opt.equals("--" + str);
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public void setMainClass(String mainClass) {
        setClassName(mainClass);
    }

    public void setClassPath(String classPath) {
        set("classpath", classPath);
    }

    public void setSourcePath(String sourcePath) {
        set("sourcepath", sourcePath);
    }

    public String getJavaArgs() {
        String args = "";
        if (isSet("thotspot")) {
            args += "-hotspot";
        } else if (isSet("tclassic")) {
            args += "-classic";
        }
        for (int i = 0; i < javaArgs.length; i++) {
            if (isSet(javaArgs[i])) {
                args += " " + format(javaArgs[i]);
            }
        }
        return args.trim();
    }

    /**
     * warning: may not work with List-based options
     */
    public String deopt(String opt) {
        for (int i = 0; i < opt.length(); i++) {
            if (opt.charAt(i) != '-' && opt.length() >= i) {
                return opt.substring(i);
            }
        }
        return opt;
    }

    public String opt(String opt) {
        return "-" + opt;
    }

    /**
     * Return a String with the formatted version of the option.
     * The result may have spaces and may resolve to multiple
     * options if opt (the option key) refers to a list.
     * @param opt the String rendition of the option key to format
     * @return a String with the formatted value for the option key.
     */
    public String format(String opt) {
        StringBuffer sb = new StringBuffer();
        if (isSet(opt)) {
            if (isListOption(opt)) {
                List list = (List) get(opt);
                if (null == list) {
                    warn(opt + " set but no list");
                } else {
                    for (Iterator it = list.iterator(); it.hasNext();) {
                        sb.append(" -" + opt + it.next());
                    }
                }
            } else {
                sb.append(opt(opt));
                String value = getOpt(opt);
                if ((null != value) && (0 < value.length())) {
                    sb.append(" ");
                    sb.append(value);
                }
            }
        }
        return sb.toString();
    }

    public boolean isSet(String opt) {
        return (get(opt) != null);
    }

    /**
     * Set the value of an option.
     * If the option is a list option, this resolves to
     * <code>setListOption(opt, arg)</code>.
     * If the option has been set and is not a list option, 
     * it will be overwritten (and a warning issued).
     */
    public void set(String opt, String arg) {
        if (isListOption(opt)) {
            setListOption(opt, arg);
        } else {
            if (hasWhitespace(arg)) {
                arg = quote(arg);
            }
            Object value = get(opt);
            if ((value != null) && (!("debugging-for-jeff".equals(value)))) {
                warn("Resetting option '" + opt + "' to '" + arg + "'.");
            }
            opts.put(opt, arg);
        }
    }

    /** 
     * @param opt the String (case-sensitive) referring to the option key
     *            (exclusive of any leading '-').
     * @return true if opt (the option key) refers to an option 
     *         whose values are represented in a List
     */
    public boolean isListOption(String opt) {
        return ("D".equals(opt) || "X".equals(opt));
    }

    /**
     * Set the value of an option represented as a List.
     * (Does nothing but warn if opt key is not a list option.)
     * @param opt the String key of the option
     * @param arg the String value to set, taken as one instance for the List.
     */
    public void setListOption(String opt, String arg) {
        if (!isListOption(opt)) {
            String args = "(\"" + opt + "\", \"" + arg + "\")";
            warn("Options.setListOption" + args + ": not a list option");
            // todo: call set(opt, arg) instead of returning?
            return;
        }
        if (hasWhitespace(arg)) {
            arg = quote(arg);
        }
        Object value = opts.get(opt);
        if (null == value) {
            value = new ArrayList();
            opts.put(opt, value);
        }
        List list = (List) value;
        list.add(arg);
    }

    public void set(String opt) {
        set(opt, "");
    }

    public void unset(String opt) {
        if (isListOption(opt)) {
            warn("Options.unset(\"" + opt + "\"): unsetting list");
        }
        set(opt, null);
    }

    public String quote(String arg) {
        return "\"" + arg + "\"";
    }

    public boolean hasWhitespace(String str) {
        for (int i = 0; i < str.length(); i++) {
            if (Character.isWhitespace(str.charAt(i))) {
                return true;
            }
        }
        return false;
    }

    public Object get(String opt) {
        return opts.get(opt);
    }

    public String getOpt(String opt) {
        return get(opt) + "";
    }


    public String toString() {
        return opts + "";
    }

    String strip(String start, String opt, String syntax) {
        int pos = start.length();
        String str = "";
        if (opt.length() < pos) {
            fail("Correct syntax: " + opt + syntax);
        } else {
            str = opt.substring(pos);
        }
        return str;
    }

    boolean isArg(String arg) {
        return (arg != null && arg.startsWith("-"));
    }

    boolean isArg(String[] args, int i) {
        return (args != null && isArg(args[i]));
    }

    void check(String[] args, int i, String s) {
        if (args == null     ||
            args.length <= i ||
            args[i] == null  ||
            isArg(args[i])) {
            fail("The option '" + s + "' needs an argument.");
        }
    }

    void fail(Object o) {
        outln(o);
        exit(1);
    }

    void warn(Object o) {
        if (false && !isSet("nowarn")) {
            outln("Warning: " + o);
        }
    }

    void exit(int i) {
        System.exit(i);
    }

    void help() {
        outln(helpString());
    }

    void outln(Object o) {
        System.err.println(o);
    }
}
