/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
package org.aspectj.debugger.base;

import org.aspectj.tools.ide.SourceLine;
import com.sun.jdi.*;
import java.util.*;
import java.io.*;

import org.aspectj.util.LineNumberTableMapper;

import org.aspectj.asm.*;

public class SourceLineFinder {

//    private SymbolManager sm;
    private String        fullSrcPath;
    private Location      loc;

    static boolean debug = false;
    static void db(Object o) {
        if (debug) System.out.println(o);
    }

    public SourceLineFinder(String fullSrcPath) {
//        this.sm = sm;
        this.fullSrcPath = fullSrcPath;
//        System.err.println(">> fullSrcPath: " + fullSrcPath);
    }

    public SourceLine findMethod(Location loc) {
        try {
            String pkgfullName = loc.declaringType().name();
            int pkgLoc = pkgfullName.lastIndexOf('.');
            String pkg = (-1 == pkgLoc) ? null : pkgfullName.substring(0, pkgLoc).replace('.', '/');
            String attrib = loc.sourceName();
            String srcPath = org.aspectj.debugger.base.AJDebugger.INSTANCE.getSourcePath();
            File srcRoot = new File(srcPath);

//            System.err.println(">> findMethod attrib: " + attrib + ", pkg: " + pkg + ", src: " + srcPath);
            LineNumberTableMapper lntm = new LineNumberTableMapper(attrib, pkg, srcRoot);
            System.err.println(">> lntm: " + lntm.getCorrespondingFile(loc.lineNumber()).getPath() + " : " + lntm.getCorrespondingLineNumber(loc.lineNumber()));
//            throw new RuntimeException();
            return new SourceLine(lntm.getCorrespondingFile(loc.lineNumber()).getPath(), lntm.getCorrespondingLineNumber(loc.lineNumber()));

//        return findMethod(method(this.loc = loc), classDec(className(loc)));
        } catch (com.sun.jdi.AbsentInformationException aie) {
            System.err.println("> ERROR: absent information.");
            aie.printStackTrace();
            return null;
        }
    }

    public SourceLine findMethod(String className, String methodName) {
        System.err.println(">> findMethod className: " + className + ", methodName: " + methodName);
//        return findMethod(methodName, classDec(className));
//        return null;
        return new SourceLine(className, -1);
    }

//    private Method method(Location loc) {
//        return (loc != null && loc.codeIndex() == 0) ? loc.method() : null;
//    }
//
//    private String className(Location loc) {
//        return loc != null ? loc.declaringType().name() : null;
//    }
//
//    private Declaration classDec(String className) {
//        return classDec(sm.getDeclarations(fullSrcPath), className);
//    }
//
//    private Declaration classDec(Declaration[] decs, String className) {
//        if (decs == null) return null;
//        List q = new Vector();
//        for (int i = 0; i < decs.length; i++) {
//            q.add(decs[i]);
//        }
//        while (!q.isEmpty()) {
//            Declaration dec = (Declaration) q.remove(0);
//            if (matchesClass(dec, className)) return dec;
//            Declaration[] newDecs = dec.getDeclarations();
//            if (newDecs != null) {
//                for (int i = 0; i < newDecs.length; i++) {
//                    q.add(newDecs[i]);
//                }
//            }
//        } while (!q.isEmpty());
//        return null;
//    }
//
//    private boolean matchesClass(Declaration dec, String className) {
//        return new ClassMatcher().matches(dec, className);
//    }
//
//    static class ClassMatcher {
//        public boolean matches(Declaration dec, String className) {
//            int idot = className.lastIndexOf('.');
//            if (idot != -1) {
//                className = className.substring(idot+1);
//            }
//            if (dec == null || className == null) return false;
//            String kind = dec.getKind();
//            String signature = dec.getSignature();
//            return kind.equals("class") && signature.equals(className);
//        }
//    }
//
//    private SourceLine findMethod(Method method, Declaration classDec) {
//        db("findMethod: method=" + method + " class=" + classDec + " loc=" + loc);
//        if (classDec == null || method == null) return null;
//        SourceLine sl = null;
//        Declaration[] decs = classDec.getDeclarations();
//        if (decs == null) return null;
//        int N = decs.length;
//        for (int i = 0; i < N; i++) {
//            Declaration dec = decs[i];
//            if (!dec.getKind().equals("method")) continue;
//            if (!methodMatches(dec, method)) continue;
//            sl = new SourceLine(fullSrcPath, dec.getBeginLine());
//            db("!!! Returning sl=" + sl + " for " + method + " loc=" + loc);
//            break;
//        }
//        return sl;
//    }
//
//    private SourceLine findMethod(String name, Declaration classDec) {
//        db("findMethod: name=" + name + " class=" + classDec + " loc=" + loc);
//        if (classDec == null || name == null) return null;
//        SourceLine sl = null;
//        Declaration[] decs = classDec.getDeclarations();
//        if (decs == null) return null;
//        int N = decs.length;
//        for (int i = 0; i < N; i++) {
//            Declaration dec = decs[i];
//            if (!isMethod(dec)) continue;
//            if (!methodMatches(dec, name)) continue;
//            sl = new SourceLine(fullSrcPath, dec.getBeginLine());
//            db("!!! Returning sl=" + sl + " for " + name + " loc=" + loc);
//            break;
//        }
//        return sl;
//    }
//
//    private boolean isMethod(Declaration dec) {
//        return dec.getKind().equals("method") || dec.getKind().equals("constructor");
//    }
//
//    private boolean methodMatches(Declaration methodDec, Method method) {
//        return new MethodMatcher().matches(methodDec, method);
//    }
//
//    private boolean methodMatches(Declaration methodDec, String name) {
//        return new MethodMatcher().matches(methodDec, name);
//    }
//
//    public static class MethodMatcher {
//
//        public boolean matches(Declaration methodDec, Object methodObject) {
//            String nomods = stripModifiers(methodDec.getFullSignature());
//            String name = nomods;
//            int iparen = name.indexOf("(");
//            name = name.substring(0, iparen); // we want the exception!
//            if (methodObject instanceof String) {
//                String methodName = methodObject+"";
//                return methodName.equals(name);
//            }
//            Method method = (Method) methodObject; // we want the exception!
//            if (!name.equals(method.name())) {
//                return false;
//            }
//
//            //!! Come back and do this in one iteration
//            String args = nomods.substring(iparen+1, nomods.length()-1); // no last paren
//            return method.argumentTypeNames().equals(args(args));
//        }
//
//        private List args(String argsString) {
//            List args = new Vector();
//            StringTokenizer tok = new StringTokenizer(argsString, "(),");
//            while (tok.hasMoreTokens()) {
//                String arg = tok.nextToken();
//                int ispace = arg.indexOf(" ");
//                arg = arg.substring(0, ispace);
//                args.add(arg);
//            }
//            return args;
//        }
//
//        private String stripModifiers(String sig) {
//            int iparen = sig.indexOf("(");
//            if (iparen != -1) {
//                for (int i = iparen-1; i >= 0; i--) {
//                    if (sig.charAt(i) == ' ') {
//                        return sig.substring(i+1).trim();
//                    }
//                }
//            }
//            return sig;
//        }
//
//    }
}
