/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved. 
 */

package org.aspectj.debugger.base;

import com.sun.jdi.*;
import java.util.*;

/**
 * ThreadLockInformation.java
 *
 *
 * Created: Fri Aug 25 16:26:07 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public class ThreadLockInformation  {

    private ThreadReference threadRef;
    
    public ThreadLockInformation(ThreadReference threadRef) {
        this.threadRef = threadRef;
    }

    public List getOwnedMonitors() throws DebuggerException {
        try {
            return threadRef.ownedMonitors();
        } catch (UnsupportedOperationException uoe) {
            throw new DebuggerException(uoe);          
        } catch (IncompatibleThreadStateException itse) {
            throw new DebuggerException(itse);            
        }
    }

    public List getWaitingThreads() throws DebuggerException {
        try {
            return threadRef.waitingThreads();
        } catch (UnsupportedOperationException uoe) {
            throw new DebuggerException(uoe);          
        } catch (IncompatibleThreadStateException itse) {
            throw new DebuggerException(itse);            
        }
    }

    public String toString() {
        try {
            String str = "Thread: " + threadRef + Consts.newline;
            str += "Owned monitors: " + getOwnedMonitors() + Consts.newline;
            str += "Waiting threads: " + getWaitingThreads();
            return str;
        } catch (DebuggerException e) {
        }
        return "Thread thread has resumed.";
    }
}
