/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import com.sun.jdi.ArrayReference;
import com.sun.jdi.IncompatibleThreadStateException;
import com.sun.jdi.InvalidTypeException;
import com.sun.jdi.ClassNotLoadedException;
import com.sun.jdi.Field;
import com.sun.jdi.ObjectReference;
import com.sun.jdi.ReferenceType;
import com.sun.jdi.ThreadReference;
import com.sun.jdi.Type;
import com.sun.jdi.Value;

public class AJArrayMemberValueGetter implements AJValueGetter {

    private ArrayReference  arrayParent     = null;
    private int             index           = -1;

    public AJArrayMemberValueGetter(ArrayReference arrayParent, int index) {
        this.arrayParent    = arrayParent;
        this.index          = index;
    }

    public Value getValue() {
        return arrayParent.getValue(index);
    }

    public Value getValue(String s) {
        return arrayParent.getValue(index);
    }

    public void setValue(Value v) throws    ClassNotLoadedException,
                                            InvalidTypeException {
        arrayParent.setValue(index, v);
    }

    public ThreadReference getThread()
            throws  ClassNotLoadedException,
                    IncompatibleThreadStateException {
        return arrayParent.owningThread();
    }

    public String getName() {
        return "[" + index + "]";
    }

    //
    //TODO: Should get the child type, but need to check for null
    //
    public Type getTypeRef() throws ClassNotLoadedException {
        return arrayParent.type();
    }

    //
    //TODO: Should get the child type, but need to check for null
    //XXX HACK to strip off the array braces at the end
    public String getTypeName() throws ClassNotLoadedException {
        String name = arrayParent.type().name();
        return name.substring(0, name.length() - 2);
    }

    public ReferenceType getDeclaringClass() {
        return null;
    }
}