/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
package org.aspectj.debugger.gui;

import org.aspectj.tools.ide.*;
import java.util.*;

public class AJCMethodHelper implements MethodHelper {

    private Declaration dec;

    public AJCMethodHelper(Declaration dec) {
        this.dec = dec;
    }

    public String getRealName() {
        return dec.getSignature();
    }

    public String getDisplayName() {
        return getRealName();
    }

    public String getToolTipText() {
        return dec.getFullSignature();
    }

    public String getClassName() {
        return dec.getDeclaringType();
    }

    public String getRelativePath() {
        return dec.getFilename();
    }

    public int getLine() {
        return dec.getBeginLine();
    }

    public String getBreakpoint() {
        return getPackagePrefix() + dec.getDeclaringType() + "." + breakpointSignature();
    }

    private String getPackagePrefix() {
        String packageName = dec.getPackageName();
        if (packageName == null ||
            packageName.equals("")) {
            return "";
        }
        return packageName + ".";
    }

    public Declaration getDec() {
        return dec;
    }

    private String breakpointSignature() {
        return dec.getSignature();
    }

    private int type = Integer.MIN_VALUE;
    public int getType() {
        return (type >= 0) ? type : (type = _getType());
    }
    public int _getType() {
        StringTokenizer tok = new StringTokenizer(dec.getFullSignature());
        boolean isStatic    = false;
        boolean isPublic    = false;
        boolean isPrivate   = false;
        boolean isProtected = false;
        boolean isInit      = false;
        boolean isCtr       = false;
        if (getRealName().startsWith("new(") || getRealName().equals("<init>")) {
            isCtr = true;
        }
        while (tok.hasMoreTokens()) {
            String token = tok.nextToken();
            if (token.indexOf("(") != -1)  break;
            if (token.equals("static"))    isStatic    = true;
            if (token.equals("public"))    isPublic    = true;
            if (token.equals("protected")) isProtected = true;
            if (token.equals("private"))   isPrivate   = true;
        }
        return MethodNode.getType(isCtr, isStatic, isPublic, isPrivate, isProtected);
    }
}
