/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import java.io.File;
import java.util.StringTokenizer;
import org.aspectj.tools.ide.Declaration;

public class AJDecParser {

    public final static int BAD = -1;

    public static int getMethodLineNumber(String fileName,
                                          String methodName) {
        if (fileName == null) {
            return BAD;
        }
        Util.debug("Looking for methodName=" + methodName + " in " + fileName);
        String fullName = new File(fileName).getAbsolutePath();
        Declaration[] decs =
            AJLineMapper.symbolManager.
            getDeclarations(fullName);
        if (decs == null) {
            return BAD;
        }
        MethodComparator c = new MethodComparator();
        for (int i = 0; i < decs.length; i++) {
            int result = getBeginLine(decs[i], methodName, c);
            if (result != BAD) {
                return result;
            }
        }
        return BAD;
  }

    public static int getBeginLine(Declaration dec,
                                   String s,
                                   DeclarationComparator c) {
        if (c.compare(dec, s)) {
            return dec.getBeginLine();
        }
        Declaration[] decs = dec.getDeclarations();
        int result = BAD;
        for (int i = 0; i < decs.length; i++) {
            if ((result = getBeginLine(decs[i], s, c)) != BAD) {
                return result;
            }
        }
        return BAD;
    }
}

interface DeclarationComparator {
  boolean compare(Declaration d, String s);
}

class MethodComparator implements DeclarationComparator {
    public boolean compare(Declaration d1, String s) {
        String method1 = getMethodPrototype(d1);
        String method2 = s;
        return (!method1.equals("") &&
                !method2.equals("") &&
                method1.equals(method2));
    }

    public static String getMethodPrototype(Declaration dec) {
        if (dec == null ||!(dec.getKind().equals("method") ||
                            dec.getKind().equals("constructor"))) {
            return "";
        }
        String fullSig = dec.getFullSignature();
        String sig     = dec.getSignature();
        String methodName = sig.substring(0, sig.indexOf("("));
        if (methodName.equals("new")) {
            methodName = dec.getDeclaringType();
        }
        String name = dec.getDeclaringType() + "." + methodName;
        String args = fullSig.substring(fullSig.indexOf("(")+1,
                                        fullSig.indexOf(")"));
        String str = name + "(";
        StringTokenizer tok = new StringTokenizer(args, ",");
        if (!tok.hasMoreTokens()) {
        str += ")";
        }
        while (tok.hasMoreTokens()) {
            String arg = tok.nextToken().trim();
            String type = arg.substring(0, arg.indexOf(" "));
            str += arg.substring(0, arg.indexOf(" ")) +
                   (tok.hasMoreTokens() ? "," : ")");
        }
        return str;
  }
}
