/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import com.sun.jdi.ClassNotLoadedException;
import com.sun.jdi.Field;
import com.sun.jdi.InvalidTypeException;
import com.sun.jdi.ObjectReference;
import com.sun.jdi.ReferenceType;
import com.sun.jdi.ThreadReference;
import com.sun.jdi.Type;
import com.sun.jdi.Value;

public class AJMemberValueGetter implements AJValueGetter, FieldGetter {

    private ObjectReference parent  = null;
    private Field           field   = null;

    public AJMemberValueGetter(ObjectReference parent, Field field) {
        this.parent = parent;
        this.field  = field;
    }

    public Value getValue() {
        return parent.getValue(field);
    }

    public Value getValue(String s) {
        return parent.getValue(field);
    }

    public void setValue(Value v) throws  ClassNotLoadedException,
                                          InvalidTypeException {
        parent.setValue(field, v);
    }

    public ThreadReference getThread() throws ClassNotLoadedException {
        try {
            return parent.owningThread();
        } catch (Exception e) { Util.ex(e); }
        return null;
    }

    public String getName() {
        return refType().name() + "." + field.name();
    }

    public Type getTypeRef() throws ClassNotLoadedException {
        return field.type();
    }

    public String getTypeName() throws ClassNotLoadedException {
        return "" + field.type();
    }

    public ReferenceType getDeclaringClass() {
        return field.declaringType();
    }

    public Field getField() {
        return field;
    }

    private ReferenceType refType() {
        return parent.referenceType();
    }
}
