/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;
import org.aspectj.util.gui.*;

import java.awt.Frame;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import com.sun.jdi.*;

public class AJNewValueDialog extends CenteredJDialog {

    private AJValueNode valueNode = null;
    private JOptionPane optionPane = null;
    private JTextField textField = null;
    private JButton button1 = null;
    private JButton button2 = null;
    private JLabel messageLabel = null;

    private final static int TEXTFIELD_WIDTH = 20;
    private final static String TAB = "        ";

    public AJNewValueDialog(Frame frame, AJValueNode valueNode) {
        super(frame, "Change a Value", true);
        this.valueNode = valueNode;

        String message1 = "Change a value";
        textField = new JTextField(TEXTFIELD_WIDTH);
        textField.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    setValue();
                }
            }});
        clearText();
        messageLabel = new JLabel();
        clearMessage();
        Object[] components = {message1, textField, messageLabel};

        button1 = new JButton("Change");
        button1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setValue();
            }});
        button2 = new JButton("Clear");
        button2.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                clearText();
            }});
        Object[] buttons = {button1, button2};

        optionPane =
            new JOptionPane
                (components,
                 JOptionPane.QUESTION_MESSAGE,
                 JOptionPane.YES_NO_OPTION,
                 null,
                 buttons,
                 buttons[0]);
        this.setContentPane(optionPane);
        this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    }

    public String getValueText() {
        String text = textField.getText();
        if ((valueNode instanceof AJObjectValueNode) && ((AJObjectValueNode) valueNode).isString()) {
            text = '"' + text + '"';
        }
        return text;
    }

    private void setValue() {
        if (valueNode == null) {
            return;
        }
        try {
            valueNode.setValue(getValueText());
            ComponentRepository.getThreadGroupTreePane().getTree().reloadAll();
            clearText();
            setVisible(false);
        } catch (NumberFormatException e4) {
            err(e4);
        } catch (IllegalArgumentException e6) {
            err(e6);
        } catch (DebuggerException de) {
            ComponentRepository.getGUIDebugger().handleDebuggerException(de);
        }
    }

    private void err(Throwable e) {
        String message = e.getMessage();
        if (e instanceof NumberFormatException) {
            message += " is too BIG!";
        }
        message(message);
        clearText();
        AJUtil.ex(e);
    }

    public void popup(AJValueNode valueNode) {
        this.valueNode = valueNode;
        this.setTitle("change: " + valueNode);
        this.clearText();
        this.clearMessage();
        this.pack();
        this.setVisible(true);
    }

    public void popup() {
        this.setTitle("change: " + valueNode);
        this.clearText();
        this.clearMessage();
        this.pack();
        this.setVisible(true);
    }

    private void message(String s) {
        messageLabel.setText(s);
    }

    private void clearMessage() {
        messageLabel.setText("");
    }

    private void clearText() {
        textField.setText("");
        textField.requestFocus();
    }
}
