/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import com.sun.jdi.IncompatibleThreadStateException;
import com.sun.jdi.InvalidTypeException;
import com.sun.jdi.ClassNotLoadedException;
import com.sun.jdi.Field;
import com.sun.jdi.ObjectReference;
import com.sun.jdi.ReferenceType;
import com.sun.jdi.StringReference;
import com.sun.jdi.ThreadReference;
import com.sun.jdi.Type;
import com.sun.jdi.Value;

public class AJObjectReferenceValueGetter implements AJValueGetter {

    private ObjectReference object  = null;
    private String          name    = null;

    public AJObjectReferenceValueGetter(ObjectReference object,
                                        String name) {
        this.object = object;
        this.name   = name;
    }

    public Value getValue() {
        return object;
    }

    public Value getValue(String s) {
        Field field = object.referenceType().fieldByName(s);
        return object.getValue(field);
    }

    // TODO: Deal with exception
    public void setValue(Value v) throws  ClassNotLoadedException,
                                          InvalidTypeException {
    }

    // TODO: Deal with exception
    public ThreadReference getThread() throws ClassNotLoadedException,
                                              IncompatibleThreadStateException
    {
        return object.owningThread();
    }

    public String getName() {
        return refType().name() + "." + this.name;
    }

    public Type getTypeRef() {
        return object.type();
    }

    public String getTypeName() {
        return object.type().name();
    }

    public ReferenceType getDeclaringClass() {
        return null; //TODO
    }

    private ReferenceType refType() {
        return object.referenceType();
    }
}
