/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;
import java.util.*;
import com.sun.jdi.*;

public class AJObjectValueNode extends AJValueNode {
    public AJObjectValueNode(AJValueGetter getter) {
        super(getter);
        setType(AJIcons.OBJECT_ICON);
    }

    public boolean isObject() {
        return true;
    }

    public boolean isString() {
        return getValue() instanceof StringReference;
    }

    public void showKids() {
        if (getChildCount() == 0)
            showValues();
    }

    public int getType() {
        return (g().getValue() == null ? AJIcons.NULL_OBJECT_ICON : super.getType());
    }

    private void showValues() {
        hideValues();
        try {
            ObjectReference parent = (ObjectReference) g().getValue();
            ReferenceType refType = parent.referenceType();
            add(new AJClassNode(refType));
            Iterator iter = refType.allFields().iterator();
            while (iter.hasNext()) {
                Field field = (Field) iter.next();
                if (!field.isStatic()) {
                    AJValueNode valueNode = AJValueNodeFactory.make(parent, field);
                    add(valueNode);
                }
            }
        } catch (Exception e) { Util.ex(e); }
    }

    public void hideValues() {
        removeAllChildren();
    }

    public String getValueString() {
        String str = "no class loader";
        try {
            String type = AJUtil.stripParens(g().getTypeName());
            if (type.startsWith("java.lang.String")) {
                str = "" + g().getValue();
            } else {
                str = type + " instance";
            }
        } catch (ClassNotLoadedException e) {}
        return str;
    }
}
