/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */
package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;
import com.sun.jdi.*;

public class AJReferenceTypeMemberValueGetter implements AJValueGetter, FieldGetter {

    private ReferenceType refType;
    private Field field;

    public AJReferenceTypeMemberValueGetter(ReferenceType refType, Field field) {
        this.refType    = refType;
        this.field      = field;
    }

    public Value getValue() {
        return refType.getValue(field);
    }

    public Value getValue(String s) {
        return getValue();
    }

    public void setValue(Value v) throws ClassNotLoadedException,
                                         InvalidTypeException {
        refType.classObject().setValue(field, v);
    }

    public ThreadReference getThread() throws ClassNotLoadedException,
                                              IncompatibleThreadStateException {
        return refType.classObject().owningThread();
    }

    public String getName() {
        return refType.name() + "." + field.name();
    }

    public Type getTypeRef() throws ClassNotLoadedException {
        return field.type();
    }
    public String getTypeName() throws ClassNotLoadedException {
        return getTypeRef().name();
    }

    public ReferenceType getDeclaringClass() {
        return refType;
    }

    public Field getField() {
        return field;
    }
}
