/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

/**
 * A wrapper class for the root node of a tree, so there is a common look
 * for all root nodes.
 */
public class AJRootNode extends AJTreeNode {

    public AJRootNode(String string, int type) {
        super(type);
        this.string = string;
        setUserObject(string);
    }

    /**
     * Constructs a new node with the name <code>string<\code>.
     *
     * @param   string  The name of the node.
     */
    public AJRootNode(String string) {
        this(string, AJIcons.ROOT_ICON);
    }

    /**
     * Overrides isRoot from AJTreeNode because this <b>is<\b> a root.
     *
     * @return  true.
     * @see     #AJTreeNode.isRoot()
     */
    public boolean isRoot() {
        return true;
    }

    public boolean isOKToExpand() {
        return true;
    }

    public boolean isOKToCollapse() {
        return false;
    }
}
