/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import com.sun.jdi.*;
import com.sun.jdi.event.*;

public class AJThreadGroupTreePane
    extends JScrollPane
    implements StopListener,
               VMListener,
               ThreadListener {

    //TODO: Should be private, but keep it public now for testing
    public AJThreadGroupTree tree = null;

    private GUIDebugger guid;

    public AJThreadGroupTreePane(AJThreadGroupTree tree) {
        super();
        this.tree = tree;
        addTree();
    }

    public AJThreadGroupTreePane(GUIDebugger guid) {
        this(new AJThreadGroupTree());
        this.guid = guid;
        guid.getDebugger().addStopListener(this);
        guid.getDebugger().addVMListener(this);
        guid.getDebugger().addThreadListener(this);
        
    }

    public boolean add(ThreadReference ref) {
        return tree.add(ref);
    }

    public boolean remove(ThreadReference ref) {
        return tree.remove(ref);
    }

    public void clear() {
        tree.clear();
    }

    public void modernizeTree() {
        try {
            tree.modernize();
        } catch (Exception e) { AJUtil.ex(e);
        }
    }

    public void invalidateAllThreads() {
        tree.invalidateAllThreads();
    }

    public void expandToFrame(StackFrame frame) {
        guid.getGui().showThreadPane();
        tree.expandToFrame(frame);
        centerTreeSelection();
    }

    public void expandToLocals(StackFrame frame) {
        guid.getGui().showThreadPane();
        tree.expandToLocals(frame);
        centerTreeSelection();
    }

    protected void centerTreeSelection() {
//          int visibleAmount = verticalScrollBar.getVisibleAmount();
//          int unitIncrement  = verticalScrollBar.getUnitIncrement();
//          int min = verticalScrollBar.getMinimum();
//          int max = verticalScrollBar.getMaximum();
//          int rowCount = tree.getRowCount();
//          int[] selections = tree.getSelectionRows();
//          int selection = -1;
//          if (selections != null && selections.length > 0) {
//              selection = selections[0];
//          }
//          System.out.println("visibleAmount:" + visibleAmount);
//          System.out.println("unitIncrement:" + unitIncrement);
//          System.out.println("min:" + min);
//          System.out.println("max:" + max);
//          System.out.println("rowCount:" + rowCount);
//          System.out.println("selection:" + selection);
//          if (selection != -1) {
            
//          }
    }

    public void expandToValue(Value value, String name) {
        tree.expandToValue(value, name);
        centerTreeSelection();
    }

    public AJThreadGroupTree getTree() {
        return tree;
    }

    private void addTree() {
        getViewport().add(tree, BorderLayout.CENTER);
    }

    /* VMListener */
    public void vmDeathEvent(VMDeathEvent e) {
        tree.clear();
    }
    public void vmDisconnectEvent(VMDisconnectEvent e) {
        tree.clear();
    }
    public void vmStartEvent(VMStartEvent e) {
        try {
            Iterator iter = ComponentRepository.vm().allThreads().iterator();
            while (iter.hasNext()) {
                tree.add((ThreadReference) iter.next());
            }
        } catch (Exception ee) {}
        modernizeTree();
    }

    /* StopListener */
    public void accessWatchpointEvent(AccessWatchpointEvent e) {
        modernizeTree();
    }
    public void breakpointEvent(BreakpointEvent e) {
        modernizeTree();
    }
    public void exceptionEvent(ExceptionEvent e) {
        modernizeTree();
    }
    public void modificationWatchpointEvent(ModificationWatchpointEvent e) {
        modernizeTree();
    }
    public void stepEvent(StepEvent e) {
        modernizeTree();
    }

    /* ThreadListener */
    public void threadDeathEvent(ThreadDeathEvent e) {
        tree.remove(e.thread());
    }
    public void threadStartEvent(ThreadStartEvent e) {
        tree.add(e.thread());
    }

    public static String d() { return "Thread Group Tree Pane"; }
    public String toString() { return d(); }
}
