/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import java.awt.Dimension;
import java.util.List;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;

/**
 * An extension of JTree providing very little else.
 */
public abstract class AJTree extends JTree {

    public AJTree(TreeModel model) {
        super(model);
        init();
    }

    /**
     * Calls init to construct an empty tree.
     */
    public AJTree() {
        init();
    }

    /**
     * Constructs a new tree with AJRootNode <code>root<\code>.
     *
     * @param   root    new root.
     */
    public AJTree(AJRootNode root) {
        super(root);
        init();
    }

    /**
     * Returns the root.
     *
     * @return  the root.
     */
    public AJRootNode getRoot() {
        return (AJRootNode) getModel().getRoot();
    }

    /**
     * Sets the root.  The node <code>root<\code> must be an instance of
     * AJDBRootNode so that it keeps with the look of all the roots.
     *
     * @param   root    the new root.
     */
    public void setRoot(AJRootNode root) {
        ((DefaultTreeModel) getModel()).setRoot(root);
    }

    //XXX Need a better way
    /**
     * Expands the tree fully.
     */
    protected void expand() {
        int row = 0;
        while (row < this.getRowCount()) {
            expandRow(row++);
        }
    }

//    public void expandPath(TreePath path) {
//        System.out.println("expandPath");
//        super.expandPath(path);
//        System.out.println("expandPath DONE");
//    }
//
//    public void treeDidChange() {
//        System.out.println("treeDidChange");
//        super.treeDidChange();
//        System.out.println("treeDidChange DONE");
//    }
//
//    public void revalidate() {
//        System.out.println("revalidate");
//        super.revalidate();
//        System.out.println("revalidate DONE");
//    }
//
//    public void repaint() {
//        System.out.println("repaint");
//        super.repaint();
//        System.out.println("repaint DONE");
//    }


    /**
     * Override this method to clear this tree to the extent desired.
     */
    public abstract void clear();

    public void reloadAll() {
        //XXX Hack, reload all the leaf nodes
        DefaultTreeModel model = (DefaultTreeModel) getModel();
        Enumeration nodes = ((DefaultMutableTreeNode) model.getRoot()).breadthFirstEnumeration();
        while (nodes.hasMoreElements()) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) nodes.nextElement();
            if (node.isLeaf() || ((AJTreeNode) node).isString()) {
                model.reload(node);
            }
        }
    }

    /**
     * A threaded upateUI.
     */
    public final void modernize() {
        Runnable update = new Runnable() {
                public void run() {
                    try {
                        doModernize();
                    } catch (VMDisconnectedException vmde) {
                    }
                }
            };
        if (SwingUtilities.isEventDispatchThread()) {
            update.run();
        } else {
            SwingUtilities.invokeLater(update);
        }
        //((DefaultTreeModel) getModel()).reload(getRoot());
    }

    public void doModernize() {
        AJTree.super.updateUI();
    }

    /**
     * Called by the constructor the initialize the tree.  This contains common
     * characteristics used by many trees, and was, for convenience, put in
     * this class.  If other trees look much different we'll take it out of
     * here and let all the trees configure themselves completely.
     */
    protected final void init() {
        putClientProperty("JTree.lineStyle", "Angled");
        setEditable(false);
        //setShowsRootHandles(true);
        setMinimumSize(new Dimension(100,200));
        getSelectionModel().setSelectionMode
            (TreeSelectionModel.SINGLE_TREE_SELECTION);
        addTreeWillExpandListener(new AJTreeWillExpandListener(this));
        addTreeExpansionListener(new AJTreeExpansionListener(this));
        //this.setCellRenderer(new AJTreeCellRenderer());
    }
}
