/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import java.awt.Component;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;

/**
 * A cell renderer that just sets the icon of the AJDBTreeNode according to its
 * type.
 *
 * @see #AJDBTreeCellRenderer.getTreeCellRendererComponent
 */
public class AJTreeCellRenderer extends DefaultTreeCellRenderer {

    /**
     * Calls the super.getTreeCellRendererComponent and changes the icon on
     * this tree node <code>value<\code> according to AJDBTreeIcons.getIcon.
     *
     * @param   too.many.to.list
     * @see     #AJTreeIcons.getIcon
     */
    public Component getTreeCellRendererComponent
                        (JTree tree,
                         Object value,
                         boolean sel,
                         boolean expanded,
                         boolean leaf,
                         int row,
                         boolean hasFocus) {
        AJTreeNode node = (AJTreeNode) value;
        super.getTreeCellRendererComponent
            (tree,
             value,
             sel,
             expanded,
             leaf,
             row,
             hasFocus);
        setIcon(AJIcons.getIcon(node));
        setToolTipText(node.getToolTipText());
        setText(node + "");
        return this;
    }

}
