/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import java.awt.Frame;
import java.io.*;
import javax.swing.JOptionPane;

import com.sun.jdi.*;
import com.sun.jdi.event.*;
import com.sun.jdi.request.*;

public class AJUtil {

    private final static Frame FRAME = null; //Main.gui;

    public static void info(Object o) {
        Object msg = check(o);
        JOptionPane.showMessageDialog
            (FRAME, msg, "Info", JOptionPane.INFORMATION_MESSAGE);
    }

    public static void warn(Object o) {
        Object msg = check(o);
        JOptionPane.showMessageDialog
            (FRAME, msg, "Warning", JOptionPane.WARNING_MESSAGE);
    }

    public static void error(Object o) {
        Object msg = check(o);
        JOptionPane.showMessageDialog
            (FRAME, msg, "Error", JOptionPane.ERROR_MESSAGE);
    }

    public static void question(Object o) {
        Object msg = check(o);
        JOptionPane.showMessageDialog
            (FRAME, msg, "Huh?", JOptionPane.QUESTION_MESSAGE);
    }

    public static void popup(Object o) {
        Object msg = check(o);
        JOptionPane.showMessageDialog
            (FRAME, msg, "Error", JOptionPane.PLAIN_MESSAGE);
    }

    public static void ex(Throwable o) {
        if (o == null) return;
        o.printStackTrace(System.err);
        JOptionPane.showMessageDialog
            (FRAME, o.getMessage(), "Exception", JOptionPane.ERROR_MESSAGE);
    }

//      public static String getStringFromSpec(BreakpointSpec spec) {
//          String specString = spec + "";
//          String strip = "breakpoint ";
//          if (specString.indexOf(strip) != -1) {
//              return specString.substring(strip.length());
//          }
//          return specString;
//      }

    public static String getStringFromRequest(BreakpointRequest request) {
        String specString = request + "";
        String strip = "breakpoint request ";
        if (specString.indexOf(strip) != -1) {
            return specString.substring(strip.length() - 1);
        }
        return specString;
    }

    public static String stripParens(Method method) {
        return stripParens(method + "");
    }

    public static String stripParens(String method) {
        int index = method.indexOf("(");
        if (index < 0) {
            return method;
        }
        String noParens = method.substring(0, index);
        return noParens.trim();
    }

    public static String getParams(Method method) {
        return getParams(method + "");
    }

    public static String getParams(String method) {
        int lparen = method.indexOf("(");
        int rparen = method.indexOf(")");
        if (lparen < 0 || rparen < 0) {
            return method;
        }
        return method.substring(lparen+1, rparen).trim();
    }

    public static String removePathFromSource(String source) {
        int isep = source.lastIndexOf(File.separatorChar);
        if (isep == -1) {
            return source;
        }
        return source.substring(isep + 1);
    }

    public static String stripTypeFromMethod(String name) {
        int iDot = locateFirstDotBeforeLParen(name);
        if (iDot != -1) {
            return name.substring(iDot + 1, name.length());
        }
        return name;
    }

    public static String getParamsFromMethod(Method method) {
        return getParamsFromMethod(method + "");
    }

    public static String getParamsFromMethod(String name) {
        int iLParen = name.indexOf("(");
        int iRParen = name.indexOf(")");
        if (iLParen == -1 || iRParen == -1) {
            return "";
        }
        return name.substring(iLParen + 1, iRParen);
    }

    public static String getTypeFromMethod(String name) {
        int iDot = locateFirstDotBeforeLParen(name);
        if (iDot != -1) {
            return name.substring(0, iDot);
        }
        return name;
    }

    public static String getNameFromMethod(String name) {
        int iDot = locateFirstDotBeforeLParen(name);
        int lParen = name.indexOf("(");
        if (iDot != -1 && lParen != -1) {
            return name.substring(iDot, lParen);
        }
        return name;
    }

    public static String safeIndex(String str, String pat) {
        int index = str.indexOf(pat);
        if (index != -1) {
            return str.substring(index + pat.length() + 1).trim();
        }
        return str;
    }

    private static int locateFirstDotBeforeLParen(String name) {
        int iDot = -1;
        for (int i = 0; i < name.length(); i++) {
            char c = name.charAt(i);
            if (c == '(') {
                break;
            }
            if (c == '.') {
                iDot = i;
            }
        }
        return iDot;
    }

    private static Object check(Object o) {
        if (o == null) {
            return "<NULL MESSAGE>";
        }
        return o;
    }
}
