/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import java.awt.datatransfer.*;
import java.awt.event.MouseEvent;
import java.util.*;
import javax.swing.tree.*;
import com.sun.jdi.*;

public class AJValueNode extends AJTreeNode {

    protected AJValueGetter getter = null;

    public AJValueNode(AJValueGetter getter, int type) {
        super(type);
        this.getter = getter;
        //setType(_getType());
    }
    public AJValueNode(AJValueGetter getter) {
        this(getter, AJIcons.BAD_ICON);
        setType(_getType());
    }

    public Value getValue() {
        return getter.getValue();
    }

    public Value getValue(String s) {
        return getter.getValue(s);
    }

    public ReferenceType getDeclaringClass() {
        return getter.getDeclaringClass();
    }

    public boolean canSetWatchPoint() {
        return (getDeclaringClass() != null);
    }

    public void setValue(Object o) throws DebuggerException {
        if (o == null) return;
        AJTreeNode parent = (AJTreeNode) getParent();
        while (!parent.isStackFrame()) {
            parent = (AJTreeNode) parent.getParent();
        }
        AJStackFrameNode parentHack = (AJStackFrameNode) parent;
        try {
            getter.setValue(ComponentRepository.getAJDebugger().getValue(o, parentHack.getStackFrame()));
        } catch (ClassNotLoadedException cnle) {
        } catch (InvalidTypeException ite) {
            AJUtil.ex(ite);
        }
    }

    public void setWatchPoint() {
        ReferenceType refType = getDeclaringClass();
        if (refType == null) {
            AJUtil.warn("You cannot set a watchpoint on: " + this);
            return;
        }
        String declaringType = refType.name();
        String name = getName();
        String command = "watch " + declaringType + "." + name;
        ComponentRepository.getCommandLine().executeCommand(command);
    }

    //
    //TODO: Deal with exception
    //
    public ThreadReference getThread()
            throws ClassNotLoadedException,
                    IncompatibleThreadStateException {
        return getter.getThread();
    }

    public String getName() {
        return getter.getName();
    }

    public Type getTypeRef() throws ClassNotLoadedException {
        return getter.getTypeRef();
    }

    //
    // TODO: Deal with exception
    //
    public String getTypeName() {
        try {
        return getter.getTypeName();
        } catch (Exception e) { Util.error(e); }
        return "";
    }

    //
    // TODO: Deal with exception
    //
    public String getValueString() {
        try {
            return "" + getter.getValue();
        } catch (Exception e) {
        }
        return "";
    }

    //
    // TODO: Deal with exception
    //
    public String toString() {
        String name  = "<no-name>";
        String type  = "<no-type>";
        Value  value = null;
        try {
            name = getter.getName();
        } catch (Exception e) {
        }
        try {
            type = AJUtil.stripParens(getter.getTypeName());
        } catch (Exception e) {
        }
        try {
            value = getValue();
        } catch (InvalidStackFrameException isfe) {
        }
        return name + ": " + type + " = " + format(value);
    }

    //!!! Should be abstract
    protected String format(Value value) {
        return value + "";
    }

    public boolean isField() {
        return g() instanceof FieldGetter;
    }

    //private int type = -1;
//      public int getType() {
//          return (type == -1) ? (type = _getType()) : type;
//      }
    private int _getType() {        
        if (isField()) {
            FieldGetter g = (FieldGetter) g();
            Field field = g.getField();
            if (field.isStatic()) {
                if (field.isPublic())           return AJIcons.FIELD_STATIC_PUBLIC_ICON;
                if (field.isPackagePrivate())   return AJIcons.FIELD_STATIC_PACKAGE_ICON;
                if (field.isProtected())        return AJIcons.FIELD_STATIC_PROTECTED_ICON;
                if (field.isPrivate())          return AJIcons.FIELD_STATIC_PRIVATE_ICON;
            } else {
                if (field.isPublic())           return AJIcons.FIELD_PUBLIC_ICON;
                if (field.isPackagePrivate())   return AJIcons.FIELD_PACKAGE_ICON;
                if (field.isProtected())        return AJIcons.FIELD_PROTECTED_ICON;
                if (field.isPrivate())          return AJIcons.FIELD_PRIVATE_ICON;
            }
        }
        return getType();
    }

    protected AJValueGetter g() {
        return getter;
    }

    public void leftMouseButton(MouseEvent e) {
    }

    public void middleMouseButton(MouseEvent e) {
        AJTreeNodeMenu.showValueNodeMenu(this, e.getComponent(), e.getX(), e.getY());
    }

    public void rightMouseButton(MouseEvent e) {
        AJTreeNodeMenu.showValueNodeMenu(this, e.getComponent(), e.getX(), e.getY());
    }

    public void setValue() {
        Dialogs.showNewValueDialog(this);
    }

    public boolean isValueNode() {
        return true;
    }
}


















