/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;
import com.sun.jdi.*;

public class AJValueNodeFactory {

    public static AJValueNode make(LocalVariable local, StackFrame frame) {
        return make(new AJLocalVariableValueGetter(local, frame),
                    local.signature());
    }

    public static AJValueNode make(ReferenceType refType, Field field) {
        return make(new AJReferenceTypeMemberValueGetter(refType, field),
                    field.signature());
    }

    public static AJValueNode make(ObjectReference object, String name) {
        return make(new AJObjectReferenceValueGetter(object, name),
                    object.referenceType().signature());
    }

    public static AJValueNode make(ObjectReference parent, Field field) {
        return make(new AJMemberValueGetter(parent, field),
                    field.signature());
    }

    public static AJValueNode make(ArrayReference arrayParent, int index) {
        return make(new AJArrayMemberValueGetter(arrayParent, index),
                    arrayParent.getValue(0).type().signature());
    }    

    private static AJValueNode make(AJValueGetter g, String sig) {
        if      (sig.equals("Z"))     return new AJBooleanValueNode(g);
        else if (sig.equals("B"))     return new AJByteValueNode(g);
        else if (sig.equals("C"))     return new AJCharValueNode(g);
        else if (sig.equals("D"))     return new AJDoubleValueNode(g);
        else if (sig.equals("F"))     return new AJFloatValueNode(g);
        else if (sig.equals("I"))     return new AJIntegerValueNode(g);
        else if (sig.equals("J"))     return new AJLongValueNode(g);
        else if (sig.startsWith("S")) return new AJShortValueNode(g);
        else if (sig.startsWith("L")) return new AJObjectValueNode(g);
        else if (sig.startsWith("[")) return new AJArrayValueNode(g);
        return new AJValueNode(g);
    }
}
