/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JMenuItem;

public class AJValueNodeMenu extends AJTreeNodeMenu {

    private AJValueNode valueNode;
    private JMenuItem menuItemChange    = new JMenuItem("Change value");
    private JMenuItem menuItemWatch     = new JMenuItem("Set watchpoint");
    private JMenuItem menuItemHex       = new JMenuItem();

    public AJValueNodeMenu() {
        this(null);
    }

    public AJValueNodeMenu(AJValueNode valueNode) {
        super();
        setValueNode(valueNode);
        add(menuItemWatch());
        add(menuItemChange());
        if (valueNode instanceof HexableAJValueNode) add(menuItemHex());
    }


    public void setValueNode(AJValueNode valueNode) {
        this.valueNode = valueNode;
        if (valueNode == null) {
            menuItemWatch.setEnabled(false);
            menuItemChange.setEnabled(false);
        } else if (!valueNode.canSetWatchPoint()) {
            menuItemWatch.setEnabled(false);
            menuItemChange.setEnabled(true);
        } else {
            boolean canSetWatchPoint = true;
            try {
                VirtualMachine vm = ComponentRepository.vm();
                canSetWatchPoint = vm.canWatchFieldAccess() &&
                                   vm.canWatchFieldModification();
            } catch (NoVMException e) {}
            menuItemWatch.setEnabled(canSetWatchPoint);
            menuItemChange.setEnabled(true);
        }
    }

    private JMenuItem menuItemChange() {
        menuItemChange.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setValue();
            }});
        return menuItemChange;
    }

    private JMenuItem menuItemWatch() {
        menuItemWatch.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setWatchPoint();
            }});
        return menuItemWatch;
    }

    private JMenuItem menuItemHex() {
        menuItemHex.setText("To " + (((HexableAJValueNode) valueNode).isHex()
                                      ? "decimal" : "hex"));
        menuItemHex.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                toHex();
            }});
        return menuItemHex;
    }

    private void setWatchPoint() {
        if (valueNode == null) return;
        if (!valueNode.canSetWatchPoint()) return;
        valueNode.setWatchPoint();
    }

    private void setValue() {
        if (valueNode == null) return;
        valueNode.setValue();
    }

    private void toHex() {
        if (valueNode == null) return;
        if (!(valueNode instanceof HexableAJValueNode)) return;
        ((HexableAJValueNode) valueNode).toggle();
    }
}