/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved. 
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.io.Serializable;
import com.sun.jdi.InvalidTypeException;
import com.sun.jdi.Value;

import com.sun.jdi.*;

public class AJValueWrapper implements Transferable,
                                       Serializable {

    Object value = null;
    
    public AJValueWrapper(Value value) {
        this.value = wrap(value);
    }
    
    public Value unwrap() {
        Value result = null;
        try {
            VirtualMachine vm = ComponentRepository.vm();
            if (value instanceof Byte) {
                result = vm.mirrorOf( ((Byte)value).byteValue() );
            } else if (value instanceof Character) {
                result = vm.mirrorOf(((Character)value).charValue());
            } else if (value instanceof Double) {
                result = vm.mirrorOf( ((Double)value).doubleValue() );
            } else if (value instanceof Float) {
                result = vm.mirrorOf( ((Float)value).floatValue() );
            } else if (value instanceof Integer) {
                result = vm.mirrorOf( ((Integer)value).intValue() );
            } else if (value instanceof Long) {
                result = vm.mirrorOf( ((Long)value).longValue() );
            } else if (value instanceof Short) {
                result = vm.mirrorOf( ((Short)value).shortValue() );
            } else if (value instanceof String) {
                result = vm.mirrorOf( ((String)value).toString() );
            }
        } catch (DebuggerException e) {
        }
        return result;
    }

    private Object wrap(Value value) {
        Object result = null;
        if (value instanceof ByteValue) {
            result = new Byte( ((ByteValue)value).byteValue() );
        } else if (value instanceof CharValue) {
            result = new Character( ((CharValue)value).charValue() );
        } else if (value instanceof DoubleValue) {
            result = new Double( ((DoubleValue)value).doubleValue() );
        } else if (value instanceof FloatValue) {
            result = new Float( ((FloatValue)value).floatValue() );
        } else if (value instanceof IntegerValue) {
            result = new Integer( ((IntegerValue)value).intValue() );
        } else if (value instanceof LongValue) {
            result = new Long( ((LongValue)value).longValue() );
        } else if (value instanceof ShortValue) {
            result = new Short( ((ShortValue)value).shortValue() );
        } else if (value instanceof StringReference) {
            result = new String( ((StringReference)value).toString() );
        }
        return result;
    }

    /*
     * Transferable stuff
     */
     //TODO: Check the DataFlavor
     public Object getTransferData(DataFlavor flavor) {
        return this;
     }

    public DataFlavor[] getTransferDataFlavors() {
        return AJDataFlavor.FLAVORS;
    }

    public boolean isDataFlavorSupported(DataFlavor flavor) {
        boolean ok = false;
        if (flavor.getHumanPresentableName().equals("AJValueWrapper")) {
            ok = true;
        }
        return ok;
    }
}
