/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.text.*;

public abstract class AbstractTextArea extends JPanel {

    protected final static int TEXT_ROWS = 30;
    protected final static int TEXT_COLS = 50;
    protected final static int TEXT_SIZE = 12;
    protected final static Dimension TEXT_DIM = new Dimension(TEXT_ROWS, TEXT_COLS);
    protected final static int OUT = 0;
    protected final static int NOTICE = 1;
    protected final static int ERR = 2;
    protected final static String NEWLINE = System.getProperty("line.separator");

    protected PrintStream printStream = null;
    protected JScrollPane scrollPane = null;
    protected DefaultStyledDocument dsd = new DefaultStyledDocument();
    protected JTextArea text; //new JTextPane(dsd);
    protected SimpleAttributeSet[] attrs = null;
    protected Color[] colors = { Color.black,
                                 Color.blue.darker(),
                                 Color.red.darker() };
    protected int textSize = TEXT_SIZE;
//      protected JScrollBar xScrollBar;
//      protected JScrollBar yScrollBar;

    protected GUIDebugger guiDebugger;

    private int count = 0;

    public final void superInit(GUIDebugger guiDebugger) {
        this.guiDebugger = guiDebugger;
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        text = new JTextArea(0,0);
        text.setEditable(false);
        printStream = createPrintStream();
        scrollPane = new JScrollPane(text);
//          xScrollBar = scrollPane.getHorizontalScrollBar();
//          yScrollBar = scrollPane.getVerticalScrollBar();
        scrollPane.setVerticalScrollBarPolicy
            (JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        add(scrollPane);
        initAttrs();
    }

//      static int count_ = 0;

    protected abstract PrintStream createPrintStream();

    public PrintStream getPrintStream() {
        return printStream;
    }

    public void setTextSize(int textSize) {
        this.textSize = textSize;
    }

    public abstract void restorePrintStream();

    public void shutDown() {
        restorePrintStream();
    }

    protected void out(String s) {
        append(s, OUT);
    }

    protected void outln(String s) {
        append(s, OUT);
        newline();
    }

    protected void err(String s) {
        append(s, ERR);
    }

    protected void errln(String s) {
        append(s, ERR);
        newline();
    }

    protected void notice(String s) {
        append(s, NOTICE);
    }

    protected void noticeln(String s) {
        append(s, NOTICE);
        newline();
    }

    protected void append(String s, int i) {
        text.append(s);
        //yScrollBar.setValue(yScrollBar.getMaximum());
        //xScrollBar.setValue(xScrollBar.getMinimum());
    }

    protected void newline() {
        append(NEWLINE, -1);
    }

    protected void initAttrs() {
        attrs = new SimpleAttributeSet[colors.length];
        for (int i = 0; i < colors.length; i++) {
            attrs[i] = getAttr(colors[i]);
        }
    }

    protected SimpleAttributeSet getAttr(Color c) {
        SimpleAttributeSet attr = new SimpleAttributeSet();
        StyleConstants.setForeground(attr, c);
        StyleConstants.setFontSize(attr, textSize);
        return attr;
    }

    protected static class TextPrintOutStream extends PrintStream {
        public TextPrintOutStream(final AbstractTextArea text) {
            super(new OutputStream() {
                public void write(int b) throws IOException {
                    text.out(String.valueOf((char)b));
                }
            });

        }
    }

    protected static class TextPrintNoticeStream extends PrintStream {
        public TextPrintNoticeStream(final AbstractTextArea text) {
            super(new OutputStream() {
                public void write(int b) throws IOException {
                    text.notice(String.valueOf((char)b));
                }
            });

        }
    }

    protected static class TextPrintErrStream extends PrintStream {
        public TextPrintErrStream(final AbstractTextArea text) {
            super(new OutputStream() {
                    public void write(int b) throws IOException {
                        text.err(String.valueOf((char)b));
                    }
                });
        }
    }
}
