/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
*
* This file is part of the debugger and core tools for the AspectJ(tm)
* programming language; see http://aspectj.org
*
* The contents of this file are subject to the Mozilla Public License
* Version 1.1 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License at
* either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is AspectJ.
*
* The Initial Developer of the Original Code is Xerox Corporation. Portions
* created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
* All Rights Reserved.
*/
package org.aspectj.debugger.gui;

import org.aspectj.tools.ide.*;
import java.util.*;

public class AdviceNode extends     AJTreeNode
                        implements  Sourceable,
                                    Breakable,
                                    Advisable {
    protected Declaration dec;
    public AdviceNode(Declaration dec) {
        super(AJIcons.ADVICE_ICON);
        setUserObject((this.dec = dec).getSignature());
        setAlwaysExpand(true);
    }

    public String getToolTipText() {
        if (dec == null) {
            //System.out.println(this + " dec is null");
            return "no declaration";
        }
        if (dec.getCrosscutDeclaration() == null) {
            //System.out.println(this + " crosscut.dec is null");
            return "no crosscut declaration";
        }
        return showClass() + dec.getSignature() + " : " +
               dec.getCrosscutDeclaration().getSignature();
    }

    protected String showClass() {
        if (getParent() instanceof MethodNode) {
            return dec.getDeclaringType() + ".";
        }
        return "";
    }

    public void showOnSource(AbstractSourcePane sourcePane) {
        sourcePane.showSourceForFileAndLine(dec.getFilename(), dec.getBeginLine());
    }

    public String getBreakpoint() {
        return AJLineMapper.removeRoot(dec.getFilename()) + ":" + dec.getBeginLine();
    }

    public List getAdvisedMethodStrings() {
        List methods = new Vector();
        if (children == null) return methods;
        Iterator iter = children.iterator();
        while (iter.hasNext()) {
            methods.add(((MethodNode) iter.next()).getBreakpoint());
        }
        return methods;
    }
}
