/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ClassLineBreakpointDialog extends JDialog {

    static final int TEXT_COLS = 20;
    static final int LINE_COLS =  5;

    private String mainMsg = "Create a class-line breakpoint";
    private JTextField classText = new JTextField(TEXT_COLS);
    private JTextField lineText = new JTextField(LINE_COLS);

    public ClassLineBreakpointDialog(Frame frame) {
        super(frame, "Set a Breakpoint", true);
        classText.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    setBreakpoint();
                }
            }});
        lineText.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    setBreakpoint();
                }
            }});

        JPanel classTextPanel = new JPanel();
        classTextPanel.add(classText);
        classTextPanel.setBorder(BorderFactory.createTitledBorder(
                                    BorderFactory.createEmptyBorder(0,0,0,0), "Class"));
        JPanel lineTextPanel = new JPanel();
        lineTextPanel.add(lineText);
        lineTextPanel.setBorder(BorderFactory.createTitledBorder(
                                    BorderFactory.createEmptyBorder(0,0,0,0), "Line"));
        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.X_AXIS));
        mainPanel.add(classTextPanel);
        mainPanel.add(lineTextPanel);
        Object[] components = { mainMsg, mainPanel };

        JButton setButton = new JButton("Set");
        setButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setBreakpoint();
            }});
        JButton clearButton = new JButton("Clear");
        clearButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                clear();
            }});
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancel();
            }});
        Object[] buttons = { setButton, clearButton, cancelButton };

        JOptionPane optionPane =
            new JOptionPane
                (components,
                 JOptionPane.QUESTION_MESSAGE,
                 JOptionPane.YES_NO_OPTION,
                 null,
                 buttons,
                 buttons[0]);
        this.setContentPane(optionPane);
        this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    }

    public ClassLineBreakpointDialog() {
        this(null);
    }

    public void show() {
        pack();
        super.show();
        classText.requestFocus();
    }

    private void setBreakpoint() {
        String className = classText.getText();
        int line = -1;
        try {
            line = Integer.parseInt(lineText.getText());
        } catch (NumberFormatException e) {
            AJUtil.warn("The line field must contain a number");
            return;
        }
        String cmd = "stop at " + className + ":" + line;
        ComponentRepository.getCommandLine().executeCommand(cmd);
        clear();
        hide();
    }

    private void clear() {
        classText.setText("");
        lineText.setText("");
    }

    private void cancel() {
        hide();
    }
}