/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
package org.aspectj.debugger.gui;

import com.sun.jdi.*;
import java.util.List;
import java.util.*;
import javax.swing.tree.*;

public abstract class ClassNode extends AJClassNode implements Sourceable {
    public ClassNode(ReferenceType refType) {
        super(refType);
        setUserObject(refType);
    }

    protected ReferenceType refType() {
        return (ReferenceType) getUserObject();
    }

    protected void addMethod(Method method) {
        if (method.declaringType().equals(refType())) {
            MethodNode methodNode = new NonAdvisableMethodNode(method);
            add(methodNode);
        } else {
            addToSuper(method);
        }
    }

    protected void addToSuper(Method method) {
        TreeNode treeNode = getChildAt(0);
        if (!(treeNode instanceof ClassNode)) return;
        ClassNode classNode = (ClassNode) treeNode;
        classNode.addMethod(method);
    }

    protected abstract String name();
    protected abstract String packageName();
    public String fullName() {
        String pname = packageName();
        String name = name();
        if (pname == null || pname.equals("")) {
            return name;
        }
        return pname + "." + name;
    }

    public String toString() {
        String name = name();
        //!!! Come back and fix this
        if (getParent() instanceof ClassTreePane.PackageNode) {
            int idot = name.lastIndexOf('.');
            return idot != -1 ? name.substring(idot+1) : name;
        }
        return name;
    }

    public String getToolTipText() {

        return
            kind() + " "
            + name();
    }

    protected abstract String kind();

    protected AJClassNode newClassNode(ReferenceType refType) {
        return ClassNodeFactory.classNode(refType);
    }

    public void showOnSource(AbstractSourcePane sourcePane) {
        sourcePane.showSourceForFileAndLine(getRelativePath(), getLine());
    }
    protected abstract String getRelativePath();
    protected abstract int getLine();

    public List getMethodStrings() {
        List methodStrings = new Vector();
        if (children == null) return methodStrings;
        Iterator iter = children.iterator();
        while (iter.hasNext()) {
            methodStrings.add(((MethodNode) iter.next()).getBreakpoint());
        }
        return methodStrings;
    }
}
