/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

import com.sun.jdi.event.*;

public class CommandLine extends JPanel {

    private String  TITLE   = "Command";
    private int     WIDTH   = 40;
    private AJTextField text = null;
    private JLabel label = null;

    private ColorChangingLabel statusLabel;

    public CommandLine(ComponentDirector gui) {
        label = new JLabel(TITLE + ": ");
        text = new AJTextField("", WIDTH);
        text.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pressEnter();
        }});
        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        add(label);
        add(text);
        statusLabel = new ColorChangingLabel();
    }

    /**
     * We want <b>all</b> key presses (including <code>tab</code>s) sent to
     * this field.
     */
    public boolean isManagingFocus() {
        return true;
    }

    public ColorChangingLabel getStatusLabel() {
        return statusLabel;
    }


    void addText(String str) {
        String getText = text.getText();
        int pos = text.getCaretPosition();
        if (getText.length() == 0) {
            text.setText(str);
        } else {
            text.setText(getText.substring(0, pos) + str +
                         getText.substring(pos, getText.length()));
            text.setCaretPosition(pos+1);
        }
    }

    void pressEnter() {
        String str = text.getText();
        executeCommand(str);
    }

    void pressBackSpace() {
        String getText = text.getText();
        int pos = text.getCaretPosition();
        if (getText.length() != 0 && pos != 0) {
            text.setText(getText.substring(0, pos-1) +
                         (pos < getText.length() ?
                            getText.substring(pos, getText.length()) :
                            ""));
            text.setCaretPosition(pos-1);
        }
    }

    void pressRight() {
        int newPos = text.getCaretPosition() + 1;
        if (newPos <= text.getText().length()) {
            text.setCaretPosition(newPos);
        }
    }

    void pressLeft() {
        int newPos = text.getCaretPosition() - 1;
        if (newPos >= 0) {
            text.setCaretPosition(newPos);
        }
    }

    void pressHome() {
        text.setCaretPosition(0);
    }

    void pressEnd() {
        text.setCaretPosition(text.getText().length());
    }

//      void processKeyStroke(KeyStroke k) {
//          text.processKeyStroke(k);
//      }

//      KeyStroke[] getTextKeyStrokes() {
//          return text.getRegisteredKeyStrokes();
//      }

    void executeCommand(String cmd) {
        ComponentRepository.getAJDebugger().executeCommand(cmd);
        text.setText("");
    }

    protected JTextField getTextField() {
        return text;
    }

    public static String d() { return "Command Line"; }
    public String toString() { return d(); }
}


class AJTextField extends JTextField {
    AJTextField(String s, int t) {
        super(s, t);
    }
//      void processKeyStroke(KeyStroke k) {
//          KeyEvent e2 = new KeyEvent(this, KeyEvent.KEY_PRESSED,
//                                     System.currentTimeMillis(),
//                                     k.getModifiers(), k.getKeyCode());
//          KeyEvent e3 = new KeyEvent(this, KeyEvent.KEY_RELEASED,
//                                     System.currentTimeMillis(),
//                                     k.getModifiers(), k.getKeyCode());
//          try {
//              this.processComponentKeyEvent(e2);
//              this.processComponentKeyEvent(e3);
//              return;
//          } catch (Exception e) {
//          }
//          try {
//              this.processKeyEvent(e2);
//              Util.msg("e2", "processKeyEvent: " + e2, Color.red);
//              return;
//          } catch (Exception e) {
//          }

//      }

    /**
     * We want <b>all</b> key presses (including <code>tab</code>s) sent to
     * this field.
     */
    public boolean isManagingFocus() {
        return true;
    }

    public boolean isFocusTraversable() {
        return false;
    }
}

class ColorChangingLabel extends JPanel {

    private String text;
    private JLabel label;

    public static Color ON_COLOR    = Color.red;
    public static Color OFF_COLOR   = Color.green.darker().darker();
    public static int   WIDTH       = 170;

    ColorChangingLabel(String text) {
        super();
        this.text = text;
        this.label = new JLabel(text, SwingConstants.CENTER);
        label.setPreferredSize
            (new Dimension(WIDTH,
                           (int)label.getPreferredSize().getHeight()));
        turnOff();
        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        add(new JLabel("Status:"));
        add(label);
    }

    ColorChangingLabel() {
        this("Idle");
    }

    void turnOn() {
        label.setForeground(ON_COLOR);
        giveUpFocus();
    }

    void giveUpFocus() {
        CommandLine cl = ComponentRepository.getCommandLine();
        if (cl != null) {
            cl.requestFocus();
        }
    }

    void turnOn(String newText) {
        label.setText(newText);
        turnOn();
    }

    void turnOff(String newText) {
        if (newText == null) {
            label.setText(text);
        } else {
            label.setText(newText);
        }
        label.setForeground(OFF_COLOR);
        giveUpFocus();
    }

    void turnOff() {
        turnOff(text);
    }
}
