/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;
import org.aspectj.debugger.ide.*;
import org.aspectj.util.gui.*;

import com.sun.jdi.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

public class ComponentRepository {

    private static StackTablePane           stackTablePane      = null;
    private static CommandLine              commandLine         = null;
    private static OutputTextArea           outputTextArea      = null;
    private static EnvTextArea              envTextArea         = null;
    private static AbstractSourcePane       sourcePane          = null;
//    private static SourceTreePane           sourceTreePane      = null;
    private static AJThreadGroupTreePane    threadGroupTreePane = null;
    private static BreakpointTreePane       breakpointTreePane  = null;
    private static ComponentDirector        gui                 = null;
    private static AJToolBar                toolbar             = null;
//    private static ClassTreePane            classTreePane       = null;
    private static AJMenuBar                menuBar             = null;
    private static ColorChangingLabel       statusLabel         = null;
    private static GUIDebugger              guiDebugger         = null;
    private static AJDebugger               debugger            = null;
    private static WatchpointTreePane       watchpointTreePane  = null;
    private static VariablesTreePane        variablesTreePane   = null;

    static boolean debug = false;
    static void db(Object o) {
        if (debug) {
            System.err.println("<DEBUG> " +o);
        }
    }

    private static String[] tasks = null;
    private static void initTasks() {
        tasks = new String[] {
            AJIcons.d(),
            GUIDebugger.d(),
            AJDebugger.d(),
            OutputTextArea.d(),
            StackTablePane.d(),
            CommandLine.d(),
            EnvTextArea.d(),
            SourcePane.d(),
//            SourceTreePane.d(),
            AJThreadGroupTreePane.d(),
            BreakpointTreePane.d(),
            WatchpointTreePane.d(),
            VariablesTreePane.d(),
            AJToolBar.d(),
//            ClassTreePane.d(),
            AJMenuBar.d(),
            ComponentDirector.d(),
        };
    }
    static {
        initTasks();
    }

    /* Logging */
    private final static int LAGER_WIDTH    = 300;
    private final static int LAGER_HEIGHT   =  30;
    private static class Lager { //extends CenteredJFrame {
        private JLabel[] labels;
        private int times[];
        private int task = 0;
        private final static Color defaultColor = new JLabel().getForeground();
        public Lager(String[] tasks) {
//            super("AJDB");
//            setResizable(false);
//            JPanel p = panel();
//            p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));

//            p.add(titlePanel());
//            labels = new JLabel[tasks.length];
//            times = new int[labels.length];
//            for (int i = 0; i < tasks.length; i++) {
//                p.add(labels[i] = center(tasks[i]));
//            }
//            addWindowListener(new WindowAdapter() {
//                public void windowClosing(WindowEvent e) {
//                    debugger.quit();
//                }
//            });
//            getContentPane().add(p, BorderLayout.CENTER);
        }
//        private JLabel center(String s) {
//            JLabel label = new JLabel(s);
//            label.setAlignmentX(CENTER_ALIGNMENT);
//            label.setAlignmentY(CENTER_ALIGNMENT);
//            return label;
//        }
        private JPanel panel() {
            JPanel p = new JPanel();
            p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
            p.setBorder(BorderFactory.createEtchedBorder());
            return p;
        }
//        private JPanel titlePanel() {
//            JPanel p = panel();
//            JLabel title = center("  While you're waiting...visit  ");
//            Font f = title.getFont();
//            f = new Font(f.getName(), f.getStyle(), 14);
//            title.setForeground(Color.black);
//            title.setFont(f);
//            JLabel link = center("  http://aspectj.org  ");
//            link.setForeground(Color.blue);
//            link.setFont(f);
//            p.add(title);
//            p.add(link);
//            return p;
//        }
        public void log(Object str) {
//            if (checkTask(task-1)) unhighlight(labels[task-1]);
//            if (checkTask(task))   highlight(labels[task]);
            task++;
        }
        private void unhighlight(JLabel label) {
            Font f = label.getFont();
            f = new Font(f.getName(), Font.BOLD, f.getSize());
            label.setFont(f);
            label.setForeground(defaultColor.darker());
        }
        private void highlight(JLabel label) {
            Font f = label.getFont();
            f = new Font(f.getName(), Font.BOLD | Font.ITALIC, f.getSize());
            label.setFont(f);
            label.setForeground(defaultColor.darker().darker().darker());
        }
        private boolean checkTask(int t) {
            return t > -1 && t < tasks.length;
        }
        public void start() {
            if (!Modes.isJBuilder() && !Modes.isForte()) {
//                pack();
//                setVisible(true);
            }
        }
        public void done() {
//            setVisible(false);
//            dispose();
        }
    }
    private static Lager lager = new Lager(tasks);
    private static void startLogging() {
        lager.start();
    }
    private static void doneLogging() {
        lager.done();
    }
    private static void task() {
        lager.log(null);
    }

    private static boolean ide() {
        return Modes.isIde();
        //return false;
    }

    public final static void init(ComponentDirector gui,
                                  GUIDebugger _guiDebugger,
                                  AbstractSourcePane _sourcePane,
                                  boolean _debug) {
        startLogging();
        debug                           = _debug;
        task(); AJIcons.init();
        task(); guiDebugger             = _guiDebugger;
        guiDebugger.setGui(gui);
        task(); debugger                = guiDebugger.getDebugger();
        task(); outputTextArea          = new OutputTextArea(guiDebugger);
        gui.setPrintStream(outputTextArea.getPrintStream());
        task(); stackTablePane          = new StackTablePane(guiDebugger);
        task(); commandLine             = new CommandLine(gui);
        task(); envTextArea             = new EnvTextArea(guiDebugger);
        task(); sourcePane              = _sourcePane;
//        task(); if (!ide()) { sourceTreePane          = new SourceTreePane(guiDebugger, sourcePane); }
        task(); threadGroupTreePane     = new AJThreadGroupTreePane(guiDebugger);
        task(); breakpointTreePane      = new BreakpointTreePane(gui, guiDebugger);
        task(); watchpointTreePane      = new WatchpointTreePane(gui, guiDebugger);
        task(); variablesTreePane       = new VariablesTreePane(gui, guiDebugger);
        task(); toolbar                 = new AJToolBar(guiDebugger);
//        task(); classTreePane           = new ClassTreePane(guiDebugger);
        task(); if (!ide()) { menuBar                 = new AJMenuBar(guiDebugger); }
        statusLabel             = commandLine.getStatusLabel();
        task(); ComponentRepository.gui = gui;
        doneLogging();
    }

    public static void init(ComponentDirector gui, GUIDebugger _guiDebugger,
                            AbstractSourcePane _sourcePane) {
        init(gui, _guiDebugger, _sourcePane, false);
    }

    public static void init(ComponentDirector gui, String[] args) {
        GUIDebugger guid = new GUIDebugger(gui, args);
        init(gui, guid, new SourcePane(guid));
    }

    public static void execute(String cmd) {
        getGUIDebugger().executeCommand(cmd);
    }

    public static StackTablePane        getStackTablePane()        { return stackTablePane;           }
    public static CommandLine           getCommandLine()           { return commandLine;              }
    public static OutputTextArea        getOutputTextArea()        { return outputTextArea;           }
    public static EnvTextArea           getEnvTextArea()           { return envTextArea;              }
    public static AbstractSourcePane    getSourcePane()            { return sourcePane;               }
//    public static SourceTreePane        getSourceTreePane()        { return sourceTreePane;           }
    public static AJThreadGroupTreePane getThreadGroupTreePane()   { return threadGroupTreePane;      }
    public static BreakpointTreePane    getBreakpointTreePane()    { return breakpointTreePane;       }
    public static Frame                 getFrame()                 { return gui;                      }
    public static ComponentDirector     getComponentDirector()     { return gui;                      }
    public static AJToolBar             getToolBar()               { return toolbar;                  }
//    public static ClassTreePane         getClassTreePane()         { return classTreePane;            }
    public static AJMenuBar             getMenuBar()               { return menuBar;                  }
    public static ColorChangingLabel    getStatusLabel()           { return statusLabel;              }
    public static GUIDebugger           getGUIDebugger()           { return guiDebugger;              }
    public static AJDebugger            getAJDebugger()            { return debugger;                 }
    public static VirtualMachine        vm() throws NoVMException  { return getAJDebugger().getVM();  }
    public static WatchpointTreePane    getWatchpointTreePane()    { return watchpointTreePane;       }
    public static VariablesTreePane     getVariablesTreePane()     { return variablesTreePane;        }

//    public static void setRoot(String newSourcePath) {
//        if (getSourceTreePane() == null) return;
//        getSourceTreePane().setRoot(newSourcePath);
//    }
}
