/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
package org.aspectj.debugger.gui;

import org.aspectj.util.gui.*;
import org.aspectj.debugger.base.Debug;
import org.aspectj.debugger.base.Util;
import org.aspectj.debugger.base.Modes;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.List;
import javax.swing.*;

import com.sun.jdi.*;
import com.sun.jdi.connect.*;

public class ConnectFrame extends CenteredJDialog {

    final boolean ALLOW_CONNECT_OPTIONS;

    private GUIDebugger guid;

    //XXX Changed true to false so we can see the progress in
    //XXX the Forte main frame.
    // --> and this screwed me over!!!!!
    public ConnectFrame(GUIDebugger guid) {
        super(null, "Please select a connector", true);//Modes.isJBuilder() ? false : true);
        this.guid = guid;
        ALLOW_CONNECT_OPTIONS =
            guid.getDebugger().getOptions().isSet("allowconnections");
    }

    //XXX made static for JBuilder compiler
    private static Connector current = null;
    private VirtualMachine vm;
    public void setVm(VirtualMachine vm) {
        this.vm = vm;
    }
    public VirtualMachine getVm() {
        return vm;
    }
    public VirtualMachine select(String vmArgs,
                                 String className,
                                 String commandLine) {
        return ALLOW_CONNECT_OPTIONS ?
            connectorOptions(vmArgs, className, commandLine) :
            launchingConnector(vmArgs, className, commandLine);
    }
    private VirtualMachine launchingConnector(String vmArgs,
                                              String className,
                                              String commandLine) {
        Iterator iter = Bootstrap.virtualMachineManager().
            allConnectors().iterator();
        while (iter.hasNext()) {
            Connector con = (Connector)iter.next();
            if (con instanceof LaunchingConnector) {
                setVm(new LaunchFrame(current = con).
                    select(vmArgs, className, commandLine));
                return getVm();
            }
        }
        return connectorOptions(vmArgs, className, commandLine);
    }
    private VirtualMachine connectorOptions(final String vmArgs,
                                            final String className,
                                            final String commandLine) {
        Container c = getContentPane();
        JPanel buttons = new JPanel();
        buttons.setLayout(new BoxLayout(buttons, BoxLayout.Y_AXIS));
        Iterator iter = Bootstrap.virtualMachineManager().
            allConnectors().iterator();
        final ButtonGroup group = new ButtonGroup();
        boolean first = true;
        final OKCancel okCancel = new OKCancel(this) {
                public void ok() {
                    setVm(new LaunchFrame(current).
                        select(vmArgs,
                               className,
                               commandLine));
                }
            };
        while (iter.hasNext()) {
            final Connector con = (Connector)iter.next();
            JRadioButton rb =
                new JRadioButton(new AbstractAction(con.description()) {
                        public void actionPerformed(ActionEvent e) {
                            current = con;
                        }
                    });
            if (first) {
                rb.setSelected(true);
                current = con;
            }
            first = false;
            group.add(rb);
            buttons.add(rb);
        }
        c.add(buttons, BorderLayout.CENTER);
        c.add(okCancel, BorderLayout.SOUTH);
        show();
        Debug.debug("vm="+vm);
        return vm;
    }

    private class OKCancel extends JPanel implements ActionListener {
        JDialog frame;
        OKCancel(final JDialog frame) {
            super();
            this.frame = frame;
            this.setLayout(new FlowLayout(FlowLayout.CENTER));
            JButton ok = new JButton(new AbstractAction("OK") {
                    public void actionPerformed(ActionEvent e) {
                        realOk();
                    }
                });
            JButton cancel = new JButton(new AbstractAction("Cancel") {
                    public void actionPerformed(ActionEvent e) {
                        realCancel();
                    }
                });
            this.add(ok);
            this.add(cancel);
        }
        public void ok() {}
        public void cancel() {}
        private void realOk() {
            ok();
            frame.dispose();
            frame.setVisible(false);
        }
        private void realCancel() {
            cancel();
            frame.dispose();
            frame.setVisible(false);
        }
        public void actionPerformed(ActionEvent e) {
            realOk();
        }
    }

    private class LaunchFrame extends CenteredJDialog {
        Connector con;
        VirtualMachine vm = null;
        LaunchFrame(final Connector con) {
            super(null,
                  "Please select launch parameters for "
                  + con.name(),
                  true);//Modes.isJBuilder() ? false : true);
            this.con = con;
        }

        VirtualMachine select(String vmArgs,
                              String className,
                              String commandLine) {
            Debug.debug("vmArgs:" + vmArgs +
                        " className:" + className +
                        " commandLine:" + commandLine);
            final Map args = con.defaultArguments();
            final List options = new Vector();
            JPanel p = new JPanel();
            p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
            okCancel = new OKCancel(this) {
                    public void ok() {
                        Iterator opts = options.iterator();
                        while (opts.hasNext()) {
                            ArgPanel ap = (ArgPanel)opts.next();
                            Connector.Argument arg = ap.arg;
                            arg.setValue(ap.text.getText().trim());
                            Debug.debug(arg.name() + " --> " + arg.value());
                        }
                        if (con instanceof AttachingConnector) {
                            try {
                                vm = ((AttachingConnector)con).attach(args);
                            } catch (IOException ioe) {
                            } catch (IllegalConnectorArgumentsException iae) {
                            }
                        } else if (con instanceof LaunchingConnector) {
                            try {
                                vm = ((LaunchingConnector)con).launch(args);
                            } catch (IOException ioe) {
                            } catch (IllegalConnectorArgumentsException iae) {
                            } catch (VMStartException vmse) {
                            }
                        } else if (con instanceof ListeningConnector) {
                            try {
                                vm = ((ListeningConnector)con).accept(args);
                            } catch (IOException ioe) {
                            } catch (IllegalConnectorArgumentsException iae) {
                            }
                        }
                    }
                };
            Iterator iter = args.keySet().iterator();
            while (iter.hasNext()) {
                final String name = iter.next() + "";
                final Connector.Argument arg = (Connector.Argument)args.get(name);
                if ("main".equalsIgnoreCase(name)) {
                    String newMain
                        = !Util.empty(className)
                        ? className
                        : guid.getClassNameToRun();
                    if (newMain != null) {
                        arg.setValue(newMain);
                    }
                }
                if ("options".equalsIgnoreCase(name)) {
                    String optionsStr = "";
                    if (!Util.empty(commandLine)) {
                        optionsStr += " " + commandLine;
                    }
                    String vmParameters =
                        !Util.empty(vmArgs)
                        ? vmArgs
                        : guid.getVMParameters();
                    boolean alreadyHasClasspath = false;
                    if (!Util.empty(vmParameters)) {
                        alreadyHasClasspath =
                            vmParameters.indexOf("-classpath") != -1;
                        optionsStr += " " + vmParameters;
                    }
                    String classpath = guid.getClasspath();
                    if (!alreadyHasClasspath && !Util.empty(classpath)) {
                        optionsStr += " -classpath \"" + classpath + "\"";
                    }

                    optionsStr = optionsStr.trim();
                    arg.setValue(optionsStr);
                }
                ArgPanel argp = new ArgPanel(arg);
                options.add(argp);
                p.add(argp);
            }
            this.getContentPane().add(p, BorderLayout.CENTER);
            this.getContentPane().add(okCancel, BorderLayout.SOUTH);
            this.show();
            return vm;
        }

        private OKCancel okCancel = null;
        private class ArgPanel extends JPanel {
            Connector.Argument arg;
            JTextField text = new JTextField(ARG_TEXT_WIDTH);
            ArgPanel(Connector.Argument arg) {
                super();
                this.arg = arg;
                this.setBorder(BorderFactory.
                               createTitledBorder(arg.description()));
                this.add(new JLabel(arg.name() + ":"));
                String value = arg.value();
                text.setText(value);
                text.addActionListener(okCancel);
                this.add(text);

            }
        }
    }

    private static int ARG_TEXT_WIDTH = 40;
    public static VirtualMachine vm(GUIDebugger guid,
                                    String vmArgs,
                                    String className,
                                    String commandLine) {
        return new ConnectFrame(guid).
            select(vmArgs, className, commandLine);
    }

    public static void main(String[] args) {
        System.out.println("my vm=" +
                           new ConnectFrame(null).
                               select(null,null,null));
    }
}

