/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */
package org.aspectj.debugger.gui;

import org.aspectj.debugger.base.*;
import org.aspectj.util.gui.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.util.List;
import java.io.*;

import com.sun.jdi.*;

public class Dialogs {

    public static void showBreakpointDialog() {
        new BreakpointDialog(ComponentRepository.
                             getComponentDirector()).show();
    }

    public static void showSetBreakpointDialog() {
        new SetBreakpointDialog(ComponentRepository.
                                getComponentDirector()).show();
    }

    public static void showClearBreakpointDialog() {
        new ClearBreakpointDialog(ComponentRepository.
                                  getComponentDirector()).show();
    }

    public static void showNewValueDialog(AJValueNode node) {
        new AJNewValueDialog(ComponentRepository.
                             getComponentDirector(), node).popup(node);
    }

    public static void showRunDialog() {
        new RunDialog().show();
    }

    public static void showCatchDialog() {
        new CatchDialog().show();
    }

    public static void showWatchDialog() {
        new WatchDialog().show();
    }    

    public static void showUseDialog() {
        new UseDialog().show();
//        new UseFileChooser().use();
    }

    public static void showDumpDialog() {
        new DumpDialog().show();
    }

    public static void showEvalDialog() {
        new EvalDialog().show();
    }

    public static void showFieldsDialog() {
        new FieldsDialog().show();
    }

    public static void showLocalsDialog() {
        new LocalsDialog().show();
    }

    public static void showPrintDialog() {
        new PrintDialog().show();
    }

    public static String showSuspendDialog() {
        return showThreadDialog("suspend", "Suspend");
    }

    public static String showResumeDialog() {
        return showThreadDialog("resume", "Ruspend");
    }

    public static String showWhereDialog() {
        return showThreadDialog("view", "View");
    }

    public static void showHelpDialog() {

    }

    public static void showExecuteDialog() {
        new ExecuteDialog().show();
    }

    public static void showAboutDialog() {
        String msg = "by the aspectj.org team (C)";
        msg += "\n" + ComponentRepository.getAJDebugger().version();
        int type = JOptionPane.INFORMATION_MESSAGE;
        String title = "About AJBuilder";
        JOptionPane.showMessageDialog
            (ComponentRepository.getComponentDirector(),
             msg,
             title,
             type);
    }

    static String showThreadDialog(String msg, String action) {
        try {
          Iterator iter = ComponentRepository.getAJDebugger().vm().
              allThreads().iterator();
            Vector v = new Vector();
            v.add(new AJToolBar.ThreadPair("All threads", -1));
            while (iter.hasNext()) {
                Object o = iter.next();
                ThreadReference thread =
                    (ThreadReference)iter.next(); //threadInfo.thread;
                String name = thread.name();
                long id = thread.uniqueID();
                AJToolBar.ThreadPair pair = new AJToolBar.ThreadPair(name, id);
                v.add(pair);
            }
            Object[] os = v.toArray();
            AJToolBar.ThreadPair selected =
                (AJToolBar.ThreadPair)
                JOptionPane.showInputDialog
                (ComponentRepository.getComponentDirector(),
                 "Please select a thread to " + msg,
                 action + " a thread",
                 JOptionPane.QUESTION_MESSAGE,
                 null,
                 os,
                 os[0]);
            if (selected != null && selected.id != -1) {
                return selected.id + "";
            }
        } catch (Exception e) {
        }
        return null;
    }

    public void showMonitorDialog() {
        new MonitorDialog(ComponentRepository.getGUIDebugger()).show();
    }

    public static void main(String[] args) {
        String s = showSuspendDialog();
        System.out.println("> " + s);
    }
}

class UseFileChooser extends JFileChooser {
    public UseFileChooser() {
        super(".");
    }

    public javax.swing.filechooser.FileFilter getFileFilter() {
        return new javax.swing.filechooser.FileFilter() {
            public boolean accept(File file) {
                return file.isDirectory();
            }
            public String getDescription() {
                return "Paths to source files";
            }
        };
    }

    public void use() {
        int val = showDialog(null, "Use");
        if (val != APPROVE_OPTION) return;
        String sourcePath = getSelectedFile().getAbsolutePath();
        try {
            ComponentRepository.getAJDebugger().
                useCommand(sourcePath);
        } catch (DebuggerException de) {
        }
    }
}

class UseDialog extends SingleCommandDialog {
    public UseDialog() {
        super("Use", "Please select the source path",
              "use", "Set");
    }
}


class RunDialog extends SingleCommandDialog {
    public RunDialog() {
        super("Run", "Please select a class to run",
              "run", "Run");
    }
}

class CatchDialog extends SingleCommandDialog {
    public CatchDialog() {
        super("Catch", "Please select an exception class to catch",
              "catch", "Catch");
    }
}

class WatchDialog extends SingleCommandDialog {
    public WatchDialog() {
        super("Watch", "Please select a field",
              "watch access", "Watch");
    }
}

class DumpDialog extends SingleCommandDialog {
    public DumpDialog() {
        super("Dump", "Please select an object to dump",
              "dump", "Dump");
    }
}

class PrintDialog extends SingleCommandDialog {
    public PrintDialog() {
        super("Print", "Please select a instance of value to print",
              "print", "Print");
    }
}

class EvalDialog extends SingleCommandDialog {
    public EvalDialog() {
        super("Evaluate", "Please select a instance of value to evaluate",
              "eval", "Evaluate");
    }
}

class FieldsDialog extends SingleCommandDialog {
    public FieldsDialog() {
        super("Fields", "Please select an object",
              "fields", "View Fields");
    }
}

class LocalsDialog extends SingleCommandDialog {
    public LocalsDialog() {
        super("Locals", "Please select an object",
              "locals", "View Locals");
    }
}

class ExecuteDialog extends SingleCommandDialog {
    public ExecuteDialog() {
        super("Execute", "Please enter a command",
              null, "Execute");
    }
}


class SingleCommandDialog extends CenteredJDialog {

    ClassTextField  field       = new ClassTextField(30);
    String          commandStr  = null;

    public SingleCommandDialog(String titleStr, String labelStr,
                               String commandStr, String buttonStr) {
        super(ComponentRepository.getComponentDirector(), titleStr, true);
        this.commandStr = commandStr;
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        JLabel title = new JLabel(labelStr);
        field.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                doStuff();
        }});
        JPanel buttons = new JPanel();
        JButton stuffButton = new JButton(buttonStr);
        stuffButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                doStuff();
        }});
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancel();
        }});
        buttons.add(stuffButton);
        buttons.add(cancelButton);
        panel.add(title);
        panel.add(field);
        panel.add(buttons);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setContentPane(panel);
    }

    private void doStuff() {
        String cmd = field.getText();
        String commandPrefix = commandStr != null ? commandStr + " " : "";
        ComponentRepository.getAJDebugger().execute(commandPrefix + cmd);
        dispose();
    }

    private void cancel() {
        dispose();
    }
}

class MonitorDialog extends CenteredJFrame {

    GUIDebugger guid;

    public MonitorDialog(GUIDebugger guid) {
        this.guid = guid;
    }
}
