/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */
package org.aspectj.debugger.gui;

import com.sun.jdi.*;

public abstract class HexableAJValueNode extends AJValueNode implements Hexable {

    private boolean isHex = false;

    public HexableAJValueNode(AJValueGetter getter, int type) {
        super(getter, type);
    }

    public abstract String getHexValue(Value value);

    public void setHex(boolean isHex) {
        this.isHex = isHex;
    }

    public boolean isHex() {
        return isHex;
    }

    public void toggle() {
        setHex(!isHex());
        ComponentRepository.getThreadGroupTreePane().getTree().reloadAll();
    }

    public String format(Value value) {
        return isHex() ? "0x" + pad(getHexValue(value)) : super.format(value);
    }

    private final static int HEX_LENGTH = 8;
    private String pad(String str) {
        int length = HEX_LENGTH - str.length();
        while (length-- > 0) str = "0"+  str;
        return str;
    }

}
