/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
*
* This file is part of the debugger and core tools for the AspectJ(tm)
* programming language; see http://aspectj.org
*
* The contents of this file are subject to the Mozilla Public License
* Version 1.1 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License at
* either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is AspectJ.
*
* The Initial Developer of the Original Code is Xerox Corporation. Portions
* created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
* All Rights Reserved.
*/
package org.aspectj.debugger.gui;

import java.util.*;

abstract class MethodNode extends    AJTreeNode
                          implements Sourceable,
                                     Breakable,
                                     Advisedable {

    protected MethodHelper helper;

    protected MethodNode(MethodHelper helper) {
        super(AJIcons.METHOD_ICON);
        this.helper = helper;
        setUserObject(helper.getDisplayName());
        setAlwaysExpand(true);
    }

    public String getRealName() {
        return helper.getRealName();
    }

    public String getToolTipText() {
        return showClass() + helper.getToolTipText();
    }

    private String showClass() {
        if (getParent() instanceof AdviceNode) {
            return helper.getClassName() + ".";
        }
        return "";
    }

    public void showOnSource(AbstractSourcePane sourcePane) {
        sourcePane.showSourceForFileAndLine(helper.getRelativePath(),
                                            helper.getLine());
    }

    public String getBreakpoint() {
        return helper.getBreakpoint();
    }

    public abstract List getAdviceStrings();

    public int getType() {
        return helper.getType();
    }

    public static int  getType(boolean isCtr,
                               boolean isStatic,
                               boolean isPublic,
                               boolean isPrivate,
                               boolean isProtected) {
        if (isStatic) {
            if (isPublic)    return AJIcons.METHOD_STATIC_PUBLIC_ICON;
            if (isProtected) return AJIcons.METHOD_STATIC_PROTECTED_ICON;
            if (isPrivate)   return AJIcons.METHOD_STATIC_PRIVATE_ICON;
            else             return AJIcons.METHOD_STATIC_PACKAGE_ICON;
        } else if (isCtr) {
            if (isPublic)    return AJIcons.CTR_PUBLIC_ICON;
            if (isProtected) return AJIcons.CTR_PROTECTED_ICON;
            if (isPrivate)   return AJIcons.CTR_PRIVATE_ICON;
            else             return AJIcons.CTR_PACKAGE_ICON;            
        } else {
            if (isPublic)    return AJIcons.METHOD_PUBLIC_ICON;
            if (isProtected) return AJIcons.METHOD_PROTECTED_ICON;
            if (isPrivate)   return AJIcons.METHOD_PRIVATE_ICON;
            else             return AJIcons.METHOD_PACKAGE_ICON;
        }
    }
}
