/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
*
* This file is part of the debugger and core tools for the AspectJ(tm)
* programming language; see http://aspectj.org
*
* The contents of this file are subject to the Mozilla Public License
* Version 1.1 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License at
* either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is AspectJ.
*
* The Initial Developer of the Original Code is Xerox Corporation. Portions
* created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
* All Rights Reserved.
*/
package org.aspectj.debugger.gui;

import com.sun.jdi.*;
import java.util.*;

public class NonAJCMethodHelper implements MethodHelper {

    private Method method;

    public NonAJCMethodHelper(Method method) {
        this.method = method;
    }

    public String getRealName() {
        return method.name();
    }

    public String getDisplayName() {
        return getRealName() + args();
    }

    public String getToolTipText() {
        return longName();
    }

    public String getClassName() {
        return method.declaringType().name();
    }

    private String longName() {
        return       accessString()
            +  " " + modsString()
            +  " " + method.returnTypeName()
            +  " " + method.name() + args();
    }

    private String modsString() {
        return (method.isAbstract()      ? " abstract"     : "")
            +  (method.isFinal()         ? " final"        : "")
            +  (method.isNative()        ? " native"       : "")
            +  (method.isSynchronized()  ? " synchronized" : "");
    }

    private String accessString() {
        if (method.isPublic())       return "public";
        if (method.isProtected())    return "protected";
        if (method.isPrivate())      return "private";
        return "";
    }

    private String args() {
        String str = "(";
        List names = method.argumentTypeNames();
        if (names != null && names.size() > 0) {
            Iterator iter = names.iterator();
            while (iter.hasNext()) {
                str += iter.next() + (iter.hasNext() ? "," : "");
            }
        }
        str += ")";
        return str;
    }

    public String getRelativePath() {
        return SourceNameFactory.sourceName(method.declaringType());
    }

    public int getLine() {
        Location loc = method.location();
        return loc != null ? loc.lineNumber() : -1;
    }

    public String getBreakpoint() {
        return method.declaringType().name() + "." + method.name() + args();
    }

    public int getType() {
        return MethodNode.getType(method.isConstructor(),
                                  method.isStatic(),
                                  method.isPublic(),
                                  method.isPrivate(),
                                  method.isProtected());        
    }
}
