/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.undo.*;

import com.sun.jdi.event.Event;

/**
 * Used in debugging.
 */
public class Util {

    private static UtilFrame util = null;
    private static boolean debug = false;

    public static void init() {
        if (util != null) return;
        util = new UtilFrame();
        util.pack();
        util.setVisible(true);
    }

    public static Container panel() {
        if (util != null) return util.getContentPane();
        util = new UtilFrame();
        return util.getContentPane();
    }

    public static void error(Object o) {
        if (check()) util.error(o);
    }

    public static void debug(Object o) {
        if (check()) util.debug(o);
    }

    public static void ex(Object t) {
        if (check()) util.ex(t);
    }

    public static void aspect(Object o) {
        if (check()) util.aspect(o);
    }

    public static void msg(Object o, Color c) {
        if (check()) util.msg(o, c);
    }

    public static void msg(String s, Object o, Color c) {
        if (check()) util.msg(s, o, c);
    }

    public static void setDebugging(boolean _debug) {
        debug = _debug;
    }

    private static boolean check() {
        if (!debug) {
            return false;
        }
        if (util == null) {
            init();
        }
        return true;
    }
}

class UtilFrame extends JFrame implements ItemListener {

    JTextPane textPane;
    JCheckBox debugBox;
    boolean debug = true;
    DefaultStyledDocument dsd;
    SimpleAttributeSet[] attrs = null;
    int count = 0;

    static final int MSG        = -1;
    static final int ERROR      =  0;
    static final int DEBUG      =  1;
    static final int ASPECT     =  2;
    static final int EXCEPTION  =  3;

    Color[] colors =        {
                                Color.red,
                                Color.blue,
                                Color.green.darker().darker(),
                                Color.darkGray,
                            };
    int[] colorsConstants = {
                                ERROR,
                                DEBUG,
                                ASPECT,
                                EXCEPTION,
                            };


    public UtilFrame() {
        super("Utilily Frame");

        dsd = new DefaultStyledDocument();

        textPane = new JTextPane(dsd);
        JScrollPane scrollPane = new JScrollPane(textPane);
        scrollPane.setPreferredSize(new Dimension(500, 500));

        debugBox = new JCheckBox("Debug?");
        debugBox.setSelected(true);
        debugBox.addItemListener(this);

        JPanel contentPane = new JPanel(new BorderLayout());
        contentPane.add(scrollPane, BorderLayout.CENTER);
        contentPane.add(debugBox, BorderLayout.NORTH);
        setContentPane(contentPane);

        initAttrs();
    }

    public void itemStateChanged(ItemEvent e) {
        Object source = e.getItemSelectable();
        if (source == debugBox) {
            if (debug == true) {
                debug = false;
            } else {
                debug = true;
            }
        }
    }


    protected void initAttrs() {
        attrs = new SimpleAttributeSet[colors.length];
        for (int i = 0; i < colors.length; i++) {
            attrs[i] = getAttr(colors[i]);
        }
    }

    protected SimpleAttributeSet getAttr(Color c) {
        SimpleAttributeSet attr = new SimpleAttributeSet();
        StyleConstants.setForeground(attr, c);
        StyleConstants.setFontSize(attr, 10);
        return attr;
    }

    public void error(Object o) {
        msg("ERROR", o, colors[ERROR]);
    }

    public void debug(Object o) {
        msg("DEBUG", o, colors[DEBUG]);
    }

    public void aspect(Object o) {
        msg("ASPECT", o, colors[ASPECT]);
    }

    public void ex(Object o) {
        if (o instanceof Exception) {
            msg("EXCEPTION", ((Exception) o).getMessage(), colors[EXCEPTION]);
        } else {
            msg("EXCEPTION", o, colors[EXCEPTION]);
        }
    }

    public void msg(Object o, Color c) {
        msg("MSG", o, c);
    }

    public void msg(String m, Object o, Color c) {
        String s = "NULL";
        if (o != null) s = o.toString();
        outln("<<" + m + ">> " + s, c);
    }

    private void outln(String s, Color c) {
        outln(s, getAttr(c));
    }

    private void outln(String s, int i) {
        outln(s, attrs[i]);
    }

    private void outln(String s, SimpleAttributeSet attr) {
        if (!debug) {
            return;
        }
        try {
            dsd.insertString(dsd.getLength(), s + "\n", attr);
            count += s.length() + 1;
            textPane.setCaretPosition(count);
        } catch (Exception e) {}
    }
}
