/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import com.sun.jdi.*;
import com.sun.jdi.event.*;
import com.sun.jdi.request.*;
import org.aspectj.debugger.base.*;
import org.aspectj.debugger.request.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class VariablesTreePane
    extends AJPanel
    implements StopListener
{

    private VariablesTree tree = null;

    public VariablesTreePane(Frame frame, GUIDebugger guid) {
        super(guid);
        debugger().addStopListener(this);
        setLayout(new BorderLayout());
        add(new JScrollPane(tree = new VariablesTree(frame)),
            BorderLayout.CENTER);

    }

    /****************************** StopListener ******************************/
    public void accessWatchpointEvent(AccessWatchpointEvent e) {
        updateLocals();
    }
    public void breakpointEvent(BreakpointEvent e) {
        updateLocals();
    }
    public void exceptionEvent(ExceptionEvent e) {
        updateLocals();
    }
    public void modificationWatchpointEvent(ModificationWatchpointEvent e) {
        updateLocals();
    }
    public void stepEvent(StepEvent e) {
        updateLocals();
    }

    private DefaultMutableTreeNode getRoot() {
        return (DefaultMutableTreeNode) tree.getModel().getRoot();
    }

    private void newRoot() {
        model().setRoot(new VariablesRootTreeNode());
    }

    private DefaultTreeModel model() {
        return (DefaultTreeModel) tree.getModel();
    }
    
    private void updateLocals() {
        newRoot();
        List locals = debugger().locals();
        Iterator iter;
        iter = locals.iterator();
        while (iter.hasNext()) {
            try {
                add(AJValueNodeFactory.make((LocalVariable) iter.next(),
                                            debugger.getDefaultFrame()));
            } catch (Exception e) { AJUtil.ex(e);
            }
        }
        try {
            ObjectReference oref = debugger().getDefaultThisObject();
            ReferenceType refType = debugger().getDefaultFrame().location().declaringType();
            List fields = refType.fields();
            iter = fields.iterator();
            while (iter.hasNext()) {
                Field field = (Field) iter.next();
                if (shouldAddField(field)) {
                    if (oref != null) {
                        add(AJValueNodeFactory.make(oref, field));
                    } else if (field.isStatic()) {
                        add(AJValueNodeFactory.make(refType, field));
                    }
                }
            }
        } catch (Exception e) { AJUtil.ex(e);
        }
        tree.expandRow(0);
    }

    private boolean shouldAddField(Field field) {
        return field != null && (field.name().indexOf("$ajc") == -1);
    }

    private void add(MutableTreeNode node) {
        boolean expand = getRoot().getChildCount() == 0;
        getRoot().add(node);
        // This was messing it up some how...
        //if (expand) tree.expandRow(0);
    }

    public static String d() { return "Variables Tree Pane"; }
    public String toString() { return d(); }    
}

class VariablesTree extends AJTree {
    private Frame frame = null;
    public VariablesTree(Frame frame) {
        super(new VariablesTreeModel());
        this.frame = frame;
        setCellRenderer(new VariablesTreeCellRenderer());
        addTreeWillExpandListener(new NoCollapsingRootExpansionListener(this));
    }
    public void clear() {
        setModel(new VariablesTreeModel());
    }
}

class VariablesTreeCellRenderer extends AJTreeCellRenderer {    
}

class VariablesTreeModel extends DefaultTreeModel {
    public VariablesTreeModel() {
        super(new VariablesRootTreeNode());
    }
}

class VariablesRootTreeNode extends AJRootNode {
    public VariablesRootTreeNode() {
        super(Titles.VARIABLES, AJIcons.VARIABLES_ICON);
    }
}



class VariablesTreeNode extends AJTreeNode {
    public VariablesTreeNode(Object o) {
        super(AJIcons.BAD_ICON);
    }
}
