/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.gui;

import com.sun.jdi.*;
import com.sun.jdi.event.*;
import com.sun.jdi.request.*;
import org.aspectj.debugger.base.*;
import org.aspectj.debugger.request.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class WatchpointTreePane
    extends AJPanel
    implements DebuggerListener
{

    private WatchpointTree tree = null;

    public WatchpointTreePane(Frame frame, GUIDebugger guid) {
        super(guid);
        debugger().addDebuggerListener(this);
        setLayout(new BorderLayout());
        add(new JScrollPane(tree = new WatchpointTree(frame)),
            BorderLayout.CENTER);

    }

    /************************* DebuggerListener *************************/

    public void requestSetEvent(RequestEvent re) {
        Request r = re.getRequest();
        if (!(r instanceof WatchRequest)) return;
        add((WatchRequest) r, true);
    }

    public void requestClearEvent(RequestEvent re) {
        Request r = re.getRequest();
        if (!(r instanceof WatchRequest)) return;
        remove((WatchRequest) r);
    }

    public void requestDeferredEvent(RequestEvent re) {
        Request r = re.getRequest();
        if (!(r instanceof WatchRequest)) return;
        add((WatchRequest) r, false);
    }

    public void requestFailedEvent(RequestEvent re) {
        Request r = re.getRequest();
        if (!(r instanceof WatchRequest)) return;
        remove((WatchRequest) r);
    }

    private void remove(WatchRequest wr) {
    }

    private DefaultMutableTreeNode getRoot() {
        return (DefaultMutableTreeNode) tree.getModel().getRoot();
    }
    
    private void add(WatchRequest wr, boolean isVerified) {
        WatchpointTreeNode node = null;
        DefaultMutableTreeNode root = getRoot();
        Enumeration enum = root.children();
        while(enum.hasMoreElements()) {
            node = (WatchpointTreeNode) enum.nextElement();
            if (node.toString().equals(wr+"")) {
                return;
            }
        }
        node = new WatchpointTreeNode(wr);
        root.add(node);
        tree.expandRow(0);
    }

    public static String d() { return "Watchpoint Tree Pane"; }
    public String toString() { return d(); }    
}

class WatchpointTree extends JTree {
    private Frame frame = null;
    public WatchpointTree(Frame frame) {
        super(new WatchpointTreeModel());
        this.frame = frame;
        setCellRenderer(new WatchpointTreeCellRenderer());
        addTreeWillExpandListener(new NoCollapsingRootExpansionListener(this));
    }
}

class WatchpointTreeCellRenderer extends AJTreeCellRenderer {    
}

class WatchpointTreeModel extends DefaultTreeModel {
    public WatchpointTreeModel() {
        super(new WatchpointRootTreeNode());
    }
}

class WatchpointRootTreeNode extends AJRootNode {
    public WatchpointRootTreeNode() {
        super(Titles.WATCHES, AJIcons.WATCHES_ICON);
    }
}

class WatchpointTreeNode extends AJTreeNode {
    private WatchRequest wr;
    public WatchpointTreeNode(WatchRequest wr) {
        super(AJIcons.WATCHVAR_ICON);
        this.wr = wr;
    }
    public String toString() {
        return wr + "";
    }
}
