/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
package org.aspectj.debugger.ide;

import org.aspectj.debugger.base.DebuggerException;
import org.aspectj.debugger.base.NoVMException;
import org.aspectj.debugger.gui.AJUtil;
import org.aspectj.debugger.gui.ComponentDirector;
import java.io.File;
import java.util.*;

public class IDEComponentDirector extends ComponentDirector {

    public static IDEComponentDirector proto;

    protected IDEInterface ide;

    public IDEComponentDirector(String title, String mode, IDEInterface ide) {
        super(title, mode);
        if (ide == null) {
            ide = new IDEInterfaceImpl();
        }
        this.ide = ide;
    }

    public void go() {
        if (proto != null) {
            AJUtil.info("Only one debugger may run at once");
            getGUIDebugger().exit();
            return;
        }
        proto = this;
        super.go();
    }

    public /*abstract*/ void showBreakpointDialog() {

    }

    public void finishUp() {
        setBreakpoints();
    }

    public void title(String msg) {}
    public final void title() {
        title("");
    }

    public IDEInterface getIde() {
        return ide;
    }

    protected void setBreakpoints() {
        ((IDEInterfaceHelper)getIde()).settingBreakpointsFromIDE();
        try {
            setBreakpoints_();
        } catch (Exception t) {
            getGUIDebugger().getDebugger().handle(t);
        } finally {
            ((IDEInterfaceHelper)getIde()).doneSettingBreakpointsFromIDE();   
        }
    }

    protected void setBreakpoints_() {
        Iterator iter = ide.getInitialBreakpoints().iterator();
        while (iter.hasNext()) {
            IDEInterface.SourceLineBreakable request = (IDEInterface.SourceLineBreakable)iter.next();
            try {
                //String source = new File(request.getSourceName()).getAbsolutePath();
                String source = request.getSourceName();
                int line = request.getLine();
                getDebugger().stopOnCommand(source, line);
            } catch (DebuggerException de) {
                getGUIDebugger().handleDebuggerException(de);
            }
        }
    }

    public void shutDown() {
        proto = null;
        super.shutDown();
    }
}
