/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
package org.aspectj.debugger.ide;

import java.io.File;
import java.util.List;
import java.util.Vector;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

/**
 * A simple implementation of IDEInterface.
 *
 * @see IDEInterface
 * @author Jeff Palm
 */
public class IDEInterfaceImpl extends IDEInterfaceHelper {

    public final static IDEInterface proto = new IDEInterfaceImpl();

    /**
     * Not implemented.
     */
    public void toggle(String sourceName, int lineNumber) { }

    public String getClasspath() {
        return ""; //get.classpath";
    }

    public String getVMParameters() {
        return "" ;//get.vm.parameters";
    }

    public String getClassNameToRun() {
        return "spacewar.Game"; //get.class.name.to.run";
    }

    public String findClassName() {
        return ""; //find.class.name";
    }

    public String findFileName() {
        JFileChooser chooser = new JFileChooser(".");
        String fileName = "bad.file.name";
        chooser.addChoosableFileFilter(new FileFilter() {
                public boolean accept(File f) {
                    return f != null && (f.getName().endsWith(".java") || f.isDirectory());
                }
                public String getDescription() {
                    return "Java files (*.java)";
                }
            });
        int result = chooser.showDialog(null, "OK");
        System.out.println("result=" + result);
        if (result == 0) {
            fileName = chooser.getSelectedFile().getAbsolutePath();
        }
        return fileName;
    }

    public List getInitialBreakpoints() {
        return new Vector();
    }

    protected void markLine(String className, List argumentTypeNames) {
        System.out.println("className:" + className + " argumentTypeNames:" + argumentTypeNames);
    }

    protected void markLineWithClass(String className, int line) {
        System.out.println("markLineWithClass:className=" + className);
    }

    protected void markLine(String sourceName, int line) {
        System.out.println("markLine() sourceName:" + sourceName + ",line:" + line);
    }

    protected void unmarkLine(String sourceName, int line) {
        System.out.println("unmarkLine() sourceName:" + sourceName + ",line:" + line);
    }

    protected void handle(Throwable t) {
        t.printStackTrace();
    }
}
