/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.ide;

public class Main {

    /**
     * Override this to return a FullPathSourceShower.
     * The methods to override are:
     *  - public void showSourceForFullPath(String fullPath);
     *  - public void showLineForCurrentModel(int line, boolean isAtBreakpoint);
     *  - public String getSourceName();
     */
    SourceShower getShower() {
        if (shower != null) {
            try {
                return (SourceShower) Class.forName(shower).newInstance();
            } catch (Throwable t) {
                t.printStackTrace();
            }
        }
        return FullPathSourceShower.proto;
    }

    IDEInterface getIDE() {
        if (ide != null) {
            try {
                IDEInterface i = (IDEInterface) Class.forName(ide).newInstance();
                System.out.println("i:" + i);
                return i;
            } catch (Throwable t) {
                t.printStackTrace();
            }
        }
        return null;
    }

    String mainClass;
    String classPath;
    String sourcePath;
    String mode;
    String shower;
    String ide;
    boolean verbose = false;

    public static void main(String[] args) {
        new Main().realMain(args);
    }


    public void realMain(String[] args) {
        parseArgs(args);
        IDEDebugger.launch(mainClass, sourcePath, classPath, mode, new String[0], getShower(), getIDE());
    }

    void parseArgs(String[] args) {
        String arg = "";
        String val = "";
        for (int i = 0; i < args.length; i++) {
            arg = args[i];
            if (arg.equals("-mainclass")) {
                if (check("mainclass", args, i)) mainClass = args[++i];
            } else if (arg.equals("-classpath")) {
                if (check("classpath", args, i)) classPath = args[++i];
            } else if (arg.equals("-sourcepath")) {
                if (check("sourcepath", args, i)) sourcePath = args[++i];
            } else if (arg.equals("-mode")) {
                if (check("mode", args, i)) mode = args[++i];
            } else if (arg.equals("-shower")) {
                if (check("shower", args, i)) shower = args[++i];
            } else if (arg.equals("-ide")) {
                if (check("ide", args, i)) ide = args[++i];                
            } else if (arg.equals("-verbose")) {
                verbose = true;
            }
        }
    }

    boolean check(String arg, String[] args, int i) {
        if (args == null || args.length < i || args[i+1] == null) {
            exit("bad argument to " + arg + " i=" + i + " length=" + args.length);
            return false;
        }
        return true;
    }

    void exit(String msg) {
        System.err.println(msg);
        System.exit(1);
    }
}
