/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.ide;

import org.aspectj.debugger.base.*;
import org.aspectj.debugger.gui.*;
import org.aspectj.debugger.gui.GUIDebugger;
import org.aspectj.debugger.request.*;

/**
 * SourcePane.java
 *
 *
 * Created: Mon Sep 18 11:08:18 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public class SourcePane extends AbstractSourcePane {

    private SourceShower shower;
    private ComponentDirector director;

    public SourcePane(GUIDebugger guid, SourceShower shower) {
        super(guid);
        this.shower = shower;
        this.director = guid.getGui();
    }

    public boolean showSource(String relativePath) {
        return shower.showSource(relativePath);
    }

    public void showLineForCurrentModel(int line, boolean isAtBreakpoint) {
        shower.showLineForCurrentModel(line, isAtBreakpoint);
    }

    public String getSourceName() {
        return shower.getSourceName();
    }

    public void requestSet(String filename, int line, BreakpointRequestAction ba) {
        showSourceForFileAndLine(filename, line);
    }

    public void requestClear(String filename, int line, BreakpointRequestAction ba) {
        showSourceForFileAndLine(filename, line);
    }

    public void requestDeferred(String filename, int line, BreakpointRequestAction ba) {
        showSourceForFileAndLine(filename, line);
    }
}
