/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import com.sun.jdi.request.*;
import java.util.*;

/**
 * AbstractStopRequest.java
 *
 *
 * Created: Fri Aug 25 11:23:46 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public abstract class AbstractStepRequest extends Request {
    public AbstractStepRequest (Debugger debugger) {
        super(debugger);
    }

    public Object go() throws NoVMException, DebuggerException {
        EventRequestManager mgr = null;
        try {
            mgr = vm().eventRequestManager();
        } catch (NoVMException nvme) {
            return null;
        } catch (NullPointerException npe) {
            return null;
        }
        ThreadReference threadRef = dbg().getStoppedThread();
        List steps = mgr.stepRequests();
        for (int i = 0; i < steps.size(); i++) {
            StepRequest request = (StepRequest) steps.get(i);
            if (request.thread().equals(threadRef)) {
                mgr.deleteEventRequest(request);
            }
        }
        StepRequest request = mgr.createStepRequest(threadRef,
                                                    getSize(),
                                                    getDepth());
        request.putProperty("name", name());
        request.addCountFilter(1);
        request.setSuspendPolicy(EventRequest.SUSPEND_ALL);
        request.enable();
        dbg().addExcludesTo(request);
        //vm().resume();
        //dbg().resume(threadRef);
        dbg().resumeUntilZero(threadRef);
        return request;
    }

    abstract int getDepth();
    abstract int getSize();
    abstract String name();
}
