/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;
import org.aspectj.debugger.gui.*;
import org.aspectj.debugger.ide.IDEInterface;
import org.aspectj.tools.ide.*;
import java.io.File;
import com.sun.jdi.*;
import com.sun.jdi.event.*;
import com.sun.jdi.request.*;

/**
 * BreakpointRequestAction.java
 *
 *
 * Created: Wed Aug 30 08:55:19 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public abstract class BreakpointRequestAction
    extends RequestAction
    implements IDEInterface.SourceLineBreakable
{

    private Location location;

    public BreakpointRequestAction(Debugger debugger) {
        super(debugger);
        if (!isSetting()) {
            debugger.removeStopListener(this);
        }
    }

    public boolean isBreakpoint() {
        return true;
    }

    public abstract int    getLine();
    public abstract String getSourceName();
    public abstract String getProto();

    public int getRealLine() {
        return getLine();
    }
    public String getRealSourceName() {
        return getSourceName();
    }

    protected void setLocation(Location location) {
        this.location = location;
    }

    protected BreakpointRequest request(Location location) throws NoVMException {
        setLocation(location);
        BreakpointRequest req = (BreakpointRequest)getRequest();
        if (isSetting()) {
            req = vm().eventRequestManager().createBreakpointRequest(getLocation());
            req.setSuspendPolicy(EventRequest.SUSPEND_ALL);
            req.enable();
        } else if (req != null) {
            vm().eventRequestManager().deleteEventRequest(req);
        }
        return req;
    }

    public Location getLocation() {
        return location;
    }

    public String toString() {
        return "breakpoint " + getProto();
    }

    public SourceLine sourceLine() {
        return null;
    }

    public boolean equals(Object o) {
        if (o instanceof AJStackFrameFormatter.MethodAndSource) {
            AJStackFrameFormatter.MethodAndSource ms = (AJStackFrameFormatter.MethodAndSource)o;
            return ms != null && getProto().equals(ms.getProto());
        }
        if (o instanceof BreakpointRequestAction) {
            BreakpointRequestAction bra = (BreakpointRequestAction)o;
            return
//                  getRealSourceName() != null && bra.getRealSourceName() != null &&
                getRealSourceName().equals(bra.getRealSourceName()) &&
                getRealLine() == bra.getRealLine();
        }
        return super.equals(o);
    }

    public void breakpointEvent(BreakpointEvent e) { print(e); }
}
