/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import java.util.*;

/**
 * FieldNeedingRequest.java
 *
 *
 * Created: Fri Aug 25 16:41:56 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public abstract class ClassAndFieldNeedingRequestAction extends RequestAction {

    protected String className;
    protected String fieldName;

    public ClassAndFieldNeedingRequestAction(Debugger debugger, String className, String fieldName) {
        super(debugger);
        this.className = className;
        this.fieldName = fieldName;
    }

    protected Field getField() throws NoVMException, DebuggerException {
        try {
            return getReferenceType().fieldByName(fieldName);
        } catch (ClassNotPreparedException cnpe) {
            throw new DebuggerException(cnpe);
        }
    }

    protected ReferenceType getReferenceType() throws NoVMException, DebuggerException {
        List refTypes = vm().classesByName(className);
        if (refTypes.size() == 0) {
            throw new InvalidClassException(className);
        } else if (refTypes.size() > 1) {
            throw new AmbiguousClassException(className, refTypes);
        }
        return (ReferenceType) refTypes.get(0);
    }

    public String getClassName() {
        return className;
    }

    public String getFieldName() {
        return fieldName;
    }

    public boolean equals(Object other) {
        if (other == null || !(other instanceof ClassAndFieldNeedingRequestAction)) {
            return super.equals(other);
        }
        ClassAndFieldNeedingRequestAction ra = (ClassAndFieldNeedingRequestAction) other;
        return    (getClassName().equals(ra.getClassName())
                && getFieldName().equals(ra.getFieldName()));
    }

    public String toString() {
        return getClassName() + "." + getFieldName();
    }
}
