/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import com.sun.jdi.request.*;
import com.sun.jdi.event.*;
import java.util.*;

/**
 * ClassBreakpointRequestAction.java
 *
 *
 * Created: Wed Aug 23 11:26:39 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public abstract class ClassBreakpointRequestAction extends BreakpointRequestAction {

    protected String className;

    ClassBreakpointRequestAction(Debugger debugger, String className) {
        super(debugger);
        this.className = className;
    }

    EventRequest resolve(ReferenceType refType)
        throws MultipleLocationsException,
               UnableToSetRequestException {
        BreakpointRequest request = null;
        try {
            if (refType.name().equals(className)) {
                EventRequestManager em = vm().eventRequestManager();
                Location location = findLocation();
                if (location == null) {
                    return null;
                }
                request = request(location);
            }
        } catch (NoVMException e) {
        }
        return request;
    }

    public String getRealSourceName() {
        return ajdbg().getFullSourcePathFromAJCClass(getClassName());
    }

    public String getClassName() {
        return className;
    }

    public int getLine() {
        if (getLocation() != null) {
            return getLocation().lineNumber();
        }
        return -1;
    }

    public String getSourceName() {
        if (getLocation() != null) {
            try {
                return getLocation().sourceName();
            } catch (AbsentInformationException aie) {
            }
        }
        return "<not-available>";
    }

    abstract Location findLocation()
        throws NoVMException,
               MultipleLocationsException,
               UnableToSetRequestException;
}
