/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import com.sun.jdi.request.*;
import com.sun.jdi.event.*;
import org.aspectj.tools.ide.SourceLine;
import java.util.*;

import org.aspectj.util.LineNumberTableMapper;
import java.io.File;

/**
 * ClassLineBreakpointRequestAction.java
 *
 *
 * Created: Wed Aug 23 11:27:47 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public abstract class ClassLineBreakpointRequestAction extends ClassBreakpointRequestAction {

    private int line;

    ClassLineBreakpointRequestAction(Debugger debugger, String className, int line) {
        super(debugger, className);
        System.err.println(">> breakpoint request, class: " + className + ", line: " + line);
        this.line  = line;
    }

    public String getProto() {
        SourceLine sl = ajdbg().getSourceLineFromClass(getClassName(), getLine());
        int newLine = line;
        if (sl == null) {
            System.err.println("sl is null for " + getClassName() + ":" + line);
        } else {
            newLine = sl.line;
        }
        return getClassName() + ":" + newLine;
    }

    public String getErrorMessage() {
        return "No code at line " + line + " in " + getClassName();
    }

    Location findLocation()
        throws NoVMException,
               MultipleLocationsException,
               UnableToSetRequestException {

        System.err.println(">>> resolving class-line breakpoint, class: " + className);

//        String fullName = loc.declaringType().name();
//        System.err.println(">>> pkg: " + pkg + ",);


//        System.err.println(">>>> " + vm().classesByName(className));

        try {

            Iterator it = vm().classesByName(className).iterator();
            ReferenceType refType = null;
            while (it.hasNext()) {
                refType = (ReferenceType) it.next(); // HACK
                System.err.println(">> refType: " + refType.sourceName());
            }
            if (refType != null) {
                String attrib = refType.sourceName();
                String pkg = className.substring(0, className.lastIndexOf('.')).replace('.', '/');
                String srcPath = org.aspectj.debugger.base.AJDebugger.INSTANCE.getSourcePath();
                File srcRoot = new File(srcPath);

                System.err.println(">> findMethod attrib: " + attrib + ", pkg: " + pkg + ", src: " + srcPath);

                LineNumberTableMapper mapper = new LineNumberTableMapper(attrib, pkg, srcRoot);
                System.err.println(">>> mapper: " + mapper.toString());
                System.err.println(">> mapped: " + mapper.getCorrespondingFile(line).getPath() + " : " + mapper.getCorrespondingLineNumber(line));

                Iterator linesIter = refType.allLineLocations().iterator();
                //System.err.println("line: " + line + " -> " + );
//                List lines = refType.locationsOfLine(mapper.getCorrespondingLineNumber(line));
//                Iterator linesIter = lines.iterator();
                while (linesIter.hasNext()) {
                    Location loc = (Location)linesIter.next();
                    if (loc.lineNumber() == mapper.getCorrespondingLineNumber(line)) {
                        return loc;
                    }
                }
//                    return (Location) linesIter.next();
//                } else {
//                    System.err.println("> ERROR: no lines");
//                    throw new AbsentInformationException();
//                }
            } else {
                throw new AbsentInformationException();
            }
        } catch (AbsentInformationException aie) {
            System.err.println("> ERROR: in class line breakpoint request");
            aie.printStackTrace();
        }
        return null;
//        Location location = null;
//        Iterator iter = vm().classesByName(getClassName()).iterator();
//        while (iter.hasNext()) {
//            ReferenceType refType = (ReferenceType) iter.next();
//            if (refType.name().equals(getClassName())) {
//                try {
//                    List lines = refType.locationsOfLine(getLine());
//                    Iterator linesIter = lines.iterator();
//                    while (linesIter.hasNext()) {
//                        return (Location) linesIter.next();
//                    }
//                } catch (AbsentInformationException e) {
//                    return null;
//                }
//                throw new UnableToSetRequestException(this);
//            }
//        } System.out.println(this + ".location=" + location);
//        return location;
    }

    public int getLine() {
        return line;
    }

    public boolean equals(Object other) {
        if (other == null || !(other instanceof ClassLineBreakpointRequestAction)) {
            return super.equals(other);
        }
        ClassLineBreakpointRequestAction ra = (ClassLineBreakpointRequestAction) other;
        return (getClassName().equals(ra.getClassName()) &&
                getLine() == ra.getLine());
    }
}
