/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.request;

import com.sun.jdi.*;
import com.sun.jdi.event.*;
import org.aspectj.debugger.base.*;

public class ClassTextPrintingRequest extends TextPrintingRequest implements ClassListener {

    public final static int PREPARE = 0x01;
    public final static int UNLOAD = 0x10;
    public final static int PREPARE_OR_UNLOAD = 0x11;

    protected int mode;
    protected String className;

    public ClassTextPrintingRequest(Debugger debugger, String className, int mode, String text) {
        super(debugger, text);
        if (mode < PREPARE || mode > PREPARE_OR_UNLOAD) {
            throw new IllegalArgumentException
                ("mode:" + mode + " where " + PREPARE + "<=mode<=" + PREPARE_OR_UNLOAD);
        }
        this.mode = mode;
        this.className = className;
    }

    public Object go() throws NoVMException, DebuggerException {
        debugger.addClassListener(this);
        return text;
    }

    public void classPrepareEvent(ClassPrepareEvent e) {
        if ((mode & PREPARE) != 0 && e.referenceType().name().equals(className)) print();
    }
    
    public void classUnloadEvent(ClassUnloadEvent e) {
        if ((mode & UNLOAD) != 0 && e.className().equals(className)) print();
    }
}
