/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved. 
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import com.sun.jdi.request.*;
import com.sun.jdi.event.*;

/**
 * ExceptionRequestAction.java
 *
 *
 * Created: Mon Aug 28 08:52:36 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public abstract class ExceptionRequestAction extends RequestAction {

    protected String className;    
    
    ExceptionRequestAction(Debugger debugger, String className) {
        super(debugger);
        this.className = className;
    }
    
    EventRequest resolve(ReferenceType refType) throws MultipleLocationsException {
        ExceptionRequest request = null;
        try {
            if (refType.name().equals(className)) {
                EventRequestManager em = vm().eventRequestManager();
                request = em.createExceptionRequest(refType,
                                                    notifyCaught(),
                                                    notifyUncaught());
                //request.addCountFilter(1);      
                request.enable();
                request.setSuspendPolicy(EventRequest.SUSPEND_ALL); 
            }
        } catch (NoVMException e) {
        } 
        return request;
    }

    public String getClassName() {
        return className;
    }

    abstract boolean notifyCaught();
    abstract boolean notifyUncaught();

    public boolean equals(Object other) {
        if (other == null || !(other instanceof ExceptionRequestAction)) {
            return super.equals(other);
        }
        ExceptionRequestAction ra = (ExceptionRequestAction) other;
        return getClassName().equals(ra.getClassName());
    }

    public void exceptionEvent(ExceptionEvent e) {
        print(e);
    }
}
