/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import java.io.*;
import java.util.*;

/**
 * ReadRequest.java
 *
 *
 * Created: Tue Aug 29 13:58:02 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public class ReadRequest extends Request {

    private String fileName;
    private boolean silent;

    public ReadRequest(Debugger debugger, String fileName, boolean silent) {
        super(debugger);
        this.fileName = fileName;
        this.silent = silent;
    }

    public ReadRequest(Debugger debugger, String fileName) {
        this(debugger, fileName, false);
    }

    public Object go() throws NoVMException, DebuggerException {
        List commands = new Vector();
        FileReader fileReader = null;
        File file;
        String filePath = fileName;
        try {            
            file = getFile();
//            filePath = file.getCanonicalPath();
            filePath = file.getAbsolutePath();
            fileReader = new FileReader(file);
        } catch (IOException ioe) {
            throw new DebuggerException("Invalid file name '" + fileName + "'.");
        }
        BufferedReader in = new BufferedReader(fileReader);
        String line = "";
        try {
            if (!silent) {
                debugger.outln("*** Reading commands from " + filePath);
            }
            while ((line = in.readLine()) != null) {
                Object o;
                if (!isComment(line)) {
                    commands.add(line);
                    if (!silent) {
                        o = debugger.execute(line);
                    }
                }
            }
        } catch (IOException ioe) {
        }

        return commands;
    }

    private boolean isComment(String line) {
        line = line.trim();
        if (line.startsWith("#")) {
            return true;
        }
        return false;
    }

    private File getFile() throws IOException {
        Iterator iter = getDirs().iterator();
        File file = new File(fileName);
        if (file.exists()) {
            return file.getAbsoluteFile();
        }
        while (iter.hasNext()) {
            String dir = iter.next() + "";
//            try {
                file =  new File(dir + File.separator + fileName);
                if (file.exists()) {
//                    return file.getCanonicalFile();
                    return file.getAbsoluteFile();
                }
//            } catch (IOException ioe) {
//            }
        }
        throw new IOException("File not found '" + fileName + "'.");

    }

    private List getDirs() {
        Vector dirs = new Vector();
        dirs.add(".");
        try {
            dirs.add(System.getProperty("user.home") + "");
        } catch (SecurityException se) {
        }
        try {
            dirs.add(System.getProperty("user.dir") + "");
        } catch (SecurityException se) {
        }
        dirs.add(debugger.getSourceManager().getSourcePath());
        return dirs;
    }
}
