/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;

/**
 * Request.java
 *
 *
 * Created: Wed Aug 23 14:06:43 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public abstract class Request  {

    protected Debugger debugger;

    protected boolean debug = true;
    protected void db(Object o) {
        if (debug)
            System.out.println("... " + o);
    }

    public Request(Debugger debugger) {
        this.debugger = debugger;
    }

    public Request() {
        this(null);
    }

    public abstract Object go() throws NoVMException, DebuggerException;

    public /*abstract*/ String getErrorMessage() {
        return "no error message";
    }

    protected VirtualMachine vm() throws NoVMException {
        return debugger.getVM();
    }

    protected Debugger dbg() {
        return debugger;
    }

    protected AJDebugger ajdbg() {
        return (AJDebugger) debugger;
    }

    protected boolean isEmpty(String str) {
        return str == null || str.trim().equals("");
    }

    public void setDebugger(Debugger debugger) {
        this.debugger = debugger;
    }

    public Debugger getDebugger() {
        return debugger;
    }
}
