/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved. 
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import com.sun.jdi.event.*;
import java.util.*;

/**
 * RequestList.java
 *
 *
 * Created: Tue Aug 22 10:42:55 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public class RequestList
    implements ClassListener, VMListener {

    /** The list of events. */
    private List requests;
    
    /** The debugger that will generate the events. */
    private Debugger debugger;
    
    public RequestList(Debugger debugger) {
        this.debugger = debugger;
        debugger.addClassListener(this);
        debugger.addVMListener(this);
        requests = Collections.synchronizedList(new Vector());        
    }

    private void add(EventRequestWrapper wrapper) {
        System.out.println("Adding wrapper=" + wrapper);
        requests.add(wrapper);
    }
    
    public EventRequestWrapper remove(EventRequestWrapper wrapper) {
        if (requests.remove(wrapper)) {
            return wrapper;
        }
        return null;
    }

    /* Requests */
    public /*private*/ ClassLineBreakpointRequestWrapper createBreakpointRequest(String className, int line) {
        ClassLineBreakpointRequestWrapper wrapper =
            new ClassLineBreakpointRequestWrapper(debugger, className, line);
        add(wrapper);
        resolveAgainstAllClasses(wrapper);
        return wrapper;
    }    

    boolean resolveAgainstAllClasses(EventRequestWrapper wrapper) {
        try {
            Iterator iter = debugger.getVM().allClasses().iterator();
            while (iter.hasNext()) {
                if (wrapper.resolve((ReferenceType) iter.next()) != null) {
                    remove(wrapper);
                    return true;
                }
            }
        } catch (NoVMException e) {
        }
        return false;
    }

    boolean resolveAgainstClass(EventRequestWrapper wrapper, ReferenceType refType) {
        if (wrapper.resolve(refType) != null) {
            remove(wrapper);
            return true;
        }
        return false;
    }


    /* ClassListener */
    public void classPrepareEvent(ClassPrepareEvent e) {
        for (int i = 0; i < requests.size(); i++) {
            resolveAgainstClass((EventRequestWrapper) requests.get(i), e.referenceType());
        }
    }
    public void classUnloadEvent(ClassUnloadEvent e) {
    }

    /* VMListener */
    public void vmStartEvent(VMStartEvent e) {
        for (int i = 0; i < requests.size(); i++) {
            resolveAgainstAllClasses((EventRequestWrapper) requests.get(i));
        }
    }
    public void vmDeathEvent(VMDeathEvent e) {
    }
    public void vmDisconnectEvent(VMDisconnectEvent e) {
    }

}
