/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved. 
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;

/**
 * SetRequest.java
 *
 *
 * Created: Fri Aug 25 14:51:09 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public class SetRequest extends ExpressionGettingRequest {

    private Object lvalue;

    public SetRequest(Debugger debugger, Object lvalue, Object rvalue) {
        super(debugger, rvalue);
        this.lvalue = lvalue;
    }

    public Object go() throws NoVMException, DebuggerException {
        Value value = getValue();
        StackFrame frame = dbg().getDefaultFrame();
        LocalVariable local = eval().getLocalVariable(lvalue, frame);        
        try {
            frame.setValue(local, value);
        } catch (InvalidTypeException ite) {
            throw new DebuggerException(ite);
        } catch (ClassNotLoadedException cnle) {
            throw new DebuggerException(cnle);
        }
        return frame.getValue(local);
    }
}
