/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved. 
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import com.sun.jdi.request.*;
import com.sun.jdi.event.*;
import java.util.*;

/**
 * ThreadRequestAction.java
 *
 *
 * Created: Mon Aug 28 09:41:48 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public abstract class ThreadRequestAction extends RequestActionSupport implements ThreadListener {

    protected String threadName;
    
    public ThreadRequestAction(Debugger debugger, String threadName) {
        super(debugger);
        this.threadName = threadName;
    }
    
    void resolveAll() {
        resolveAgainstAllThreads();
    }

    protected void addListeners() {
        super.addListeners();
        debugger.addThreadListener(this);        
    }

    protected void removeListeners() {
        super.removeListeners();
        debugger.removeThreadListener(this);        
    }      

    abstract boolean isEnter();

    protected EventRequest resolve(ThreadReference threadRef) {
        try {
            if (isSame(threadRef)) {
                EventRequestManager em = vm().eventRequestManager();
                if (isEnter()) {
                    MethodEntryRequest entry = em.createMethodEntryRequest();
                    entry.addThreadFilter(threadRef);
                    entry.setSuspendPolicy(EventRequest.SUSPEND_NONE); //TODO: Change to ALL
                    entry.enable();
                    debugger.addExcludesTo(entry);
                    return entry;
                } else {
                    MethodExitRequest exit = em.createMethodExitRequest();                
                    exit.addThreadFilter(threadRef);
                    exit.setSuspendPolicy(EventRequest.SUSPEND_NONE); //TODO: Change to ALL
                    exit.enable();
                    debugger.addExcludesTo(exit);
                    return exit;
                }
            }
        } catch (NoVMException e) {
        }
        return null;
    }

    protected boolean isSame(ThreadReference threadRef) {
        if (threadRef.name().equals(threadName)) {
            return true;
        } else {
            try {
                long ourID = Long.parseLong(threadName);
                long refID = threadRef.uniqueID();
                if (ourID == refID) {
                    return true;
                }
            } catch (NumberFormatException e) {
            }
        }
        return false;
    }

    protected boolean resolveAgainstAllThreads() {
        try {
            Iterator iter = debugger.getVM().allThreads().iterator();
            while (iter.hasNext()) {
                if (setRequest(resolve((ThreadReference) iter.next())) != null) {
                    return success();
                }
            }
        } catch (NoVMException e) {
        }
        return failure(true);
    }
    
    protected boolean resolveAgainstThread(ThreadReference threadRef) {
        if (setRequest(resolve(threadRef)) != null) {
            return success();
        }
        return failure(false);
    }      
    
    /* ThreadListener */
    public void threadStartEvent(ThreadStartEvent e) {
        if (!isSet()) {
            resolve(e.thread());
        }
    }
    public void threadDeathEvent(ThreadDeathEvent e) {
    }    
}
