/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved. 
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import java.util.*;

/**
 * ThreadsNeedingRequest.java
 *
 *
 * Created: Thu Aug 24 16:42:46 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public abstract class ThreadsNeedingRequest extends Request {

    protected List threadNames;
    
    public ThreadsNeedingRequest(Debugger debugger, List threadNames) {
        super(debugger);
        this.threadNames = threadNames;
    }

    public List getThreads() throws NoVMException, DebuggerException {
        if (threadNames == null || threadNames.size() == 0) {
            return vm().allThreads();
        }
        Iterator namesIter = threadNames.iterator();        
        List threads = new ArrayList();
        List badThreads = new ArrayList();
        while (namesIter.hasNext()) {
            String threadName = namesIter.next() + "";                            
            Iterator iter = vm().allThreads().iterator();
            boolean found = false;
            while (iter.hasNext()) {
                ThreadReference threadRef = (ThreadReference) iter.next();
                if (threadRef.name().equals(threadName)) {
                    found = true;
                    threads.add(threadRef);
                    break;
                } else {
                    try {
                        long ourID = Long.parseLong(threadName);
                        long refID = threadRef.uniqueID();
                        if (ourID == refID) {
                            found = true;
                            threads.add(threadRef);
                            break;
                        }
                    } catch (NumberFormatException e) {
                    }
                }
            }
            if (!found) {
                badThreads.add(threadName);
            }
        }
        if (badThreads.size() > 0) {
            throw new ThreadsNotFoundException(badThreads);
        }
        return threads;        
    }
}
