/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import java.util.*;

/**
 * ThreadsRequest.java
 *
 *
 * Created: Thu Aug 24 13:18:28 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public class ThreadsRequest extends Request {

    private String threadGroupName;

    public ThreadsRequest(Debugger debugger, String threadGroupName) {
        super(debugger);
        this.threadGroupName = threadGroupName.trim();
    }

    public Object go() throws NoVMException, DebuggerException {
        List list = new ArrayList();
        if (threadGroupName == null || threadGroupName.length() == 0) {
            list = getAllThreads();
        } else {
            list = getThreads();
        }
        return list;
    }

    private List getAllThreads() throws NoVMException, DebuggerException  {
        return vm().allThreads();
    }

    private List getThreads() throws NoVMException, DebuggerException  {
        List result = getThreads(null);
        if (result == null) {
            throw new ThreadGroupNotFoundException();
        }
        return result;
    }

    private List getThreads(ThreadGroupReference ref) throws NoVMException, DebuggerException  {
        if (ref != null) {
            if (ref.name().equals(threadGroupName)) {
                return ref.threads();
            }

            try {
                long ourID = Long.parseLong(threadGroupName);
                long refID = ref.uniqueID();
                if (ourID == refID) {
                    return ref.threads();
                }
            } catch (NumberFormatException e) {
            }
        }

        List threadGroups = null;
        if (ref == null) {
            threadGroups = vm().topLevelThreadGroups();
        } else {
            threadGroups = ref.threadGroups();
        }
        Iterator iter = threadGroups.iterator();
        while (iter.hasNext()) {
            List threads = getThreads((ThreadGroupReference) iter.next());
            if (threads != null) {
                return threads;
            }
        }
        return null;
    }

    class ThreadGroupNotFoundException extends DebuggerException {
        public ThreadGroupNotFoundException() {
            super(ThreadsRequest.this.threadGroupName +
                  " is not a valid thread group.");
        }
    }
}
