/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved. 
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import com.sun.jdi.request.*;

/**
 * UntraceMethodsRequest.java
 *
 *
 * Created: Mon Aug 28 11:09:17 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public class UntraceMethodsRequest extends Request {

    private String threadName;
    
    public UntraceMethodsRequest(Debugger debugger, String threadName) {
        super(debugger);
        this.threadName = threadName;
    }

    public Object go() throws NoVMException, DebuggerException {
        MethodEntryRequest entry = (MethodEntryRequest) new MethodEntryRequestAction(dbg(), threadName).go();
        MethodExitRequest  exit  = (MethodExitRequest)  new MethodExitRequestAction(dbg(), threadName).go();
        return new EntryExitPair(entry, exit);
    }

    public static class EntryExitPair {
        private MethodEntryRequest entry;
        private MethodExitRequest exit;
        public EntryExitPair(MethodEntryRequest entry, MethodExitRequest exit) {
            this.entry = entry;
            this.exit  = exit;
        }
        public MethodEntryRequest getEntry() {
            return entry;
        }
        public MethodExitRequest getExit() {
            return exit;
        }
        public String toString() {
            return "UNTRACE:\n\tEntry: " + entry + "\n\t" + "Exit: " + exit;
        }        
    }

    static class MethodEntryRequestAction extends ThreadRequestAction {
        public MethodEntryRequestAction(Debugger debugger, String threadName) {
            super(debugger, threadName);
        }
        public Object go() throws NoVMException, DebuggerException {
            return unset();
        }    
        boolean isEnter() {
            return true;
        }
    }
    
    static class MethodExitRequestAction extends ThreadRequestAction {
        public MethodExitRequestAction(Debugger debugger, String threadName) {
            super(debugger, threadName);
        }
        public Object go() throws NoVMException, DebuggerException {
            return unset();
        }
        boolean isEnter() {
            return false;
        }
    }    
}
