/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved. 
 */

package org.aspectj.debugger.request;

import org.aspectj.debugger.base.*;

import com.sun.jdi.*;
import com.sun.jdi.event.*;
import com.sun.jdi.request.*;
import java.util.*;

/**
 * WatchRequest.java
 *
 *
 * Created: Fri Aug 25 16:50:02 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public abstract class WatchRequest extends ClassAndFieldNeedingRequestAction {

    public final static int ACCESS = 0x01;
    public final static int ALL    = 0x10;
    
    public WatchRequest(Debugger debugger, String className, String fieldName) {
        super(debugger, className, fieldName);
    }

    EventRequest resolve(ReferenceType refType) throws MultipleLocationsException {
        EventRequest request = null;
        try {
            if (refType.name().equals(className)) {
                if (getType() == ALL) {
                    request = vm().eventRequestManager().createModificationWatchpointRequest(getField());
                } else {
                    request = vm().eventRequestManager().createAccessWatchpointRequest(getField());
                }
                //request.addCountFilter(1);
                request.enable();
                request.setSuspendPolicy(EventRequest.SUSPEND_ALL); 
            }
        } catch (NoVMException e) {
        } catch (DebuggerException de) {
            //TODO deal with this!
        }
        return request;        
    }

    abstract int getType();
}
