/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.debugger.tty;

import com.sun.jdi.*;
import com.sun.jdi.event.*;
import com.sun.jdi.request.*;
import com.sun.jdi.connect.*;
import org.aspectj.debugger.base.*;
import org.aspectj.debugger.request.*;
import org.aspectj.tools.ide.SourceLine;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.List;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * CommandLineDebugger.java
 *
 *
 * Created: Wed Aug 23 14:53:57 2000
 *
 * @author <a href="mailto:palm@parc.xerox.com"Jeffrey Palm</a>
 */

public class CommandLineDebugger
    implements
        DebuggerApp,
        StopListener,
        VMListener,
        DebuggerListener,
        PromptListener,
        Runnable {

    protected      AJDebugger      debugger; //         = null;
    private        boolean         isRunning        = true;
    private        BufferedReader  in               = new BufferedReader(new InputStreamReader(System.in));
    private static String          title            = "AspectJ Example Debugger";
    private static String          version          = "v0.1";
    private static String          copyright        = "(C) 2000 Xerox Corporation";
    private        String          lastCommand      = "";
    private        boolean         echo             = false;
    private static String[]        echoStrings      = {

    };
    private        boolean         showPrompt;
    private        String[]        cmdLineArgs;
    private        Thread          thread;
    private        Options         options          = new Options();
    protected      boolean         wantsToExit      = true;

    private boolean debug = false; //true;
    private void db(Object o) {
        if (debug) {
            System.err.println("<CL DEBUGGER>: " + o);
        }
    }

    public CommandLineDebugger(String[] args) {
        this(args, true);
    }

    public CommandLineDebugger(AJDebugger debugger, String[] args) {
        this(debugger, args, true);
    }

    public CommandLineDebugger(String[] args, String scriptFile) {
        this(args, false);
        List commands = new Vector();
        try {
            commands = debugger.readCommand(scriptFile, true);
        } catch (NoVMException nvme) {
        } catch (DebuggerException de) {
        }
        for (int i = 0; i < commands.size(); i++) {
            String command = commands.get(i) + "";
            executeCommand(command);
        }
        while (debugger.isRunning()) {}
        exit();
    }

    public CommandLineDebugger(String args[], boolean showPrompt) {
        this.debugger = new AJDebugger(this, showPrompt);
        debugger.addStopListener(this);
        debugger.addVMListener(this);
        debugger.addDebuggerListener(this);
        debugger.addPromptListener(this);
        this.showPrompt = showPrompt;
        this.cmdLineArgs = args;
        options.fill(args);
    }

    public CommandLineDebugger(AJDebugger db,
                               String args[],
                               boolean showPrompt) {
        this.debugger = db;
        debugger.addStopListener(this);
        debugger.addVMListener(this);
        debugger.addDebuggerListener(this);
        debugger.addPromptListener(this);
        this.showPrompt = showPrompt;
        this.cmdLineArgs = args;
        options.fill(args);
    }

    public void go() {
        debugger.setOptions(options);
        String mainClass = options.getClassName();
        boolean runNow = false;
        if (!options.isSet("now") &&
            mainClass != null &&
            !mainClass.equals("")) {
            debugger.loadMainClass(options);
        }
        start();
        if (options.isSet("now")) {
            debugger.runCommand();
        }
    }

    public void start() {
        if (thread == null) {
            thread = new Thread(this);
            thread.start();
        }
    }

    public void run() {
        if (showPrompt /*&& !isGUI()*/) {
            while (true) {
                prompt();
                executeCommand(line());
            }
        }
    }

    public boolean isGUI() {
        return false;
    }

    public AJDebugger getDebugger() {
        return debugger;
    }

    /****************************** Misc. Stuff ******************************/

    private boolean runNow(String[] args) {
        return !options.getClassName().equals("");
    }

    String className(String[] args) {
        if (args != null && args.length > 0) {
            return args[0];
        }
        return "";
    }

    String exArgs(String[] args) {
        String exArgs = "";
        if (args != null && args.length > 1) {
            for (int i = 1; i < args.length; i++) {
                exArgs += args[i] + " ";
            }
        }
        return exArgs.trim();
    }



    /****************************** DebuggerApp ******************************/

    // start-commands

    public void runCommand(String className,
                           String exArgs)
        throws NoVMException, DebuggerException {
    }

    public void runCommand(String className,
                           String vmArgs,
                           String exArgs,
                           boolean isSuspended, int debugTraceMode)
        throws NoVMException, DebuggerException {
    }

    public void clearAtCommand(String className,
                               int line,
                               EventRequest request)
        throws NoVMException, DebuggerException {
    }

    public void clearInCommand(String className,
                               String methodProto,
                               EventRequest request)
        throws NoVMException, DebuggerException {
    }

    public void clearOnCommand(String sourceName,
                               int line,
                               EventRequest request)
        throws NoVMException, DebuggerException {
    }

    public void clearCommand(List breakpoints)
        throws NoVMException, DebuggerException {
        result("Breakpoints:\n" + debugger.iter(breakpoints));
    }

    public void clearAllCommand(List breakpoints)
        throws NoVMException, DebuggerException {
        result("Breakpoint requests cleared:\n" + debugger.iter(breakpoints));
    }

    public void stopAtCommand(String className,
                              int line,
                              EventRequest request)
        throws NoVMException, DebuggerException {
    }

    public void stopInCommand(String className,
                              String methodProto,
                              EventRequest request)
        throws NoVMException, DebuggerException {
    }

    public void stopOnCommand(String sourceName,
                              int line,
                              EventRequest request)
        throws NoVMException, DebuggerException {
    }

    public void stopCommand(List breakpoints)
        throws NoVMException, DebuggerException {
        clearCommand(breakpoints);
    }

    public void contCommand()
        throws NoVMException, DebuggerException {
    }

    public void threadsCommand(String threadGroupName, List threads)
        throws NoVMException, DebuggerException {
        result(debugger.threads(threads));
    }

    public void threadGroupsCommand(List threadGroups)
        throws NoVMException, DebuggerException {
        result(debugger.threadGroups(threadGroups));
    }

    public void fieldsCommand(String className, List fields)
        throws NoVMException, DebuggerException {
        result(debugger.fields(fields));
    }

    public void methodsCommand(String className, List methods)
        throws NoVMException, DebuggerException {
        result(debugger.methods(methods));
    }

    public void classesCommand(List classes)
        throws NoVMException, DebuggerException {
        result(debugger.classes(classes));
    }

    public void classCommand(String className,
                             ReferenceType refType)
        throws NoVMException, DebuggerException {
        result(debugger.clazz(refType));
    }

    public void threadCommand(String threadName,
                              ThreadReference threadRef)
        throws NoVMException, DebuggerException {
    }

    public void threadGroupCommand(String threadGroupName,
                                   ThreadGroupReference threadGroupRef)
        throws NoVMException, DebuggerException {
    }

    public void suspendCommand(List threadNames, List threads)
        throws NoVMException, DebuggerException {
    }

    public void resumeCommand(List threadNames, List threads)
        throws NoVMException, DebuggerException {
    }

    public void whereCommand(String threadName, List frames)
        throws NoVMException, DebuggerException {
        result(debugger.frames(frames));
    }

    public void localsCommand(List locals)
        throws NoVMException, DebuggerException {
        result(debugger.locals(locals));
    }

    public void interruptCommand(String threadName,
                                 ThreadReference threadRef)
        throws NoVMException, DebuggerException {
    }

    public void upCommand(int frames, StackFrame currentFrame)
        throws NoVMException, DebuggerException {
    }

    public void downCommand(int frames, StackFrame currentFrame)
        throws NoVMException, DebuggerException {
    }

    public void stepCommand(StepRequest request)
        throws NoVMException, DebuggerException {
    }

    public void stepUpCommand(StepRequest request)
        throws NoVMException, DebuggerException {
    }

    public void stepiCommand(StepRequest request)
        throws NoVMException, DebuggerException {
    }

    public void nextCommand(StepRequest request)
        throws NoVMException, DebuggerException {
    }

    public void printCommand(Object valueRep, Value value)
        throws NoVMException, DebuggerException {
        result(" " + valueRep + " = " + getValue(value));
    }

    public void dumpCommand(Object valueRep, Value value)
        throws NoVMException, DebuggerException {
        result(" " + valueRep + " = " + getValue(value));
    }

    public void evalCommand(Object valueRep, Value value)
        throws NoVMException, DebuggerException {
        result(" " + valueRep + " = " + getValue(value));
    }

    private String getValue(Value value) {
        String valString = value + "";
        if (!debugger.getOptions().isSet("extra")) {
            return valString;
        }
        valString += " = " + value(value);
        return valString;
    }

    /**
     * render value by unwrapping as ObjectReference
     * @return "null" if null, empty String if unsuccessful, rendering otherwise
     */
    private String value(Value value) { 
        if (value == null) {
            return "null";
        }
        if (!(value instanceof ObjectReference)) {
            return value+""; // todo: omit if only client is this.getValue(Value)
        }
        // get object reference to invoke toString or primitive value method
        String valString = "";
        try {
            ObjectReference oRef = (ObjectReference) value;
            if (oRef instanceof ArrayReference) {
                String result = "{";
                ArrayReference arrayRef = (ArrayReference) oRef;
                int length = arrayRef.length();
                for (int i = 0; i < length; i++) {
                    try {
                        result += value(arrayRef.getValue(i));
                    } catch (NullPointerException npe) {
                        result += "null";
                    }
                    result += i < length-1 ? ", " : "";
                }
                result += "}";
                return result;
            }
            ReferenceType refType = oRef.referenceType();
            String name = refType.name();
            ThreadReference thread = null;
            try {
                if (debugger.getVM().canGetOwnedMonitorInfo()) {
                    try {
                        thread = oRef.owningThread();
                    } catch (UnsupportedOperationException uoe) {
                    }
                }
                if (thread == null) {
                    thread = debugger.getDefaultThread();
                }
            } catch (NoVMException nvme) {
                return valString;
            }
            int options = ObjectReference.INVOKE_NONVIRTUAL;
            Method method = null;
            // unwrap if a primitive value
            if (name.equals("java.lang.Byte")) {
                method = (Method)refType.methodsByName("byteValue").get(0);
            } else if (name.equals("java.lang.Boolean")) {
                method = (Method)refType.methodsByName("booleanValue").get(0);
            } else if (name.equals("java.lang.Character")) {
                method = (Method)refType.methodsByName("charValue").get(0);
            } else if (name.equals("java.lang.Double")) {
                method = (Method)refType.methodsByName("doubleValue").get(0);
            } else if (name.equals("java.lang.Float")) {
                method = (Method)refType.methodsByName("floatValue").get(0);
            } else if (name.equals("java.lang.Integer")) {
                method = (Method)refType.methodsByName("intValue").get(0);
            } else if (name.equals("java.lang.Long")) {
                method = (Method)refType.methodsByName("longValue").get(0);
            } else if (name.equals("java.lang.Short")) {
                method = (Method)refType.methodsByName("shortValue").get(0);
            } else { 
                List list = refType.methodsByName("toString");
                if (null != list) {
                    for (Iterator it = list.iterator(); it.hasNext();) {
                        Method m = (Method) it.next();
                        List types = m.argumentTypes();
                        if ((null == types) || (0 == types.size())) {
                            method = m;
                            System.err.println("method: " + method);
                            break;
                        }
                    }
                }
            }
            if ((null != thread) && (null != method)) {
                Value newValue = oRef.invokeMethod(thread, method,
                                               new Vector(), options);
                valString += newValue+"";
            }
        } catch (InvalidTypeException ite) { // todo: log all these?
        } catch (ClassNotLoadedException cnle) {
        } catch (IncompatibleThreadStateException itse) {
        } catch (InvocationException ie) {
        }
        return valString;
    }

    public void setWantsToExit(boolean wantsToExit) {
        this.wantsToExit = wantsToExit;
    }

    public void setCommand(Object lvalue,
                           Object rvalue,
                           Value oldValue,
                           Value newValue)
        throws NoVMException, DebuggerException {
        result("Changed '" + lvalue + "' from '" +
               oldValue + "' to '" + newValue + "'");
    }

    public void classpathCommand(String baseDirectory, List paths)
        throws NoVMException, DebuggerException {
        String result =
            "base directory: " + baseDirectory + "\n" +
            "classpath: " + paths;
        result(result);
    }

    public void lockCommand(Object valueRep, LockInformation lockInfo)
        throws NoVMException, DebuggerException {
        result("Lock information for '" + valueRep + "':\n" + lockInfo);
    }

    public void threadlocksCommand(String threadName,
                                   ThreadLockInformation threadLockInfo)
        throws NoVMException, DebuggerException {
        result("Thread lock information for '" +
               threadName + "':\n" + threadLockInfo);
    }

    public void watchAccessCommand(String className,
                                   String fieldName,
                                   WatchpointRequest request)
        throws NoVMException, DebuggerException {
    }

    public void  watchAllCommand(String className,
                                 String fieldName,
                                 WatchpointRequest request)
        throws NoVMException, DebuggerException {
    }

    public void unwatchAccessCommand(String className,
                                     String fieldName,
                                     WatchpointRequest request)
        throws NoVMException, DebuggerException {
    }

    public void unwatchAllCommand(String className,
                                  String fieldName,
                                  WatchpointRequest request)
        throws NoVMException, DebuggerException {
    }

    public void catchCommand(String className, ExceptionRequest request)
        throws NoVMException, DebuggerException {
    }

    public void ignoreCommand(String className, ExceptionRequest request)
        throws NoVMException, DebuggerException {
    }

    public void traceMethodsCommand(String threadName,
                                    TraceMethodsRequest.EntryExitPair pair)
        throws NoVMException, DebuggerException {
    }

    public void untraceMethodsCommand(String threadName,
                                      UntraceMethodsRequest.EntryExitPair pair)
        throws NoVMException, DebuggerException {
    }

    public void excludeCommand(List classNames, List classes)
        throws NoVMException, DebuggerException {
        result("Exclude:\n" + debugger.iter(classes));
    }

    public void quitCommand()
        throws NoVMException, DebuggerException {
    }

    public void helpCommand(String helpString)
        throws NoVMException, DebuggerException {
        result(helpString);
    }

    public void versionCommand(Object version)
        throws NoVMException, DebuggerException {
        result(version);
    }

    public void killCommand(String threadName,
                            String valueRep,
                            ThreadReference threadRef)
        throws NoVMException, DebuggerException {
        result("Killed thread '" + threadName + "' = " +
               threadRef + " with '" + valueRep + "'");
    }

    public void useCommand(String sourcePath, String newSourcePath)
        throws NoVMException, DebuggerException {
        result("Using sourcepath: " + newSourcePath);
    }

    public void workingdirCommand(String workingdirPath, String newWorkingdirPath)
        throws NoVMException, DebuggerException {
        result("Using workingdir: " + newWorkingdirPath);
    }    

    public void listCommand(List sourceLines)
        throws NoVMException, DebuggerException {
        Object o = debugger.sourceLines(sourceLines, true);
        if (sourceLines.size() == 0) {
            o = "Source file not found: unkown.source";
        }
        result(o);
    }

    public void listCommand(String sourceName, List sourceLines)
        throws NoVMException, DebuggerException {
        Object o = debugger.sourceLines(sourceLines, true);
        if (sourceLines.size() == 0) {
            o =
                "Source file not found: " +
                (sourceName != null ? sourceName : "<unkown>");
        }
        result(o);
    }

    public void listCommand(String sourceName,
                            int lineNumber,
                            SourceManager.SourceLine sl)
        throws NoVMException, DebuggerException {
        Object o = debugger.sourceLine(sl);
        if (sl.getLineNumber() < 0) {
            o =
                "<no source available for " +
                sourceName + ":" + lineNumber + ">";
        }
        result(o);
    }

    public void listCommand(String sourceName,
                            int startLine,
                            int endLine,
                            List sourceLines)
        throws NoVMException, DebuggerException {
        listCommand(sourceLines);
    }

    public void monitorCommand(String command, String result)
        throws NoVMException, DebuggerException {
    }

    public void monitorCommand(List monitors)
        throws NoVMException, DebuggerException {
        result(monitors);
    }

    public void unmonitorCommand(int number, MonitorRequest request)
        throws NoVMException, DebuggerException {
        result("Unmonitoring " + request);
    }

    public void readCommand(String fileName, List commands)
        throws NoVMException, DebuggerException {
    }

    public void viewCommand(String sourceName, List sourceLines)
        throws NoVMException, DebuggerException {
        listCommand(sourceName, sourceLines);
    }

    public void tostringCommand(String tostring)
        throws NoVMException, DebuggerException {
        result(tostring);
    }

    public void connectCommand(VirtualMachine vm)
        throws NoVMException, DebuggerException {
        throw new DebuggerException("connect is not valid on the command line");
    }

    public void pwdCommand(File file)
        throws NoVMException, DebuggerException {
        if (file != null) result(file.getAbsolutePath());
    }

    
    public void lsCommand(String dir, List files)
        throws NoVMException, DebuggerException {
        result(dir + ":\n" + debugger.iter(files));
    }

    // end-commands


    /************************** Handling Exceptions **************************/

    public void handleParseException(ParseException e) {
        result(e.getMessage());
    }

    public void handleNoVMException(NoVMException e) {
        result("There is no VM: " + nonull(e.getMessage()));
    }

    public void handleDebuggerException(DebuggerException e) {
        result(e.getMessage());
    }

    public void handleVMDisconnectedException(VMDisconnectedException e) {
        result("The VM has already disconnected.");
    }

    public void handleInternalException(Throwable e) {
        result("Internal error: " + e);
        e.printStackTrace(getOutputStream());
        new ErrorLogger(debugger).log(e);
    }



    public PrintStream getOutputStream() {
        return System.out;
    }

    /*************************** End of DebuggerApp **************************/

    public Object executeCommand(String str) {
        if (echo(str)) {
            result(str);
        }
        return debugger.execute(str.trim());
    }

    private boolean echo(String str) {
        if (str != null) {
            str = str.trim();
            for (int i = 0; i < echoStrings.length; i++) {
                if (str.startsWith(echoStrings[i])) {
                return true;
                }
            }
        }
        return false;
    }

    private void repeat() {
        executeCommand(lastCommand);
    }

    protected boolean nonull(String str) {
        return str != null && !"null".equals(str);
    }

    protected void result(Object str) {
        if (str != null) outln(str);
    }

    protected void result(List list) {
        Iterator iter = list.iterator();
        while (iter.hasNext()) {
            result(iter.next());
        }
    }

    private void warn(String str) {
        outln("  *** " + str);
    }

    public String line() {
        String str = "";
        try { db("get line?");
            str = in.readLine();
            db("got line str=" + str);
        } catch (IOException e) {
        }
        return str;
    }

    public void exit() {
        isRunning = false;
        debugger.shutDown();
        if (wantsToExit()) {
            System.exit(0);
        }
    }

    private void prompt() {
        if (showPrompt) out(debugger.getPrompt() + " ");
    }

    private void out(Object o) {
        System.out.print(o);
        System.out.flush();
    }

    public void outln(Object o) {
        out(o);
        out("\n");
    }

    public void log(Object o) {
    }

    private VirtualMachine vm() {
        return debugger.vm();
    }


    /****************************** Listeners ******************************/

    /* PromptListener */
    public void resetPrompt() {
        prompt();
    }

    /* StopListener */
    public void accessWatchpointEvent(AccessWatchpointEvent e){
    }
    public void breakpointEvent(BreakpointEvent e){
        result("\n\nBreakpoint hit: " + debugger.format(e));
    }
    public void exceptionEvent(ExceptionEvent e){
        result("\n\nException occured: " + debugger.format(e));
    }
    public void modificationWatchpointEvent(ModificationWatchpointEvent e){
    }
    public void stepEvent(StepEvent e){
        result("\n\nStep completed: " + debugger.format(e));
    }

    /* VMListener */
    private int deaths = 0;
    public void vmDeathEvent(VMDeathEvent e) {
        die();
    }
    public void vmDisconnectEvent(VMDisconnectEvent e) {
        die();
    }
    public void vmStartEvent(VMStartEvent e) {
        deaths = 0;
        result("VM Started: ");
    }
    private void die() {
        if (deaths++ == 0) result("\nThe application has exited.");
        exit();
    }

    /* DebuggerListener */
    public void requestSetEvent(RequestEvent re) {
        RequestAction action = (RequestAction) re.getRequest();
        result("Set " +
               (action.isDeferred() ? " deferred " : "") +
               re.getRequest());
    }
    public void requestClearEvent(RequestEvent re) {
        result("Clear " + re.getRequest());
    }
    public void requestDeferredEvent(RequestEvent re) {
        if (!debugger.isRunning()) {
            result("Deferring " + re.getRequest() + "\n" +
                   "It will be set after the class is loaded.");
        }
    }
    public void requestFailedEvent(RequestEvent re) {
        result("Unable to set " + re.getRequest() +
               " : " + re.getErrorMessage());
    }

    public boolean wantsToExit() {
        return wantsToExit;
    }

    public boolean canRestart() {
        return false;
    }

    public VirtualMachine connect(String vmArgs,
                                  String className,
                                  String commandLine) {
        return null;
    }
}


