/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.lang;

/** 
 * Wrapper for checked exceptions matched by a 'declare soft'.
 * You can soften checked exceptions at join points by using
 * the form <code>declare soft: TypePattern: Pointcut</code>.
 * At the join points, any exceptions thrown which match
 * TypePattern will be wrapped in <code>SoftException</code>
 * and rethrown. You can get the original exception using
 * <code>getWrappedThrowable()</code>.
 */
public class SoftException extends RuntimeException {
    Throwable inner;
    public SoftException(Throwable inner) {
        super();
        this.inner = inner;
    }
    
    public Throwable getWrappedThrowable() { return inner; }
    
    //XXX should add a getCause() method to parallel j2se 1.4's new
    //XXX chained exception mechanism
}
