
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */


package org.aspectj.tools.ajbrowser;

import java.util.*;
import java.io.*;
import org.aspectj.ajde.*;
import org.aspectj.ajde.ui.*;

public class BrowserProperties implements ProjectPropertiesAdapter {
    
    UserPreferencesAdapter preferencesAdapter = null;
    
    public BrowserProperties(UserPreferencesAdapter preferencesAdapter) {
    	this.preferencesAdapter = preferencesAdapter;
    }

    public String getLastOpenSourceFilePath() {
        return preferencesAdapter.getProjectPreference("editor.lastOpenFile");
    }

    public void setLastOpenSourceFilePath(String value) {
        preferencesAdapter.setProjectPreference("editor.lastOpenFile",value);
    }

    public String getLastOpenSourceLineNumber() {
        return preferencesAdapter.getProjectPreference("editor.lastOpenLineNumber");
    }

    public void setLastOpenSourceLineNumber(String value) {
        preferencesAdapter.setProjectPreference("editor.lastOpenLineNumber",value);
    }

	public List getBuildConfigFiles() {
		return BrowserManager.getDefault().getConfigFiles();
	}
	
	public String getDefaultBuildConfigFile() {
		return null;	
	}

	public String getLastActiveBuildConfigFile() {
		return null;	
	}

    public String getProjectName() {
        return null;
    }

    public String getClassToExecute() {
        return preferencesAdapter.getProjectPreference("runtime.mainClass");
    }

    public void setClassToExecute(String mainClass) {
        preferencesAdapter.setProjectPreference("runtime.mainClass", mainClass);
    }

    public String getRootProjectDir() {
        return new File(Ajde.getDefault().getConfigurationManager().getActiveConfigFile()).getParent();
    }

	public String getExecutionArgs() {
		return null;	
	}

	public List getProjectSourceFiles() {
		return null;	
	}
    
    public String getVmArgs() {
    	return null;	
    }

    public String getProjectSourcePath() {
        return null;
    }

    public String getBootClasspath() {
        return null;
    }

    public void setAjcOptions(String flags) {
        preferencesAdapter.setProjectPreference("build.flags", flags);
    }

    public String getAjcOptions() {
        return preferencesAdapter.getProjectPreference("build.flags");
    }

    public String getOutputPath() {
        String outputPath = preferencesAdapter.getProjectPreference("build.outputpath");
        if (outputPath == null) {
            return ".";
        } else {
            return outputPath;
        }
    }

    public void setOutputPath(String path) {
        preferencesAdapter.setProjectPreference("build.outputpath", path);
    }

    public String getUserClasspath() {
        return preferencesAdapter.getProjectPreference("build.classpath");
    }

    public String getClasspath() {
        String systemPath = System.getProperty("java.class.path", ".");
        String userPath = preferencesAdapter.getProjectPreference("build.classpath");
        return userPath + File.pathSeparator + systemPath;
    }

    public void setClasspath(String path) {
        preferencesAdapter.setProjectPreference("build.classpath", path);
    }

    public String getAjcWorkingDir() {
        String workingDir = preferencesAdapter.getProjectPreference("build.workingdir");
        if (workingDir == null || workingDir.equals(getOutputPath())) {
            return getOutputPath() + "./ajworkingdir";
        } else {
            return workingDir;
        }
    }

    public void setAjcWorkingDir(String path) {
        preferencesAdapter.setProjectPreference("build.workingdir", path);
    }
}
