
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.tools.ajbrowser;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.text.StyledDocument;
import org.aspectj.asm.*;
import org.aspectj.compiler.base.ast.ThisExpr;
import org.aspectj.ajde.*;
import org.aspectj.ajde.TaskListManager;
import org.aspectj.ajde.Ajde;
import org.aspectj.ajde.ui.swing.*;

/**
 * Used to display a list of compiler messages that can be clicked in order
 * to seek to their corresponding sourceline.
 *
 * @author Mik Kersten
 */
public class CompilerMessagesPanel extends JPanel implements TaskListManager {
    private JScrollPane jScrollPane1 = new JScrollPane();
    //private JScrollPane jScrollPane2 = new JScrollPane();
    private JList list = new JList();
    private DefaultListModel listModel = new DefaultListModel();
    private BorderLayout borderLayout1 = new BorderLayout();

    public CompilerMessagesPanel() {
        try {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
        list.setModel(listModel);

        MouseListener mouseListener = new MouseAdapter() {
             public void mouseClicked(MouseEvent e) {
                 if (e.getClickCount() >= 1) {
                     int index = list.locationToIndex(e.getPoint());
                     if (listModel.getSize() >= index && index != -1) {
                     	CompilerMessage cm = (CompilerMessage)listModel.getElementAt(index);
                     	Ajde.getDefault().getEditorManager().showSourceLine(cm.sourceLocation, true);
                     }
                  }
             }
        };
        list.addMouseListener(mouseListener);
        list.setCellRenderer(new CompilerMessagesCellRenderer());
    } 

    public void addSourcelineTask(String message, SourceLocation sourceLocation, StructureMessage.Kind kind) {   
        listModel.addElement(new CompilerMessage(message, sourceLocation,kind));
        BrowserManager.getDefault().showMessages();
    }

    public void addProjectTask(String message, StructureMessage.Kind kind) {
		listModel.addElement(new CompilerMessage(message,kind));
		BrowserManager.getDefault().showMessages();
	}

    public void clearTasks() {
        listModel.clear();
    }

    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);
        this.add(jScrollPane1, BorderLayout.CENTER);
        jScrollPane1.getViewport().add(list, null);
    }
}



