/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.tools.ajdb;

import org.aspectj.debugger.gui.ComponentDirector;
import org.aspectj.debugger.tty.CommandLineDebugger;
import org.aspectj.debugger.ide.FullPathSourceShower;
// import org.aspectj.debugger.ide.ForteDebuggerFrame;
// import org.aspectj.debugger.ide.JBuilder4DebuggerFrame;
import org.aspectj.debugger.ide.SourceShower;
import org.aspectj.debugger.ide.IDEInterface;
import org.aspectj.debugger.ide.IDEInterfaceImpl;

import org.aspectj.util.gui.CenteredJFrame;

import java.awt.Container;
import java.awt.Component;
import java.awt.event.ActionEvent;
import javax.swing.*;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

public class Main {
    public static void main(String[] args) {
        new Main().debug(args);
    }

    public Main() {}

    /**
     * All we're doing is launching a debugger, if the debugger is
     * a command line version, the return value will be non-null,
     * otherwise, it'll be null.
     *
     * This is basically a hack.  We started with a command line
     * debugger, then a GUI one, then IDE debuggers, so that
     * is from where this comes.
     *
     */
    public CommandLineDebugger debug(String[] args) {

        // The return value
        CommandLineDebugger debugger = null;

        // These are set to denote the kind
        // of debugger we're using
        boolean launch = false;
        boolean gui = false;
        boolean forte = false;
        boolean jbuilder = false;

        // In the commandline version and the GUI version
        // we want to exit with the window closing, etc...
        // we don't want this in the IDE versions
        boolean wantsToExit = true;

        // The link between the debugger and a source pane
        SourceShower shower = FullPathSourceShower.proto;

        // The link between the debugger and an IDE
        IDEInterface ide = IDEInterfaceImpl.proto;

        // Check out the arguments, pull out any that
        // are not valid for the command line debugger
        for (int i = 0; i < args.length; i++) {
            if (args[i].equals("-noexit")) {
                wantsToExit = false;
                System.setProperty("no.logger", "true");
                args = shift(args, i--);
            } else if (args[i].equals("-launch")) {
                launch = true;
                args = shift(args, i--);
            } else if (args[i].equals("-gui")) {
                gui = true;
                args = shift(args, i--);
            } else if (args[i].equals("-forte")) {
                forte = true;
                args = shift(args, i--);
            } else if (args[i].equals("-jbuilder")) {
                jbuilder = true;
                args = shift(args, i--);
            } else if (args[i].equals("-ide")) {
                args = shift(args, i);
                try {
                    ide = (IDEInterface)Class.forName(args[i]).newInstance();
                } catch (Throwable t) { t.printStackTrace(); }
                args = shift(args, i);
            } else if (args[i].equals("-shower")) {
                args = shift(args, i);
                try {
                    shower = (SourceShower)Class.forName(args[i]).newInstance();
                } catch (Throwable t) { t.printStackTrace(); }
                args = shift(args, i);
            }
        }

//          // Launch the forte or jbuilder version
//          if (launch) {
//              if (forte)    new ForteLauncher(args, shower, ide).go();
//              if (jbuilder) new JBuilderLauncher(args, shower, ide).go();
//              return null;
//          }

//          // Launch forte
//          if (forte) {
//              ForteDebuggerFrame director =
//                  new ForteDebuggerFrame(args, shower, ide);
//              if (ide != null) {
//                  ide.setComponentDirector(director);
//              }
//              director.go();
//              return null;
//          }

//          // Launch jbuilder
//          if (jbuilder) {
//              JBuilder4DebuggerFrame director =
//                  new JBuilder4DebuggerFrame(args, shower, ide);
//              if (ide != null) {
//                  ide.setComponentDirector(director);
//              }
//              director.go();
//              return null;
//          }
        // GUI
        if (gui) {
            new ComponentDirector(args).go();
        }
        debugger = new CommandLineDebugger(args);
        if (!gui) {
            debugger.setWantsToExit(wantsToExit);
        }

        // Start up the debugger...
        debugger.go();
        return debugger;
    }

    /**
     * Take out unwanted parameters
     */
    private String[] shift(String[] args, int i) {
        String[] newArgs = new String[args.length - 1];
        System.arraycopy(args, 0, newArgs, 0, i);
        System.arraycopy(args, i+1, newArgs, i, args.length-i-1);
        return newArgs;
    }

//      private class JBuilderLauncher extends AbstractLauncher {
//          public JBuilderLauncher(String[] args,
//                                  SourceShower shower,
//                                  IDEInterface ide) {
//              super(args, shower, ide);
//          }
//          protected ComponentDirector debugger(String[] args) {
//              return new JBuilder4DebuggerFrame(args , shower(), ide);
//          }
//          protected String getIDEName() {
//              return "JBuilder";
//          }
//      }

//      private class ForteLauncher extends AbstractLauncher {
//          public ForteLauncher(String[] args,
//                               SourceShower shower,
//                               IDEInterface ide) {
//              super(args, shower, ide);
//          }
//          protected ComponentDirector debugger(String[] args) {
//              return new ForteDebuggerFrame(args, shower(), ide);
//          }
//          protected String getIDEName() {
//              return "Forte";
//          }
//      }

    private abstract class AbstractLauncher extends CenteredJFrame {
        private final static int TEXT_WIDTH  = 30;
        private   JTextField mainClassField  = new JTextField(TEXT_WIDTH);
        private   JTextField classpathField  = new JTextField(TEXT_WIDTH);
        private   JTextField sourcepathField = new JTextField(TEXT_WIDTH);
        protected SourceShower shower;
        protected IDEInterface ide;
        protected SourceShower shower() { return shower; }
        {
            mainClassField.setText("New");
            sourcepathField.setText("C:/ajdb-src");
        }
        protected String[]args = null;
        private JButton launchButton =
            new JButton(new AbstractAction("Launch debugger") {
                public void actionPerformed(ActionEvent e) {
                    launch();
                    setVisible(false);
                    dispose();
                }
            });
        private void launch() {
            Collection list = new Vector();
            for (int i = 0; i < args.length; i++) {
                list.add(args[i]);
            }
            add(list, "-classpath", classpathField);
            add(list, "-sourcepath", sourcepathField);
            add(list, "-now", mainClassField);
            System.out.println("args: " + list);
            String[] newArgs = new String[list.size()];
            Iterator iter = list.iterator();
            int i = 0;
            while (iter.hasNext()) {
                newArgs[i++] = iter.next() + "";
            }
            ComponentDirector cd = debugger(newArgs);
            cd.go();
        }
        private void add(Collection list, String arg, JTextField field) {
            String text = field.getText().trim();
            if (text == null || text.length() == 0) return;
            if (arg.length() > 0) {
                list.add(arg);
                list.add(text);
            } else {
                list.add(text);
            }
        }
        protected abstract ComponentDirector debugger(String[] args);
        protected abstract String getIDEName();
        AbstractLauncher(String[] args, SourceShower shower, IDEInterface ide) {
            super();
            setTitle(getIDEName() + " Debugger Launcher");
            this.args = args;
            this.shower = shower;
            this.ide = ide;
            Container c = getContentPane();
            c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));
            c.add(p("Main class",  mainClassField));
            c.add(p("Class path",  classpathField));
            c.add(p("Source path", sourcepathField));
            c.add(launchButton);
            launchButton.requestFocus();
            setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        }
        void go() {
            pack();
            setVisible(true);
        }
        private Component p(String title, Component c2) {
            JPanel p = new JPanel();
            p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
            p.add(new JLabel(title + ": "));
            p.add(c2);
            return p;
        }
        
    }
}
