import sys
from string import capitalize, join, split, find, rfind

keywords = ['class', 'interface', 'for', 'else', 'if', 'while', 'do',
            'extends', 'implements', 'import', 'package', 'throws',
            'true', 'false', 'finally', 'static']

def capname(name):
    return capitalize(name[0])+name[1:]


class Property:
    def __init__(self, typeName, name, comment):
        self.typeName = typeName
        self.name = name
        self.comment = comment
        self.parseComment(comment)

    def parseComment(self, comment):
        self.afterName = None
        self.beforeName = None

        if comment is None or len(comment) == 0: return
        
        key = split(comment)[0]
        print self.name, repr(comment), repr(key)
        
        if key == '@after':
            self.afterName = split(comment)[1]
        elif key == '@before':
            self.beforeName = split(comment)[1]
        
    def getLegalName(self):
        if self.name in keywords:
            return '_'+self.name
        else:
            return self.name
        
    def makeField(self):
        return 'protected %s %s;' % (self.typeName, self.getLegalName())

    def getArgName(self):
        return '_'+self.getLegalName()

    def getGetterName(self):
        return 'get'+capname(self.name)

    def getSetterName(self):
        return 'set'+capname(self.name)


    def makeGetter(self):
        return 'public %s %s() { return %s; }' % (self.typeName, self.getGetterName(), self.getLegalName())

    def makeSetter(self):
        return 'public void %s(%s %s) { %s = %s; }' % (self.getSetterName(), self.typeName, self.getArgName(), self.getLegalName(), self.getArgName())

    def isChild(self):
        return 0

    def makeCopy(self):
        return self.getLegalName()

    def makeCopyWalk(self):
        return "ret.%(name)s = %(name)s;" % {'name':self.getLegalName()}

    def makeWalk(self):
        return ""

    def isOutOfOrder(self):
        return self.afterName is not None or self.beforeName is not None

    def findIndexOf(self, list, name):
        for i in range(len(list)):
            if list[i].name == name: return i
        raise "can't find " + name + " in parent"

    def addTo(self, list):
        if self.afterName is not None:
            list.insert(self.findIndexOf(list, self.afterName)+1, self)
        elif self.beforeName is not None:
            list.insert(self.findIndexOf(list, self.beforeName), self)
        else:
            list.append(self)


childSetter = """\
public void %(setterName)s(%(typeName)s %(argName)s) {
    if (%(argName)s != null) %(argName)s.setParent(this);
    %(fieldName)s = %(argName)s;
}"""


class Child(Property):
    def makeSetter(self):
        return childSetter % {'argName':self.getArgName(), 'typeName':self.typeName,
                 'setterName':self.getSetterName(), 'fieldName':self.getLegalName()}
    
    def isChild(self):
        return 1

    def makeCopy(self):
        # would be nice to be able to assume non-null here...
        return '(%s)copy(%s)' % (self.typeName, self.getLegalName())

    def makeCopyWalk(self):
        # would be nice to be able to assume non-null here...
        d = {'name':self.getLegalName(), 'typeName':self.typeName, 'setterName':self.getSetterName()} 
        return 'if (%(name)s != null) ret.%(setterName)s( (%(typeName)s)walker.process(%(name)s) );' % d

    def makeWalk(self):
        # would be nice to be able to assume non-null here...
        d = {'name':self.getLegalName(), 'typeName':self.typeName, 'setterName':self.getSetterName()} 
        return 'if (%(name)s != null) { ASTObject newChild = walker.process(%(name)s); if (newChild != %(name)s) { %(setterName)s( (%(typeName)s)newChild ); }}' % d


copyWalkMethod = """\
public ASTObject copyWalk(CopyWalker walker) {
    %(typeName)s ret = new %(typeName)s(getSourceLocation());
    ret.preCopy(walker, this);
%(body)s
    return ret;
}
"""

shallowCopyMethod = """\
public ASTObject shallowCopy() {
}
"""

newWalkMethod = """\
public void walk(Walker walker) {
%(body)s
}
"""

copyMethod = """\
public ASTObject copy() {
    return finishCopy(new %(typeName)s(%(args)s));
}
"""

walkMethod = """\
public void walk(ASTWalker walker) {
%s
}
"""

constructor = """\
public %s(%s) {
    super(%s);
%s
}
"""

emptyConstructor = """\
protected %s(SourceLocation source) {
    super(source);
}
"""

childGetterMethod = """\
public ASTObject getChildAt(int childIndex) {
    switch(childIndex) {
%s
    default: return super.getChildAt(childIndex);
    }
}
"""

childSetterMethod = """\
public void setChildAt(int childIndex, ASTObject child) {
    switch(childIndex) {
%s
    default: super.setChildAt(childIndex, child); return;
    }
}
"""


childNameGetterMethod = """\
public String getChildNameAt(int childIndex) {
    switch(childIndex) {
%s
    default: return super.getChildNameAt(childIndex);
    }
}
"""

childCountMethod = """\
public int getChildCount() {
    return %d;
}
"""

displayNameMethod = """\
public String getDefaultDisplayName() {
    return \"%s(%s)\";
}
"""

header = "//BEGIN: Generated from @child and @property"
footer = "//END: Generated from @child and @property"
class ASTNode:
    def __init__(self, typeName, superTypeName, properties, isAbstract):
        self.typeName = typeName
        self.properties = properties
        self.superTypeName = superTypeName
        self.isAbstract = isAbstract
        self.world = None
        self.cleanText = ""
        
    def getAllProperties(self):
        props = []
        if self.world is not None:
            #print self.world.nodes
            mySuperNode = self.world.nodes.get(self.superTypeName, None)
            if mySuperNode is not None:
                props = mySuperNode.getAllProperties()[:]
        for p in self.properties:
            p.addTo(props)
        return props

    def getMyProperties(self):
        for p in self.properties:
            if p.isOutOfOrder(): return self.getAllProperties()
        return self.properties
        
    def makeCopier(self):
        args = ["getSourceLocation()"] + map(lambda arg: arg.makeCopy(), self.getAllProperties())
        return copyMethod % {'typeName':self.typeName, 'args':join(args, ', ')}

    # would like to add some more efficient variants...    
    def makeNewWalker(self):
        props = []
        for prop in self.getAllProperties():
            #if not prop.isChild(): continue
            props.append("    " + prop.makeWalk())
        return newWalkMethod % {'typeName':self.typeName, 'body':join(props, '\n')}

    def makeCopyWalker(self):
        props = []
        for prop in self.getAllProperties():
            #if not prop.isChild(): continue
            props.append("    " + prop.makeCopyWalk())
        return copyWalkMethod % {'typeName':self.typeName, 'body':join(props, '\n')}

    def makeDisplayName(self):
        displayProps = []
        for prop in self.getAllProperties():
            if not prop.isChild():
                displayProps.append("%s: \"+%s+\"" % (prop.name, prop.getLegalName()))
                
        return displayNameMethod % (self.typeName, join(displayProps, ', "+\"'))
        

    def makeChildGetters(self):
        childGetters = []
        childSetters = []
        childNameGetters = []
        offset = 0
        for prop in self.getAllProperties():
            if prop in self.getMyProperties(): break
            if prop.isChild(): offset = offset + 1

        index = offset
        for prop in self.getMyProperties():
            if prop.isChild():
                childSetters.append("    case %d: %s((%s)child); return;" %
                                        (index, prop.getSetterName(), prop.typeName))
                childGetters.append("    case %d: return %s;" % (index, prop.getLegalName()))
                childNameGetters.append("    case %d: return \"%s\";" % (index, prop.name))
                index = index + 1

        if len(childSetters) == 0:
            return ""

        methods = [
            childGetterMethod % join(childGetters, '\n'),
            childNameGetterMethod % join(childNameGetters, '\n'),
            childSetterMethod % join(childSetters, '\n'),
            childCountMethod % index,
        ]
                      
        return join(methods)
    
    def makeConstructor(self):
        args = []
        props = []
        superargs = []
        args.append("SourceLocation location");
        superargs.append("location");
        for prop in self.getAllProperties():
            args.append('%s %s' % (prop.typeName, prop.getArgName()))

            if prop in self.properties: #getMyProperties():
                tmp = (prop.getSetterName(), prop.getArgName())
                props.append('    %s(%s);' % tmp)
            else:
                superargs.append(prop.getArgName())
                
        ret = constructor % (self.typeName, join(args, ', '),
                              join(superargs, ', '),
                              join(props, '\n'))

        if len(args) > 1:
            ret = ret + emptyConstructor % (self.typeName)

        return ret

    def makeGettersSetters(self):
        body = []
        for prop in self.properties: #getMyProperties():
            body.append(prop.makeField())
            body.append(prop.makeGetter())
            body.append(prop.makeSetter())
            body.append("")
        return join(body, '\n')

    def makeBody(self):
        body = ["", "", header]

        body.append(self.makeGettersSetters())
        body.append(self.makeConstructor())
        if not self.isAbstract:
            body.append(self.makeCopyWalker())
            #body.append(self.makeNewWalker())
        #body.append(self.makeWalker())
        body.append(self.makeChildGetters())
        body.append(self.makeDisplayName())
        #if not self.isAbstract:
        #    body.append(self.makeVisitor())
        body.append(footer)

        return indent("    ", join(body, '\n'))

    def makeNewBody(self):
        lastLine = rfind(self.cleanText, '}')
        lastLine = rfind(self.cleanText, '\n', 0, lastLine)
        return self.cleanText[:lastLine] + self.makeBody() + self.cleanText[lastLine:]


copyWalkMethod1 = """\
public ASTObject copyWalk(CopyWalker walker) {
    final int N = size;
    %(childTypeName)s[] copiedChildren = new %(childTypeName)s[N];
    int newIndex = 0;
    for(int oldIndex=0; oldIndex<N; oldIndex++) {
        %(childTypeName)s newChild = (%(childTypeName)s)walker.process(children[oldIndex]);
        if (newChild != null) copiedChildren[newIndex++] = newChild;
    }
    %(typeName)s ret = new %(typeName)s(getSourceLocation(),copiedChildren);
    ret.size = newIndex;
    ret.setSource(this);
    return ret;
}
"""


newWalkMethod1 = """\
public void walk(Walker walker) {
    for (int index = 0; index < size(); index++) {
        %(childTypeName)s child = get(index);
        %(childTypeName)s newChild = (%(childTypeName)s)walker.process(child);
        if (child == newChild) continue;
        if (newChild == null) remove(index);
        else set(index, newChild);
    }
}
"""

copyMethod1 = """public ASTObject copy() {
    final int N = size;
    %(childTypeName)s[] copiedChildren = new %(childTypeName)s[N];
    for(int i=0; i<N; i++) {
        copiedChildren[i] = (%(childTypeName)s)children[i].copy();
    }
    return finishCopy(new %(typeName)s(getSourceLocation(),copiedChildren));
}
"""        

walkMethod1 = """\
public void walk(ASTWalker walker) {
    final int N = size;
    for(int i=0; i<N; i++) {
        set(i, (%(childTypeName)s)walker.walk(children[i]));
    }
}
"""

constructor1 = """\
public %(typeName)s(SourceLocation location, %(childTypeName)s[] _children) {
	super(location);
    for(int i=0; i<_children.length; i++) {
        if (_children[i] != null) _children[i].setParent(this);
    }
    children = _children;
    size = _children.length;
}

public %(typeName)s(SourceLocation location) {
    this(location, new %(childTypeName)s[] {});
}

public %(typeName)s(SourceLocation location, %(childTypeName)s child1) {
    this(location, new %(childTypeName)s[] {child1});
}

public %(typeName)s(SourceLocation location, %(childTypeName)s child1, %(childTypeName)s child2) {
    this(location, new %(childTypeName)s[] {child1, child2});
}

public %(typeName)s(SourceLocation location, %(childTypeName)s child1, %(childTypeName)s child2, %(childTypeName)s child3) {
    this(location, new %(childTypeName)s[] {child1, child2, child3});
}
"""



childGetterMethods1 = """\
public ASTObject getChildAt(int childIndex) { return get(childIndex); }
public void setChildAt(int childIndex, ASTObject child) { set(childIndex, (%(typeName)s)child); }
public String getChildNameAt(int childIndex) { return \"%(childName)s\"+childIndex; }

public int getChildCount() { return size; }
public int size() { return size; }

public %(typeName)s get(int index) {
    if (index >= size) throw new ArrayIndexOutOfBoundsException();
    return children[index];
}

public void set(int index, %(typeName)s child) {
    if (index >= size) throw new ArrayIndexOutOfBoundsException();
    children[index] = child;
    child.setParent(this);
}

public void resize(int newSize) {
    if (newSize > children.length) {
        %(typeName)s[] newChildren = new %(typeName)s[children.length*2 + 1];
        System.arraycopy(children, 0, newChildren, 0, children.length);
        children = newChildren;
    }
    size = newSize;
}

public void addAll(%(myTypeName)s collection) {
    addAll(size, collection);
}

public void addAll(int index, %(myTypeName)s collection) {
    for(int i=0; i<collection.size(); i++) {
        add(index+i, collection.get(i));
    }
}

public void add(%(typeName)s child) {
    add(size, child);
}

public void add(int index, %(typeName)s child) {
    if (child == null) return;

    if (index < 0 || index > size) throw new ArrayIndexOutOfBoundsException();

    resize(size+1);
    
    for(int moveIndex = size-1; moveIndex > index; moveIndex--) {
        children[moveIndex] = children[moveIndex-1];
    }

    children[index] = child;
    child.setParent(this);
}

public void remove(int index) {
    if (index < 0 || index > size) throw new ArrayIndexOutOfBoundsException();

    size -= 1;
    
    for(int moveIndex = index; moveIndex < size; moveIndex++) {
        children[moveIndex] = children[moveIndex+1];
    }
}
"""





class MultiASTNode(ASTNode):
    def __init__(self, typeName, superTypeName, childTypeName, childName):
        self.typeName = typeName
        self.childName = childName
        self.childTypeName = childTypeName
        self.superTypeName = superTypeName
        self.world = None
        self.cleanText = ""
        self.isAbstract = 0
        
    def makeCopier(self):
        return copyMethod1 % {'typeName':self.typeName, 'childTypeName':self.childTypeName}

    # would like to add some more efficient variants...    
    def makeCopyWalker(self):
        return copyWalkMethod1 % {'typeName':self.typeName, 'childTypeName':self.childTypeName}
    def makeNewWalker(self):
        return newWalkMethod1 % {'typeName':self.typeName, 'childTypeName':self.childTypeName}

    def makeWalker(self):
        return walkMethod1 % {'typeName':self.typeName, 'childTypeName':self.childTypeName}

    def makeDisplayName(self):
        return displayNameMethod % (self.typeName, '')

    def makeChildGetters(self):
        return childGetterMethods1 % {'childName': self.childName,
                                      'typeName':self.childTypeName,
                                      'myTypeName':self.typeName}

    def makeConstructor(self):
        return constructor1 % {'typeName':self.typeName, 'childTypeName':self.childTypeName}

    def makeGettersSetters(self):
        return "protected int size;\npublic %s[] children;\n" % self.childTypeName

def indent(tab, text):
    return join(map(lambda x, i=tab: i+x, split(text, '\n')), '\n')

childrenPattern = r"@children\s+(\w+)\s+(\w+)\s+([\w\s]*)"
propertyPattern = r"@(child|property)\s+(\w+)\s+(\w+)\s+([\w\s@]*)"
classPattern = r"(\w*)public\s+(abstract\s+)?class\W+(\w+)\W+extends\W+(\w+)"

import re
def makeASTNode(text):
    classMatch = re.search(classPattern, text)
    if classMatch is None: return None

    isAbstract = classMatch.group(2)
    typeName = classMatch.group(3)
    superTypeName = classMatch.group(4)
    #print isAbstract, typeName, superTypeName

    childrenMatch = re.search(childrenPattern, text)
    if childrenMatch is not None:
        childrenType, childrenName, childrenComment = childrenMatch.groups()
        return MultiASTNode(typeName, superTypeName, childrenType, childrenName)

    props = []
    for kind, propType, propName, propComment in re.findall(propertyPattern, text):
        if kind == 'child':
            props.append( Child(propType, propName, propComment) )
        else:
            props.append( Property(propType, propName, propComment) )

    #print props

    return ASTNode(typeName, superTypeName, props, isAbstract is not None)

def cleanText(text):
    start = re.search('\s+'+re.escape(header), text, re.MULTILINE)
    if start is None: return text
    start = start.start(0)
    
    end = find(text, footer)
    if end != -1:
        #start = rfind(text, '\n', 0, start)+1
        end = find(text, '\n', end)+1
        return text[:start]+'\n'+text[end:]
    else:
        return text

import os

class NodeWorld:
    def __init__(self):
        self.nodes = {}

    def addDirectory(self, directory):
        for filename in os.listdir(directory):
            if not filename[-5:] == '.java': continue
            
            fullname = os.path.join(directory, filename)
            if not os.path.isfile(fullname): continue
            
            fp = open(fullname)
            text = fp.read()
            fp.close()

            if find(text, '@grammar') == -1: continue

            #print fullname
            newText = cleanText(text)
            node = makeASTNode(newText)
            node.world = self
            node.filename = fullname
            node.cleanText = newText
            node.fullText = text
            self.nodes[node.typeName] = node


"""
visitorMethod = "    public void visit(%s node);"

visitorMethods = []
def addToVisitor(node):
    if find(node.filename, 'base/ast') != -1:
        visitorMethods.append(visitorMethod % node.typeName)
"""

aspectjroot = "c:/aspectj/modules/compiler"

# We should only pay for this large comparison on the files that this
# script previously would think changed.  I.e., the test used to be a
# == b.  There's almost certainly a way of writing this that would be
# faster.
from string import whitespace
def isSameModuloWhitespace(a, b):
    if a == b:
        return 1
    ai = len(a) - 1;
    bi = len(b) - 1;
    while (1):
        if ai < 0:
            while bi >= 0 and b[bi] in whitespace:
                bi = bi - 1
            return bi < 0
        elif bi < 0:
            a, ai, b, bi = b, bi, a, ai      # flip
        elif a[ai] in whitespace:
            ai = ai - 1
        elif b[bi] in whitespace:
            a, ai, b, bi = b, bi - 1, a, ai  # flip
        elif a[ai] != b[bi]:
            return 0
        else:
            ai, bi = ai - 1, bi - 1

def test():
    world = NodeWorld()
    world.addDirectory(aspectjroot + '/src/org/aspectj/compiler/base/ast')
    world.addDirectory(aspectjroot + '/src/org/aspectj/compiler/base/cst')
    world.addDirectory(aspectjroot + '/src/org/aspectj/compiler/crosscuts/ast')
    for node in world.nodes.values():
        #print node.typeName, node.superTypeName, node, node.getAllProperties()

        newText = node.makeNewBody()
        
        if isSameModuloWhitespace(newText, node.fullText):
            #print '    unchanged'
            continue
        else:
            print 'modified', node.typeName, 'extends', node.superTypeName
        
        fp = open(node.filename, 'w')
        fp.write(newText)
        fp.close()

    #visitorMethods.sort()
    #print join(visitorMethods, '\n')

if __name__ == '__main__':
    if len(sys.argv) > 1:
        aspectjroot = sys.argv[1]
    test()
